Package: rtpcr
Type: Package
Title: qPCR Data Analysis
Version: 2.1.8
Authors@R: c(person("Ghader", "Mirzaghaderi", role=c("aut", "cre", "cph"), email = "mirzaghaderi@gmail.com"))
Description: Tools for qPCR data analysis using Delta Ct and Delta Delta Ct methods, including t-test, Wilcoxon-test, ANOVA models, and publication-ready visualizations. The package supports multiple target, and multiple reference genes, and uses a calculation framework adopted from Ganger et al. (2017) <doi:10.1186/s12859-017-1949-5> and Taylor et al. (2019) <doi:10.1016/j.tibtech.2018.12.002>, covering both the Livak and Pfaffl methods.
URL: https://mirzaghaderi.github.io/rtpcr/,
        https://github.com/mirzaghaderi/rtpcr
License: GPL-3
Imports: multcomp, ggplot2, lme4, lmerTest, purrr, reshape2, tidyr,
        dplyr, grid, emmeans, lifecycle
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-18 19:05:58 UTC; GM
Author: Ghader Mirzaghaderi [aut, cre, cph]
Depends: R (>= 3.5.0)
Suggests: knitr, rmarkdown, multcompView
VignetteBuilder: knitr
LazyData: true
Maintainer: Ghader Mirzaghaderi <mirzaghaderi@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 09:20:02 UTC
Built: R 4.6.0; ; 2026-05-25 04:26:52 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ANOVA_DCt)
export(ANOVA_DDCt)
export(Means_DDCt)
export(TTEST_DDCt)
export(WILCOX_DDCt)
export(compute_wDCt)
export(efficiency)
export(long_to_wide)
export(meanTech)
export(multiplot)
export(plotFactor)
import(dplyr)
import(emmeans)
import(ggplot2)
import(grid)
import(purrr)
import(reshape2)
import(tidyr)
importFrom(stats,setNames)

## ----setup, include = FALSE, fig.align='center', warning = F, message=F-------
options(tinytex.verbose = TRUE)
knitr::opts_chunk$set(echo = TRUE)

## ----eval= T, include= F, message=FALSE, warning = FALSE----------------------
library(rtpcr)
library(multcomp)
library(dplyr)
library(reshape2)
library(tidyr)
library(ggplot2)
library(grid)

## ----eval= F, include= T, message=FALSE, warning = FALSE----------------------
# 
# # install `rtpcr` from github (under development)
# 
# devtools::install_github("mirzaghaderi/rtpcr")
# 
# # I strongly recommend to install the package with the vignette as it contains information about how to use the 'rtpcr' package. Through the following code, Vignette is installed as well.
# 
# devtools::install_github("mirzaghaderi/rtpcr", build_vignettes = TRUE)

## ----eval= T------------------------------------------------------------------
data_efficiency

## ----eval = T, , fig.height = 3, fig.width = 5, fig.align = 'center', fig.cap = 'Standard curve and the amplification efficiency analysis of genes. Required iput data include dilutions and Ct value columns for different genes.', warning = FALSE, message = FALSE----
efficiency(data_efficiency)

## ----eval= T, fig.height = 3, fig.width = 5, fig.align = 'center'-------------
data_ttest

## ----eval= T------------------------------------------------------------------
qpcrTTEST(data_ttest, 
          numberOfrefGenes = 1,
          paired = F, 
          var.equal = T,
          p.adj = "none")

## ----eval= T, fig.height=3, fig.width=8, fig.align='center', fig.cap = "Average Fold changes of three target genes relative to the control condition computed by unpaired t-tests via `qpcrTTESTplot` function. Confidence interval (ci) and standard error (se) has been used as error bar in 'A' and 'B', respectively.", warning = F, message = F----

# Producing the plot
t1 <- qpcrTTESTplot(data_ttest,
              numberOfrefGenes = 1,
              fontsizePvalue = 4,
              errorbar = "ci",
              p.adj = "none")

# Producing the plot: specifying gene order
t2 <- qpcrTTESTplot(data_ttest,
              numberOfrefGenes = 1,
              order = c("C2H2-01", "C2H2-12", "C2H2-26"),
              paired = FALSE,
              var.equal = TRUE,
              width = 0.5,
              fill = "palegreen",
              y.axis.adjust = 3,
              y.axis.by = 2,
              ylab = "Average Fold Change",
              xlab = "none",
              fontsizePvalue = 4)

multiplot(t1, t2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))

## ----eval = T, fig.height = 3, fig.width = 5, fig.align='center', fig.cap = "Statistical table and figure of the Fold change expression of a gene in three different levels of Drough stress relative to the D0 as reference or calibrator level produced by the `qpcrANOVAFC` function. The other factor i.e. Genotype has been concidered as covariate."----
# See sample data
data_2factor

qpcrANOVAFC(data_2factor, 
           numberOfrefGenes = 1, 
           block = NULL,
           analysisType = "ancova",
           mainFactor.column = 2,
           fontsizePvalue = 4,
           x.axis.labels.rename = "none")

## ----eval= T------------------------------------------------------------------
# See a sample dataset
data_3factor

## ----eval= T, fig.height = 3, fig.width = 5-----------------------------------
# If the data include technical replicates, means of technical replicates
# should be calculated first using meanTech function.

# Applying ANOVA analysis
res <- qpcrANOVARE(data_2factor,
                   numberOfrefGenes = 1,
                   block = NULL)
res$Result
res$Post_hoc_Test

## ----eval= T, fig.height = 4, fig.width = 9, fig.align = 'center', fig.cap = "A: bar plot representing Relative expression of a gene under three levels of a factor generated using `oneFACTORplot` function, B: Plot of the Fold change expression produced by the `qpcrANOVAFC` function from the same data used for 'A'. The first element in the `mainFactor.level.order` argument (here L1) is served as the Reference level, although the x-axis names have later been renamed by the `x.axis.labels.rename` argument. Error bars represent 95% confidence interval in A and standard error in B."----

# Before plotting, the result needs to be extracted as below:
out2 <- qpcrANOVARE(data_1factor, numberOfrefGenes = 1, block = NULL)$Result

f1 <- oneFACTORplot(out2,
              width = 0.2,
              fill = "skyblue",
              y.axis.adjust = 0.5,
              y.axis.by = 1,
              errorbar = "ci",
              show.letters = TRUE,
              letter.position.adjust = 0.1,
              ylab = "Relative Expression",
              xlab = "Factor Levels",
              fontsize = 12,
              fontsizePvalue = 4)


addline_format <- function(x,...){
    gsub('\\s','\n',x)
}
f2 <- qpcrANOVAFC(data_1factor,
                 numberOfrefGenes = 1,
                 mainFactor.column = 1,
                 block = NULL,
                 mainFactor.level.order = c("L1","L2","L3"),
                 width = 0.5,
                 fill = c("skyblue","#79CDCD"),
                 y.axis.by = 1,
                 letter.position.adjust = 0,
                 y.axis.adjust = 1,
                 ylab = "Fold Change",
                 fontsize = 12, plot = F,
                 x.axis.labels.rename = addline_format(c("Control", 
                                                       "Treatment_1 vs Control", 
                                                       "Treatment_2 vs Control")))


multiplot(f1, f2$FC_Plot_of_the_main_factor_levels, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))

## ----eval= T, include = T, fig.height = 4, fig.width = 9, fig.align = 'center', fig.cap = "Relative expression of a target gene under two different factors of genotype (with two levels) and drought (with three levels). Error bars represent standard error. Means (columns) lacking letters in common have significant difference at alpha = 0.05 as resulted from a `LSD.test`."----

# Before plotting, the result needs to be extracted as below:
res <- qpcrANOVARE(data_2factor, numberOfrefGenes = 1, block = NULL)$Result
Final_data <- qpcrANOVARE(data_2factor, numberOfrefGenes = 1, block = NULL)$Final_data

# Plot of the 'res' data with 'Genotype' as grouping factor
q1 <- twoFACTORplot(res,
   x.axis.factor = Drought,
   group.factor = Genotype,
   errorbar = "se",
   width = 0.5,
   fill = "Greens",
   y.axis.adjust = 0.5,
   y.axis.by = 2,
   ylab = "Relative Expression",
   xlab = "Drought Levels",
   legend.position = c(0.15, 0.8),
   show.letters = TRUE,
   fontsizePvalue = 4)

# Plotting the same data with 'Drought' as grouping factor
q2 <- twoFACTORplot(res,
   x.axis.factor = Genotype,
   group.factor = Drought,
   errorbar = "se",
   xlab = "Genotype",
   fill = "Blues",
   legend.position = c(0.15, 0.8),
   show.letters = FALSE,
   show.errorbars = T,
   fontsizePvalue = 4)

multiplot(q1, q2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))

## ----fig.height = 5, fig.width = 11, fig.align = 'center', fig.cap = "A and B) Relative expression (RE) of a target gene from a three-factorial experiment data produced by  `threeFACTORplot`  function. Error bars represent standard error (A), although can be set to confidence interval (B). Means (columns) lacking letters in common have significant differences at alpha = 0.05 as resulted from an ‘LSD.test’."----
# Before plotting, the result needs to be extracted as below:
res <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Result
res

# releveling a factor levels first
res$Conc <- factor(res$Conc, levels = c("L","M","H"))
res$Type <- factor(res$Type, levels = c("S","R"))

# Arrange the first three colunms of the result table.
# This determines the columns order and shapes the plot output.
p1 <- threeFACTORplot(res,
    arrangement = c(3, 1, 2),
    errorbar = "se",
    legend.position = c(0.2, 0.85),
    xlab = "condition",
    fontsizePvalue = 4)


# When using ci as error, increase y.axis.adjust to see the plot correctly!
p2 <- threeFACTORplot(res,
   arrangement = c(2, 3, 1),
   bar.width = 0.8,
   fill = "Greens",
   xlab = "Drought",
   ylab = "Relative Expression",
   errorbar = "ci",
   y.axis.adjust = 2,
   y.axis.by = 2,
   letter.position.adjust = 0.6,
   legend.title = "Genotype",
   fontsize = 12,
   legend.position = c(0.2, 0.8),
   show.letters = TRUE,
   fontsizePvalue = 4)

multiplot(p1, p2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))

## ----eval=T, fig.height = 4, fig.width = 7, fig.align = 'center', fig.cap = "Fold change expression (FC) of a target gene from a one and a two factorial experiment data produced by  `qpcrREPEATED`  function. Error bars represent standard error (A), although can be set to confidence interval."----

a <- qpcrREPEATED(data_repeated_measure_1,
                  numberOfrefGenes = 1,
                  block = NULL,
                  fill = c("#778899", "#BCD2EE"),
                  factor = "time",
                  axis.text.x.angle = 45,
                  axis.text.x.hjust = 1,
                  plot = F)

b <- qpcrREPEATED(data_repeated_measure_2,
                  numberOfrefGenes = 1,
                  factor = "time",
                  block = NULL,
                  axis.text.x.angle = 45,
                  axis.text.x.hjust = 1,
                  plot = F)

multiplot(a, b, cols = 2)

## ----eval=T-------------------------------------------------------------------
# Returning fold change values from a fitted model.
# Firstly, result of `qpcrANOVAFC` or `qpcrREPEATED` is 
# acquired which includes a model object:
res <- qpcrANOVAFC(data_3factor, numberOfrefGenes = 1, mainFactor.column = 1, block = NULL)

# Returning fold change values of Conc levels from a fitted model:
qpcrMeans(res$lm_ANOVA, specs = "Conc")


# Returning fold change values of Conc levels sliced by Type*SA:
qpcrMeans(res$lm_ANOVA, specs = "Conc | (Type*SA)")

# Returning fold change values of Conc
qpcrMeans(res$lm_ANOVA, specs = "Conc * Type")

# Returning fold change values of Conc levels sliced by Type:
res2 <- qpcrMeans(res$lm_ANOVA, specs = "Conc | Type")
twoFACTORplot(res2, x.axis.factor = contrast, ylab = "Fold Change",
              group.factor = Type, errorbar = "ci")

## ----eval=T, include = T, fig.height = 4, fig.width = 6, fig.align = 'center'----

library(ggplot2)
b <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Result
a <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Final_data

# Arrange factor levels to your desired order:
b$Conc <- factor(b$Conc, levels = c("L","M","H"))
a$Conc <- factor(a$Conc, levels = c("L","M","H"))

# Generating plot
ggplot(b, aes(x = Type, y = RE, fill = factor(Conc))) +
  geom_bar(stat = "identity", position = "dodge") +
  facet_wrap(~ SA) +
  scale_fill_brewer(palette = "Reds") +
  xlab("Type") +
  ylab("Relative Expression") +
  geom_point(data = a, aes(x = Type, y = (2^(-wDCt)), fill = factor(Conc)), 
             position = position_dodge(width = 0.9), color = "black") +
  ylab("ylab") +
  xlab("xlab") +
  theme_bw() +
  theme(axis.text.x = element_text(size = 12, color = "black", angle = 0, hjust = 0.5),
        axis.text.y = element_text(size = 12, color = "black", angle = 0, hjust = 0.5),
        axis.title  = element_text(size = 12),
        legend.text = element_text(size = 12)) +
  theme(legend.position  = c(0.2, 0.7)) +
  theme(legend.title = element_text(size = 12, color = "black")) +
  scale_y_continuous(breaks = seq(0, max(b$RE) + max(b$se) + 0.1, by = 5), 
                     limits = c(0, max(b$RE) + max(b$se) + 0.1), expand = c(0, 0)) 

## ----eval= T, eval= T, fig.height = 5, fig.width = 10, fig.align = 'center', fig.cap = "QQ-plot for the normality assessment of the residuals derived from `t.test` or `lm` functions."----

residuals <- qpcrANOVARE(data_1factor, numberOfrefGenes = 1, block = NULL)$lmCRD$residuals
shapiro.test(residuals) 

par(mfrow = c(1,2))
plot(residuals)
qqnorm(residuals)
qqline(residuals, col = "red")

## ----eval= T, eval= T, fig.height = 4, fig.width = 4, fig.align = 'center', fig.cap = "QQ-plot for the normality assessment of the residuals derived from `t.test` or `lm` functions."----

a <- qpcrREPEATED(data_repeated_measure_2, 
                  numberOfrefGenes = 1, 
                  factor = "time", 
                  block = NULL,
                  y.axis.adjust = 1.5)


residuals(a$lm)
plot(residuals(a$lm))
qqnorm(residuals(a$lm))
qqline(residuals(a$lm), col = "red")

## ----eval= T------------------------------------------------------------------
# See example input data frame:
data_withTechRep

# Calculating mean of technical replicates
meanTech(data_withTechRep, groups = 1:4)

## ----eval= T, eval= T, , fig.height = 4, fig.width = 5, fig.align = 'center', fig.cap = "Fold change expression of two different genes. FC tables of any number of genes can be combined and used as input data frame for `twoFACTORplot` function."----

a <- qpcrREPEATED(data_repeated_measure_1,
             numberOfrefGenes = 1,
             factor = "time", block = NULL)

b <- qpcrREPEATED(data_repeated_measure_2,
                  factor = "time",
                  numberOfrefGenes = 1, block = NULL)


a1 <- a$FC_statistics_of_the_main_factor
b1 <- b$FC_statistics_of_the_main_factor

c <- rbind(a1, b1)
c$gene <- factor(c(1,1,1,2,2,2))
c

twoFACTORplot(c, x.axis.factor = contrast, 
              group.factor = gene, fill = 'Reds', errorbar = "se",
              ylab = "FC", axis.text.x.angle = 45, y.axis.adjust = 1.5,
              axis.text.x.hjust = 1, legend.position = c(0.2, 0.8))

## ----eval= F, include = T, fig.height = 4, fig.width = 5----------------------
# 
# b <- qpcrANOVAFC(data_2factor,
#             numberOfrefGenes = 1,
#             mainFactor.column = 1,
#             block = NULL,
#             mainFactor.level.order = c("S", "R"),
#             fill = c("#CDC673", "#EEDD82"),
#             analysisType = "ancova",
#             fontsizePvalue = 7,
#             y.axis.adjust = 0.1, width = 0.35)
# 
# 
# 
# library(ggplot2)
# p2 <- b$FC_Plot_of_the_main_factor_levels
# p2 + theme_set(theme_classic(base_size = 20))
# 

## ----eval= F------------------------------------------------------------------
# citation("rtpcr")

## ----setup, include = FALSE, fig.align='center', warning = F, message=F-------
options(tinytex.verbose = TRUE)
knitr::opts_chunk$set(echo = TRUE)
library(rtpcr)

## ----eval= F------------------------------------------------------------------
# # install.packages("rtpcr")
# # install.packages("shiny")
# library(shiny)
# library(rtpcr)
# # Run the following code in Rstudio
# runApp(system.file("shinyapp/app.R", package = "rtpcr"))

## ----eval = F-----------------------------------------------------------------
# # Installing from CRAN
# install.packages("rtpcr")
# 
# # Loading the package
# library(rtpcr)

## ----eval = F-----------------------------------------------------------------
# devtools::install_github("mirzaghaderi/rtpcr", build_vignettes = TRUE)

## ----eval= F------------------------------------------------------------------
# # Applying the efficiency function
# data <- read.csv(system.file("extdata", "data_efficiency.csv", package = "rtpcr"))
# data
# # dilutions	Gene1	Gene2	Gene3
# # 1.00	25.58	24.25	22.61
# # 1.00	25.54	24.13	22.68
# # 1.00	25.50	24.04	22.63
# # 0.50	26.71	25.56	23.67
# # 0.50	26.73	25.43	23.65
# # 0.50	26.87	26.01	23.70
# # 0.20	28.17	27.37	25.11
# # 0.20	28.07	26.94	25.12
# # 0.20	28.11	27.14	25.11
# # 0.10	29.20	28.05	26.17
# # 0.10	29.49	28.89	26.15
# # 0.10	29.07	28.32	26.15
# # 0.05	30.17	29.50	27.12
# # 0.05	30.14	29.93	27.14
# # 0.05	30.12	29.71	27.16
# # 0.02	31.35	30.69	28.52
# # 0.02	31.35	30.54	28.57
# # 0.02	31.35	30.04	28.53
# # 0.01	32.55	31.12	29.49
# # 0.01	32.45	31.29	29.48
# # 0.01	32.28	31.15	29.26
# 
# # Analysis
# efficiency(data)
# 
# # $Efficiency
# #    Gene     Slope        R2        E
# # 1 Gene1 -3.388094 0.9965504 1.973110
# # 2 Gene2 -3.528125 0.9713914 1.920599
# # 3 Gene3 -3.414551 0.9990278 1.962747
# #
# # $Slope_compare
# # $contrasts
# #  contrast          estimate    SE df t.ratio p.value
# #  C2H2.26 - C2H2.01   0.1400 0.121 57   1.157  0.4837
# #  C2H2.26 - GAPDH     0.0265 0.121 57   0.219  0.9740
# #  C2H2.01 - GAPDH    -0.1136 0.121 57  -0.938  0.6186

## ----eval= F------------------------------------------------------------------
# data <- read.csv(system.file("extdata", "data_Yuan2006PMCBioinf.csv", package = "rtpcr"))
# 
# # Anova analysis
# ANOVA_DDCt(
#   data,
#   specs = "condition",
#   numOfFactors = 1,
#   numberOfrefGenes = 1,
#   block = NULL)
# 
# # An example of a properly arranged dataset from a repeated-measures experiment.
# data <- read.csv(system.file("extdata", "data_repeated_measure_1.csv", package = "rtpcr"))
# 
# # time	id  E_Target	Ct_target   E_Ref      Ct_Ref
# #    1	 1	   	2	    18.92	2	32.77
# #    1	 2	   	2	    15.82	2	32.45
# #    1	 3	   	2	    19.84	2	31.62
# #    2	 1	  	2	    19.46	2	33.03
# #    2	 2	   	2	    17.56	2	33.24
# #    2	 3	   	2	    19.74	2	32.08
# #    3	 1	  	2	    15.73	2	32.95
# #    3	 2	  	2	    17.21	2	33.64
# #    3	 3	   	2	    18.09	2	33.40
# 
# # Repeated measure analysis
# res <- ANOVA_DDCt(
#   data,
#   numOfFactors = 1,
#   numberOfrefGenes = 1,
#   specs = "time",
#   block = NULL, model = wDCt ~ time + (1 | id))
# 
# 
# # Paired t.test (equivalent to repeated measure analysis, but not
# # always the same results, due to different calculation methods!)
# TTEST_DDCt(
#   data[1:6,],
#   numberOfrefGenes = 1,
#   paired = T)
# 
# 
# # Anova analysis
# data3 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
# 
# res <- ANOVA_DDCt(
#   x = data3,
#   specs = "Type | Concentration",
#   numOfFactors = 2,
#   numberOfrefGenes = 3,
#   block = "block",
#   analyseAllTarget = TRUE)

## ----eval= F------------------------------------------------------------------
# # Relative expression table for the specified column in the input data:
# data3 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
# 
# res <- ANOVA_DDCt(
#   x = data3,
#   specs = "Concentration",
#   numOfFactors = 2,
#   numberOfrefGenes = 3,
#   block = "block",
#   analyseAllTarget = TRUE)
# 
# # Relative Expression
# # gene contrast    ddCt     RE  log2FC    LCL    UCL      se L.se.RE U.se.RE L.se.log2FC U.se.log2FC  pvalue sig
# #   PO       L1  0.0000 1.0000  0.0000 0.0000 0.0000 0.13940 0.90790 1.10144     0.00000     0.00000 1.00000
# #   PO L2 vs L1 -0.9461 1.9266  0.9461 1.2586 2.9493 0.14499 1.74245 2.13036     0.85564     1.04613 0.00116  **
# #   PO L3 vs L1 -2.1919 4.5693  2.1919 3.0806 6.7772 0.29402 3.72685 5.60221     1.78783     2.68748 0.00000 ***
# #  NLM       L1  0.0000 1.0000  0.0000 0.0000 0.0000 0.91809 0.52921 1.88962     0.00000     0.00000 1.00000
# #  NLM L2 vs L1  0.8656 0.5487 -0.8656 0.3983 0.7561 0.36616 0.42577 0.70734    -1.11579    -0.67163 0.00018 ***
# #  NLM L3 vs L1 -1.4434 2.7196  1.4434 1.9467 3.7994 0.17132 2.41511 3.06256     1.28179     1.62542 0.00000 ***
# #
# # The L1 level was used as calibrator.
# # Note: Using default model for statistical analysis: wDCt ~ block + Concentration * Type
# 
# 
# ANOVA_table <- res$perGene$PO$ANOVA_table
# ANOVA_table
# 
# lm <- res$perGene$PO$lm
# lm
# 
# lm_formula <- res$perGene$gene_name$lm_formula
# lm_formula
# 
# residuals <- resid(res$perGene$gene_name$lm)
# residuals

## ----eval= F, warning = F, fig.height = 7, fig.width = 12.5, fig.align = 'center', warning = F----
# data <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))
# #Perform analysis first
# res <- ANOVA_DCt(
#   data,
#   numOfFactors = 3,
#   numberOfrefGenes = 1,
#   block = NULL)
# 
# df <- res$relativeExpression
# df
#  # Generate three-factor bar plot
# plotFactor(
#   df,
#   x_col = "SA",
#   y_col = "log2FC",
#   group_col = "Type",
#   facet_col = "Conc",
#   Lower.se_col = "Lower.se.log2FC",
#   Upper.se_col = "Upper.se.log2FC",
#   letters_col = "sig",
#   letters_d = 0.3,
#   col_width = 0.7,
#   dodge_width = 0.7,
#   fill_colors = c("palegreen3", "skyblue"),
#   color = "black",
#   base_size = 14,
#   alpha = 1,
#   legend_position = c(0.1, 0.2))

## ----eval= F, fig.height = 7, fig.width = 12.5, fig.align = 'center', warning = F----
# data <- read.csv(system.file("extdata", "data_2factorBlock.csv", package = "rtpcr"))
# res <- ANOVA_DCt(data,
#       numOfFactors = 2,
#       block = "block",
#       numberOfrefGenes = 1)
# 
# df <- res$relativeExpression
# 
# plotFactor(
#   data = df,
#   x_col = "Concentration",
#   y_col = "RE",
#   group_col = "Type",
#   Lower.se_col = "Lower.se.RE",
#   Upper.se_col = "Upper.se.RE",
#   letters_col = "sig",
#   letters_d = 0.2,
#   fill_colors = c("aquamarine4", "gold2"),
#   color = "black",
#   alpha = 1,
#   col_width = 0.7,
#   dodge_width = 0.7,
#   base_size = 16,
#   legend_position = c(0.2, 0.8))

## ----eval= F, warning = F-----------------------------------------------------
# # Using data from Heffer et al., 2020, PlosOne
# library(dplyr)
# 
# res <- ANOVA_DDCt(
#   data_Heffer2020PlosOne,
#   numOfFactors = 1,
#   specs = "Treatment",
#   numberOfrefGenes = 1,
#   block = NULL)
# 
# data <- res$relativeExpression
# 
# # Selecting only the first words in 'contrast' column to be used as the x-axis labels.
# data$contrast <- sub(" .*", "", data$contrast)
# 
# plotFactor(
#   data = data,
#   x_col = "contrast",
#   y_col = "RE",
#   group_col = "contrast",
#   facet_col = "gene",
#   Lower.se_col = "Lower.se.RE",
#   Upper.se_col = "Upper.se.RE",
#   letters_col = "sig",
#   letters_d = 0.2,
#   alpha = 1,
#   fill_colors = palette.colors(4, recycle = TRUE),
#   color = "black",
#   col_width = 0.5,
#   dodge_width = 0.5,
#   base_size = 16,
#   legend_position = "none")

## ----eval= F------------------------------------------------------------------
# res <- ANOVA_DDCt(
#   data_3factor,
#   numOfFactors = 3,
#   numberOfrefGenes = 1,
#   specs = "Conc",
#   block = NULL)
# 
# model <- res$perGene$E_PO$lm
# # Relative expression values for Concentration main effect
# Means_DDCt(model, specs = "Conc")
# 
# # contrast        RE        SE df       LCL       UCL p.value sig
# # L vs H   0.1703610 0.2208988 24 0.1242014 0.2336757 <0.0001 ***
# # M vs H   0.2227247 0.2208988 24 0.1623772 0.3055004 <0.0001 ***
# # M vs L   1.3073692 0.2208988 24 0.9531359 1.7932535  0.0928 .
# #
# #Results are averaged over the levels of: Type, SA
# #Confidence level used: 0.95
# 
# # Relative expression values for Concentration sliced by Type
# Means_DDCt(model, specs = "Conc | Type")
# 
# # Type = R:
# #  contrast       RE        SE df       LCL      UCL p.value sig
# #  L vs H   0.103187 0.3123981 24 0.0659984 0.161331 <0.0001 ***
# #  M vs H   0.339151 0.3123981 24 0.2169210 0.530255 <0.0001 ***
# #  M vs L   3.286761 0.3123981 24 2.1022126 5.138776 <0.0001 ***
# #
# # Type = S:
# #  contrast       RE        SE df       LCL      UCL p.value sig
# #  L vs H   0.281265 0.3123981 24 0.1798969 0.439751 <0.0001 ***
# #  M vs H   0.146266 0.3123981 24 0.0935518 0.228684 <0.0001 ***
# #  M vs L   0.520030 0.3123981 24 0.3326112 0.813055  0.0059 **
# #
# # Results are averaged over the levels of: SA
# # Confidence level used: 0.95
# 
# # Relative expression values for Concentration sliced by Type and SA
# Means_DDCt(model, specs = "Conc | Type * SA")

## ----eval= F------------------------------------------------------------------
# data <- read.csv(system.file("extdata", "data_repeated_measure_1.csv", package = "rtpcr"))
# res3 <- ANOVA_DDCt(
#   data,
#   numOfFactors = 1,
#   numberOfrefGenes = 1,
#   specs = "time",
#   block = NULL,
#   model = wDCt ~ time + (1 | id))
# 
# residuals <- resid(res3$perGene$Target$lm)
# shapiro.test(residuals)
# par(mfrow = c(1,2))
# plot(residuals)
# qqnorm(residuals)
# qqline(residuals, col = "red")

## ----eval= F------------------------------------------------------------------
# # Example input data frame with technical replicates
# data1 <- read.csv(system.file("extdata", "data_withTechRep.csv", package = "rtpcr"))
# 
# # Calculate mean of technical replicates using first four columns as groups
# meanTech(data1,
#          groups = 2,
#          numOfFactors = 1,
#          block = NULL)

#install.packages("rtpcr")
library(rtpcr)
library(shiny)
library(multcompView)
library(dplyr)
library(ggplot2)
library(emmeans)
library(multcomp)
library(lme4)
library(lmerTest)

# UI
ui <- fluidPage(
  
  
  tags$head(
    tags$link(rel = "stylesheet", href = "https://fonts.googleapis.com/css2?family=Roboto:wght@300;400;700&display=swap"),
    tags$style(HTML("
      body { font-family: 'Roboto', sans-serif; font-size: 16px; line-height: 1.7;  color: #333; }
      .intro-container { padding: 20px; }
      h2 { font-size: 30px; font-weight: 700; color: #2c3e50; }
      h3 { font-size: 24px; font-weight: 700; color: #2c3e50; margin-top: 30px; }
      h4 { font-size: 20px; font-weight: 400; }
      .info-box { background-color: #f0f7ff; border-left: 4px solid #2a7df6; padding: 20px; margin: 25px 0; border-radius: 4px; font-size: 15.5px; }
      .info-box2 { background-color: #f0ffff; border: 1px solid #2a7dff; padding: 20px; border-radius: 4px; font-size: 15.5px; }
      .info-box3 { border: 2px solid #fff; padding: 20px; border-radius: 4px; font-size: 15.5px; }
      .info-box strong { color: #2a7df6; }
      .info-box br { margin-bottom: 10px; content: ' '; display: block; }
      .note-title { font-weight: bold; color: #2a7df6; text-transform: uppercase; font-size: 0.85em; }
      .img-container { text-align: center; margin: 35px 0; border: 1px solid #EE3B3B; padding: 15px; background: #fff; border-radius: 8px; }  
      .logo-container { text-align: center; border-radius: 8px; }
      .txt-container { margin: 35px 0; border: 1px solid #ddd; padding: 15px; background: #fff; border-radius: 8px; }
      .fig-caption { font-style: italic; color: #666; margin-top: 12px; display: block; font-size: 14px; }
      .section-divider { border-top: 1px solid #C1CDCD; margin: 20px 0; }
    "))
),
  
  
  tags$script(HTML("
    $(document).on('keyup', function(e) {
      if (e.which == 13) {
        var activeTab = $('#func_tabs .active a').attr('data-value');
        var btnId = '';
        if (activeTab === 'meanTech') btnId = '#run_mt';
        if (activeTab === 'ANOVA_DCt') btnId = '#run_dc';
        if (activeTab === 'ANOVA_DDCt') btnId = '#run_ddct';
        if (activeTab === 'Efficiency') btnId = '#run_eff';
        if (activeTab === 'TTEST_DDCt') btnId = '#run_tt';
        if (activeTab === 'WILCOX_DDCt') btnId = '#run_wx';
        if (activeTab === 'plotFactor') btnId = '#run_pf';
        
        if (btnId !== '') {
          $(btnId).click();
        }
      }
    });
  ")),

  
  titlePanel('rtpcr: qPCR Data Analysis & Plotting'),
  fluidRow(column(12,
           div(style = "margin-bottom: 20px; color: #333; line-height: 1.6;", 
               p("Welcome! This is the shiny version of the rtpcr package, a web application developed using R/Shiny for comparative (dCt and ddCt) analysis of qPCR data.")))),
  br(),
  
  sidebarLayout(
    sidebarPanel(
      tabsetPanel(id = "func_tabs",
                  tabPanel("Home", value = "Home",
                           br(),
                           div(class = "info-box3", h4("Citation"), "Ghader Mirzaghaderi (2025). rtpcr: a package for statistical analysis 
                               and graphical presentation of qPCR data in R. PeerJ 13:e20185.", 
                               a(href="https://doi.org/10.7717/peerj.20185", "doi.org/10.7717/peerj.20185")),
                           
                           ),

                  
                  
                  
                  
                  
                  tabPanel("ANOVA_DCt", value = "ANOVA_DCt",
                           radioButtons("src_dc", "Data Source", 
                                        choices = c("Upload CSV" = "user", "Sample Data" = "sample", "Result: meanTech" = "res_mt"), 
                                        selected = "user"),
                           conditionalPanel("input.src_dc == 'user'", fileInput("file_dc", "Upload CSV", accept = ".csv")),
                           numericInput("numFactors_dc", "Number of factors", 1, min = 1),
                           numericInput("numRefGenes_dc", "Number of reference genes", 1, min = 1),
                           textInput("block_dc", "Block column name (if present)", ""),
                           textInput("model_dc", "model (optional; e.g., model = wDCt ~ factorA * factorB)", ""),
                           checkboxInput("set40_dc", "Set missing Ct to 40", FALSE),
                           checkboxInput("setModelse_dc", "Model_based se", TRUE),
                           selectInput("pAdj_dc", "p-value adjustment", choices = c("none","holm","bonferroni","fdr")),
                           actionButton("run_dc", "Run ANOVA_DCt"),
                  ),
                  
                  tabPanel("ANOVA_DDCt", value = "ANOVA_DDCt",
                           radioButtons("src_ddct", "Data Source", 
                                        choices = c("Upload CSV" = "user", "Sample Data" = "sample", "Result: meanTech" = "res_mt"), 
                                        selected = "user"),
                           conditionalPanel("input.src_ddct == 'user'", fileInput("file_ddct", "Upload CSV", accept = ".csv")),
                           numericInput("numFactors_ddct", "Number of factors", 1, min = 1),
                           numericInput("numRefGenes_ddct", "Number of reference genes", 1, min = 1),
                           textInput("specs_ddct", "specs (e.g., A or A | B or A | B * C)", ""),
                           textInput("block_ddct", "Block column name (if present)", ""),
                           textInput("calibrator_ddct", "Calibrator Level (Optional; default is the first level)", ""),
                           selectInput("seType_ddct", "SE type", choices = c("single.group",  "two.group", "paired.group")),
                           textInput("model_ddct", "model (optional; e.g., model = wDCt ~ factorA * factorB)", ""),
                           selectInput("pAdj_ddct", "p-value adjustment", choices = c("none","holm","bonferroni","fdr")),
                           checkboxInput("set40_ddct", "Set missing Ct to 40", FALSE),
                           checkboxInput("setModelse_ddct", "Model_based se", TRUE),
                           actionButton("run_ddct", "Run ANOVA_DDCt")
                  ),
                  
                  tabPanel("Efficiency", value = "Efficiency",
                           radioButtons("src_eff", "Data Source",choices = c("Upload CSV" = "user", "Sample Data" = "sample"), selected = "user"),
                           conditionalPanel("input.src_eff == 'user'", fileInput("file_eff", "Upload CSV", accept = ".csv")),
                           numericInput("baseSize_eff", "Base font size", 12, min = 6),
                           textInput("extra_eff", "Further ggplot layers (Optional; e.g., + ylab('Ct value'))", ""),
                           hr(),
                           h4("Plot size"),
                           fluidRow(
                             column(6, numericInput("eff_w", "Width (in)", 8, min = 1)),
                             column(6, numericInput("eff_h", "Height (in)", 6, min = 1))
                           ),
                           actionButton("run_eff", "Run Efficiency")
                  ),
                  
                  tabPanel("TTEST_DDCt", value = "TTEST_DDCt",
                           radioButtons("src_tt", "Data Source", 
                                        choices = c("Upload CSV" = "user", "Sample Data" = "sample", "Result: meanTech" = "res_mt"), 
                                        selected = "user"),
                           conditionalPanel("input.src_tt == 'user'", fileInput("file_tt", "Upload CSV", accept = ".csv")),
                           numericInput("numRefGenes_tt", "Number of reference genes", 1),
                           textInput("factorLevels_tt", "Factor levels (comma separated)", ""),
                           checkboxInput("paired_tt", "Paired test", FALSE),
                           checkboxInput("equalVar_tt", "Equal variance", TRUE),
                           selectInput("pAdj_tt", "p-value adjustment", choices = c("none","holm","bonferroni","fdr")),
                           checkboxInput("set40_tt", "Set missing Ct to 40", FALSE),
                           actionButton("run_tt", "Run TTEST_DDCt")
                  ),
                  
                  tabPanel("WILCOX_DDCt", value = "WILCOX_DDCt",
                           radioButtons("src_wx", "Data Source", 
                                        choices = c("Upload CSV" = "user", "Sample Data" = "sample", "Result: meanTech" = "res_mt"), 
                                        selected = "user"),
                           conditionalPanel("input.src_wx == 'user'", fileInput("file_wx", "Upload CSV", accept = ".csv")),
                           numericInput("numRefGenes_wx", "Number of reference genes", 1),
                           textInput("factorLevels_wx", "Factor levels (comma separated)", ""),
                           checkboxInput("paired_wx", "Paired test", FALSE),
                           selectInput("pAdj_wx", "p-value adjustment", choices = c("none","holm","bonferroni","fdr")),
                           checkboxInput("set40_wx", "Set missing Ct to 40", FALSE),
                           actionButton("run_wx", "Run WILCOX_DDCt")
                  ),
                  
                  tabPanel("plotFactor", value = "plotFactor",
                           radioButtons("src_pf", "Data Source", 
                                        choices = c("Upload CSV" = "user", 
                                                    "Sample Data" = "sample",
                                                    "Result: ANOVA_DCt" = "res_dc",
                                                    "Result: ANOVA_DDCt" = "res_ddct",
                                                    "Result: TTEST_DDCt" = "res_tt",
                                                    "Result: WILCOX_DDCt" = "res_wx"), 
                                        selected = "user"),
                           conditionalPanel("input.src_pf == 'user'", fileInput("file_pf", "Upload CSV", accept = ".csv")),
                           selectInput("pf_x", "X Axis Column", choices = NULL),
                           selectInput("pf_y", "Y Axis Column", choices = NULL),
                           selectInput("pf_low", "Lower Error Column", choices = NULL),
                           selectInput("pf_up", "Upper Error Column", choices = NULL),
                           selectInput("pf_group", "Group (Fill) Column", choices = c("None" = "")),
                           selectInput("pf_facet", "Facet Column", choices = c("None" = "")),
                           selectInput("pf_letters", "Letters Column (Optional)", choices = c("None" = "")),
                           numericInput("pf_letters_d", "Letters vertical offset", 0.2, step = 0.05),
                           numericInput("pf_col_width", "Column width", 0.8, step = 0.05),
                           numericInput("pf_err_width", "Error bar width", 0.15, step = 0.05),
                           numericInput("pf_dodge_width", "Dodge width", 0.8, step = 0.05),
                           textInput("pf_fill_colors", "Fill colors (comma separated)", ""),
                           textInput("pf_color", "Outline color", "black"),
                           sliderInput("pf_alpha", "Transparency (alpha)", min = 0, max = 1, value = 0.5, step = 0.05),
                           numericInput("pf_base_size", "Base font size", 12, min = 6),
                           checkboxInput("pf_legend_none", "Hide legend", FALSE),
                           checkboxInput("pf_removeRows", "Remove Calibrator Cols", FALSE),
                           checkboxInput("pf_removeText", "Remove Calibrator Text", FALSE),
                           fluidRow(column(6, numericInput("pf_legend_x", "Legend X", 0.85, min = 0, max = 1, step = 0.05) ),
                                    column(6, numericInput("pf_legend_y", "Legend Y", 0.85, min = 0, max = 1, step = 0.05) ) ),
                           fluidRow(column(6, numericInput("pf_w", "Width (in)", 8, min = 1)),
                                    column(6, numericInput("pf_h", "Height (in)", 9, min = 1))),
                           textInput("extra_pf", "Further ggplot layers (Optional; e.g., + ylab('Fold change'))", ""),
                           actionButton("run_pf", "Run plotFactor")
                  ),
                  tabPanel("meanTech", value = "meanTech",
                           p("Only apply this function if your dataset requires averaging across technical replicates before further analysis.", 
                             style = "color: #555; font-style: italic; margin-top: 10px; margin-bottom: 15px;"),
                           
                           radioButtons("src_mt", "Data Source", choices = c("Upload CSV" = "user", "Sample Data" = "sample"), selected = "user"),
                           conditionalPanel("input.src_mt == 'user'", fileInput("file_mt", "Upload CSV", accept = ".csv")),
                           numericInput("groups_mt", "Groups (Number of grouping columns)", 3, min = 1),
                           numericInput("numFactors_mt", "Number of factors", 2, min = 1),
                           numericInput("numRefGenes_mt", "Number of reference genes", 1, min = 1),
                           textInput("block_mt", "Block column (optional)", ""),
                           checkboxInput("set40_mt", "Set missing Ct to 40", FALSE),
                           actionButton("run_mt", "Run meanTech")
                  )
      )
    ),
    mainPanel(
      tabsetPanel(id = "output_tabs",
                  tabPanel("Introduction", value = "Introduction",
                           div(style = "padding: 20px; line-height: 1.6;",
                               h2("Getting Started with rtpcr"),
                               p("The rtpcr package facilitates relative expression analysis using delta Ct (dCt) and delta delta Ct (ddCt) methods. It supports t-test, ANOVA, and publication-ready visualizations. The package implements a general calculation method adopted from Ganger et al. (2017) and Taylor et al. (2019), covering both the Livak and Pfaffl methods."),
                               
                               div(class = "section-divider"),
                               
            
                              h2("rtpcr Funtions"),
                              p("In the rtpcr package, functions with _DDCt at the end of their name (ANOVA_DDCt, TTEST_DDCt, WILCOX_DDCt) perform expression analysis based on the delta delta Ct (ddCt)
                               method, while ANOVA_DCt function analyze gene expression using the delta Ct (dCt) method. The ANOVA prefix in the function name means that the function uses analysis of variance (using a default full factorial
                               model or a user defined model) for statistical analysis, and mean comparisons. Mean comparisons is performed by the emmeans function using the model resulting from the ANOVA analysis."),

                              helpText(div(class = "info-box2",
                                           tags$strong("ANOVA_DCt: "), " dCt expression analysis for all the level combinations of factor(s).", 
                                           br(),
                                             tags$strong("ANOVA_DDCt: "), " ddCt expression analysis for levels of a factor (geneally or per levels of another factors(s)), specified by the `specs` argument. ", tags$br(),
                                             tags$strong("TTEST_DDCt: "), " ddCt method t.test analysis for paired or unpaired samples.", tags$br(),
                                             tags$strong("WILCOX_DDCt: "), " ddCt method wilcox.test analysis for paired or unpaired samples.", tags$br(),
                                             tags$strong("plotFactor: "), " Bar plot of gene expression", tags$br(),
                                             tags$strong("efficiency: "), " Amplification efficiency statistics and standard curves", tags$br(),
                                             tags$strong("meanTech: "), " Calculate mean of technical replicates. This is used if your data needs averaging over biological replicates. ")),

                              

                               h3("1. Input Data Structure"),
                               p("For relative expression analysis, use TTEST_DDCt, WILCOX_DDCt, ANOVA_DCt, and ANOVA_DDCt functions from the left side panel. 
                               You need to prepare your input data based on the experimenta design, amplification efficiency 
                                 (E) and Ct values (the mean of technical replicates) which should include the following columns from left to 
                                wright:"),
                               tags$ol(
                                 tags$li("Experimental condition columns (and one block column if you want the variation from different plates be concidered by the nodel)."),
                                 tags$li("Replicate information (biological replicates or subjects)."),
                                 tags$li("Target genes efficiency (E) and Ct values (paired columns)."),
                                 tags$li("Reference genes efficiency (E) and Ct values (paired columns).")
                               ),
                               p("If the E values are not available you should use 
                                 ‘2’ instead representing the complete primer amplification efficiency. The package supports one or more target or reference gene(s),
                               supplied as efficiency–Ct column pairs. Reference gene columns must
                               always appear last. Two sample input data sets are presented below."),
                               
                               div(class = "img-container",
                                   img(src = "sampleData1.png", style = "width: 70%; max-width: 600px;"),
                                   span(class = "fig-caption", tags$strong("Figure 1: "), "A sample input data with one experimetal factor, replicate column and E/Ct information of target and reference genes.")),
                               
                               p("If there is no blocking factor, omit that column. However, a replicate column (e.g., 'Rep' or 'id') is always required."),
                               
                               div(class = "img-container",
                                   img(src = "dataStructure1.png", style = "width: 100%; max-width: 800px;"),
                                   span(class = "fig-caption", tags$strong("Figure 2: "), "A sample input data with two experimental factors, blocking factor, replicate column and E/Ct information of target and reference genes.")),

                               
                               h3("2. Notes"),
                               div(class = "info-box",
                                   tags$strong("Note 1: Blocking:"), " When a qPCR experiment is done in multiple qPCR plates, variation resulting from the plates may interfere with the actual amount of gene 
                                                                       expression. One solution is to conduct each plate as a randomized block so that at least one replicate of each treatment and control is present on a plate. Block effect is usually considered as random and its
                                                                       interaction with any main effect is not considered.", tags$br(),
                                   br(),
                                   tags$strong("Note 2: Biological Replicates:"), " For TTEST_DDCt and WILCOX_DDCt (independent groups), ANOVA_DCt,
                                                                       and ANOVA_DDCt each row is from a separate and unique biological replicate. For example, a data frame with 12 rows has come from an experiment with 12 individuals. The repeated measure models are intended for experiments with repeated observations (e.g. time-course data). In repeated measure experiments the Replicate column contains identifiers for each individual (id or subject). For example, all rows with a `1` at Rep column correspond to a single individual, all rows with a `2` correspond to another individual, and so on, which have been sampled at
                                                                       specific time points.", tags$br(),
                                   br(),
                                   tags$strong("Note 3: Technical Replicates:"), " Your data table may also include a column of technical replicates (For example, using one target and one reference genes, if you want to have 4 
                                                biological and 3 technical replicates under Control and Treatment conditions, there would be a table of 24 rows containing both biological 
                                                replicates and technical replicate columns in the data). In this case, the meanTech function should be applied first to calculate the mean of 
                                                the technical replicates. The resulting collapsed table is then used as the input for expression analysis. To use the meanTech function correctly, the technical replicate column must appear immediately after the biological replicate column (see the sample data of the meanTech function).", tags$br(),
                                   br(),
                                   tags$strong("Note 4: Efficiency (E):"), " Complete amplification efficiency (E) in the input data is denoted by
                                                2. This means that 2 indicates 100%, and 1.85 and 1.70 indicate 0.85%
                                                and 0.70% amplification efficiencies."),

                               
                               
                               
                               h3("3. Handling Missing Data"),
                               p("Missing Ct values for target genes is Handled using the `set_missing_target_Ct_to_40` function. If TRUE, missing target gene
                                  Ct values become 40; if FALSE (default), they become NA. missing Ct values of reference genes are always converted to NA. If there are more
                                  than one reference gene, NA in the place of the E or the Ct value cause skipping that gene and remaining references are geometrically averaged in that replicate."),
                               
                               div(class = "section-divider"),
                               
                               p("Contact email: gh.mirzaghaderi at uok.ac.ir"),
                               p("For further details please visit:",
                                 a(href="https://github.com/mirzaghaderi/rtpcr", "https://github.com/mirzaghaderi/rtpcr")),
                               )
                  ),
                  tabPanel("ANOVA_DCt", value = "ANOVA_DCt",
                           br(),
                           tabsetPanel(id = "sub_dc",
                                       tabPanel("Input data", tableOutput("preview_dc")),
                                       tabPanel("Relative Expression", br(), downloadButton("download_dc","Download"), br(), tableOutput("table_dc")),
                                       tabPanel("Per-Gene Results",
                                                selectInput("gene_dc", "Select gene:", choices = character(0)),
                                                tabsetPanel(
                                                  tabPanel("ANOVA, Normality & Singularity", verbatimTextOutput("singular_dc"),
                                                           p("NOTE ", style = "color: #EE3B3B;"),
                                                           p("If the singularity of the model is TRUE, the statistical results including ANOVA and significance is not reliable !",
                                                           style = "color: #9C9C9C; font-style: italic; margin-top: 10px; margin-bottom: 15px;")),
                                                  tabPanel("LM Object", br(), downloadButton("download_lm_dc", "Download LM (.rds)")),
                                                  tabPanel("Final table", br(), downloadButton("download_final_dc", "Download Final Table"), br(), tableOutput("final_data_dc"))
                                                ))
                           )
                  ),
                  tabPanel("ANOVA_DDCt", value = "ANOVA_DDCt",
                           br(),
                           tabsetPanel(id = "sub_ddct",
                                       tabPanel("Input data", tableOutput("preview_ddct")),
                                       tabPanel("Relative Expression", br(), downloadButton("download_ddct", "Download"), br(), tableOutput("table_ddct")),
                                       tabPanel("Per-Gene Results",
                                                selectInput("gene_ddct", "Select gene:", choices = character(0)),
                                                tabsetPanel(
                                                  tabPanel("ANOVA, Normality & Singularity", verbatimTextOutput("singular_ddct"), 
                                                           p("NOTE ", style = "color: #EE3B3B;"),
                                                           p("If the singularity of the model is TRUE, the statistical results including ANOVA and significance is not reliable !",
                                                           style = "color: #9C9C9C; font-style: italic; margin-top: 10px; margin-bottom: 15px;")),
                                                  tabPanel("LM Object", br(), downloadButton("download_lm_ddct", "Download LM (.rds)")),
                                                  tabPanel("Final table", br(), downloadButton("download_final_ddct", "Download Final Table"), br(), tableOutput("final_data_ddct"))
                                                ))
                           )
                  ),
                  tabPanel("Efficiency", value = "Efficiency",
                           br(),
                           tabsetPanel(id = "sub_eff",
                                       tabPanel("Input data", tableOutput("preview_eff")),
                                       tabPanel("Values & Statistics", br(), downloadButton("download_eff", "Download CSV"), br(), verbatimTextOutput("table_eff")),
                                       tabPanel("Plot", br(), downloadButton("download_eff_png", "PNG"), downloadButton("download_eff_pdf", "PDF"), hr(), plotOutput("plot_eff"))
                           )
                  ),
                  tabPanel("TTEST_DDCt", value = "TTEST_DDCt",
                           br(),
                           tabsetPanel(id = "sub_tt",
                                       tabPanel("Input Data", tableOutput("preview_tt")),
                                       tabPanel("Results", br(), downloadButton("download_tt", "Download CSV"), br(), tableOutput("table_tt"))
                           )
                  ),
                  tabPanel("WILCOX_DDCt", value = "WILCOX_DDCt",
                           br(),
                           tabsetPanel(id = "sub_wx",
                                       tabPanel("Input Data", tableOutput("preview_wx") ),
                                       tabPanel("Results", br(), downloadButton("download_wx", "Download CSV"), br(), tableOutput("table_wx"))
                           )
                  ),
                  tabPanel("plotFactor", value = "plotFactor",
                           br(),
                           tabsetPanel(id = "sub_pf",
                                       tabPanel("Input Data", tableOutput("preview_pf")),
                                       tabPanel("Plot", br(), downloadButton("download_pf_plot", "PNG"), downloadButton("download_pf_pdf", "PDF"), hr(), plotOutput("plot_pf_main"))
                                       )
                  ),
                  
                  tabPanel("meanTech", value = "meanTech",
                           br(),
                           tabsetPanel(id = "sub_mt",
                                       tabPanel("Input data", tableOutput("preview_mt")),
                                       tabPanel("Results", br(), downloadButton("download_mt", "Download Results"), br(), tableOutput("table_mt"))
                           )
                  )
      )
    )
  )
)

# SERVER
server <- function(input, output, session) {
  
  # Helper for data sources
  get_data <- function(src_input, file_input, sample_path, mean_res = NULL) {
    if (src_input == "sample") {
      return(read.csv(sample_path))
    } else if (src_input == "res_mt") {
      req(mean_res)
      return(mean_res)
    } else {
      req(file_input)
      return(read.csv(file_input$datapath))
    }
  }
  
  # meanTech Logic
  data_mt <- reactive({ get_data(input$src_mt, input$file_mt, "mean/data_withTechRep.csv") })
  
  res_mt <- eventReactive(input$run_mt, {
    tryCatch({
      meanTech(data_mt(), groups = input$groups_mt, numOfFactors = input$numFactors_mt, 
               numberOfrefGenes = input$numRefGenes_mt, block = if(input$block_mt=="") NULL else input$block_mt,
               set_missing_target_Ct_to_40 = input$set40_mt)
    }, error = function(e) { showNotification(e$message, type="error"); NULL })
  })
  
  output$preview_mt <- renderTable({ req(data_mt()); head(data_mt(), 50) })
  output$table_mt   <- renderTable({ req(res_mt()); head(res_mt(), 50) }, digits = 5)
  output$download_mt <- downloadHandler(filename = "meanTech_results.csv", content = function(f) write.csv(res_mt(), f, row.names=F))
  
  # Analysis Logic
  data_dc   <- reactive({ get_data(input$src_dc, input$file_dc, "exp/data_3factor.csv", res_mt()) })
  data_ddct <- reactive({ get_data(input$src_ddct, input$file_ddct, "exp/data_2factorBlock3ref.csv", res_mt()) })
  data_eff  <- reactive({ get_data(input$src_eff, input$file_eff, "eff/data_efficiency1.csv") })
  data_tt   <- reactive({ get_data(input$src_tt, input$file_tt, "exp/data_ttest18genes.csv", res_mt()) })
  data_wx   <- reactive({ get_data(input$src_wx, input$file_wx, "exp/data_ttest18genes.csv", res_mt()) })
  
  output$preview_dc <- renderTable({ req(data_dc()); head(data_dc(), 50) })
  output$preview_ddct <- renderTable({ req(data_ddct()); head(data_ddct(), 50) })
  output$preview_eff <- renderTable({ req(data_eff()); head(data_eff(), 50) })
  output$preview_tt <- renderTable({ req(data_tt()); head(data_tt(), 50) })
  output$preview_wx <- renderTable({ req(data_wx()); head(data_wx(), 50) })
  
  # Sample updates
  observeEvent(input$src_mt, {
    if (input$src_mt == "sample") {
      updateNumericInput(session, "groups_mt", value = 2)
      updateNumericInput(session, "numFactors_mt", value = 1)
      updateNumericInput(session, "numRefGenes_mt", value = 1)
    }
  })
  observeEvent(input$src_dc, {
    if (input$src_dc == "sample") { updateNumericInput(session, "numFactors_dc", value = 3); updateNumericInput(session, "numRefGenes_dc", value = 1) }
  })
  observeEvent(input$src_ddct, {
    if (input$src_ddct == "sample") { updateNumericInput(session, "numFactors_ddct", value = 2); updateNumericInput(session, "numRefGenes_ddct", value = 3); updateTextInput(session, "specs_ddct", value = "Concentration"); updateTextInput(session, "block_ddct", value = "block") }
  })
  
  # Event Handlers
  observeEvent(input$run_mt,   { updateTabsetPanel(session, "sub_mt", selected = "Results") })
  observeEvent(input$run_dc,   { updateTabsetPanel(session, "sub_dc", selected = "Relative Expression") })
  observeEvent(input$run_ddct, { updateTabsetPanel(session, "sub_ddct", selected = "Relative Expression") })
  observeEvent(input$run_eff,  { updateTabsetPanel(session, "sub_eff", selected = "Values & Statistics") })
  observeEvent(input$run_tt,   { updateTabsetPanel(session, "sub_tt", selected = "Results") })
  observeEvent(input$run_wx,   { updateTabsetPanel(session, "sub_wx", selected = "Results") })
  observeEvent(input$run_pf,   { updateTabsetPanel(session, "sub_pf", selected = "Plot") })
  
  res_dc <- eventReactive(input$run_dc, {
    tryCatch({ ANOVA_DCt(data_dc(), numOfFactors = input$numFactors_dc, numberOfrefGenes = input$numRefGenes_dc,
                         block = if(input$block_dc == "") NULL else input$block_dc, 
                         model = if(input$model_dc == "") NULL else input$model_dc,
                         p.adj = input$pAdj_dc, set_missing_target_Ct_to_40 = input$set40_dc,
                         modelBased_se = input$setModelse_dc)
    }, error = function(e) { showNotification(e$message, type="error"); NULL })
  })
  
  res_ddct <- eventReactive(input$run_ddct, {
    tryCatch({ ANOVA_DDCt(data_ddct(), numOfFactors = input$numFactors_ddct, numberOfrefGenes = input$numRefGenes_ddct,
                          specs = input$specs_ddct, block = if(input$block_ddct == "") NULL else input$block_ddct,
                          calibratorLevel = if(input$calibrator_ddct == "") NULL else input$calibrator_ddct,
                          p.adj = input$pAdj_ddct, set_missing_target_Ct_to_40 = input$set40_ddct, 
                          model = if(input$model_ddct == "") NULL else input$model_ddct,
                          se.type = input$seType_ddct, modelBased_se = input$setModelse_ddct)
    }, error = function(e) { showNotification(e$message, type="error"); NULL })
  })
  
  res_eff <- eventReactive(input$run_eff, {
    res <- efficiency(data_eff(), base_size = input$baseSize_eff)
    if (nzchar(trimws(input$extra_eff))) {
      tryCatch({ res$plot <- eval(parse(text = paste0("res$plot", input$extra_eff))) }, error = function(e) { showNotification(e$message, type = "error") })
    }
    res
  })
  
  res_tt <- eventReactive(input$run_tt, {
    TTEST_DDCt(data_tt(), numberOfrefGenes = input$numRefGenes_tt,
               Factor.level.order = if(input$factorLevels_tt == "") NULL else trimws(unlist(strsplit(input$factorLevels_tt, ","))),
               paired = input$paired_tt, var.equal = input$equalVar_tt, p.adj = input$pAdj_tt, 
               set_missing_target_Ct_to_40 = input$set40_tt)
  })
  
  res_wx <- eventReactive(input$run_wx, {
    tryCatch({ WILCOX_DDCt(data_wx(), numberOfrefGenes = input$numRefGenes_wx,
                           Factor.level.order = if(input$factorLevels_wx == "") NULL else trimws(unlist(strsplit(input$factorLevels_wx, ","))),
                           paired = input$paired_wx, p.adj = input$pAdj_wx, set_missing_target_Ct_to_40 = input$set40_wx)
    }, error = function(e) { showNotification(e$message, type = "error"); NULL })
  })
  
  # Gene selection updates
  observe({
    req(res_dc())
    gene_list_dc <- names(res_dc()$perGene)
    if(!is.null(gene_list_dc)) {
      updateSelectInput(session, "gene_dc", choices = gene_list_dc, selected = gene_list_dc[1])
    }
  })
  
  observe({
    req(res_ddct())
    gene_list_ddct <- names(res_ddct()$perGene)
    if(!is.null(gene_list_ddct)) {
      updateSelectInput(session, "gene_ddct", choices = gene_list_ddct, selected = gene_list_ddct[1])
    }
  })
  
  output$table_dc <- renderTable({ res_dc()$relativeExpression }, digits = 5)
  output$singular_dc <- renderPrint({ req(res_dc(), input$gene_dc); list(
    Linear_model = res_dc()$perGene[[input$gene_dc]]$lm_formula,
    ANOVA_table = res_dc()$perGene[[input$gene_dc]]$ANOVA_table,
    Normality_test_of_residuals = shapiro.test(res_dc()$perGene[[input$gene_dc]]$Final_data$residual),
    Singularity_check_of_model = res_dc()$perGene[[input$gene_dc]]$is_singular)})
  output$final_data_dc <- renderTable({ req(res_dc(), input$gene_dc); res_dc()$perGene[[input$gene_dc]]$Final_data }, digits = 5)
  
  output$table_ddct <- renderTable({ res_ddct()$relativeExpression }, digits = 5)
  output$singular_ddct <- renderPrint({ req(res_ddct(), input$gene_ddct); list(
    Linear_model = res_ddct()$perGene[[input$gene_ddct]]$lm_formula,
    res_ddct()$perGene[[input$gene_ddct]]$ANOVA_table,
    Normality_test_of_residuals = shapiro.test(res_ddct()$perGene[[input$gene_ddct]]$Final_data$residual),
    Singularity_check_of_model = res_ddct()$perGene[[input$gene_ddct]]$singular)})
  output$final_data_ddct <- renderTable({ req(res_ddct(), input$gene_ddct); res_ddct()$perGene[[input$gene_ddct]]$Final_data }, digits = 5)
  
  output$table_eff <- renderPrint({ list(Efficiency = res_eff()$Efficiency, 
                                         Slope_compare = res_eff()$Slope_compare) })
  output$plot_eff <- renderPlot({ res_eff()$plot }, width = function() input$eff_w * 72, height = function() input$eff_h * 72)
  output$table_tt <- renderTable({ res_tt() }, digits = 5)
  output$table_wx <- renderTable({ res_wx() }, digits = 5)
  
  # plotFactor Logic
  df_pf <- reactive({ 
    if (input$src_pf == "user") { req(input$file_pf); read.csv(input$file_pf$datapath) }
    else if (input$src_pf == "sample") { read.csv("plot/out_ANOVA_DDCt.csv") }
    else if (input$src_pf == "res_dc") { req(res_dc()); res_dc()$relativeExpression }
    else if (input$src_pf == "res_ddct") { req(res_ddct()); res_ddct()$relativeExpression }
    else if (input$src_pf == "res_tt") { req(res_tt()); res_tt() }
    else if (input$src_pf == "res_wx") { req(res_wx()); res_wx() }
  })
  
  observeEvent(df_pf(), {
    cols <- colnames(df_pf()); pick_col <- function(i) cols[min(i, length(cols))]
    updateSelectInput(session, "pf_x", choices = cols, selected = pick_col(2))
    updateSelectInput(session, "pf_y", choices = cols, selected = pick_col(4))
    updateSelectInput(session, "pf_low", choices = cols, selected = pick_col(9))
    updateSelectInput(session, "pf_up", choices = cols, selected = pick_col(10))
    updateSelectInput(session, "pf_group", choices = c("None" = "", cols), selected = if(input$src_pf=="sample") "gene" else "")
    updateSelectInput(session, "pf_facet", choices = c("None" = "", cols), selected = if(input$src_pf=="sample") "gene" else "")
    updateSelectInput(session, "pf_letters", choices = c("None" = "", cols), selected = if(input$src_pf=="sample") "sig" else "")
  })
  
  pf_plot_obj <- eventReactive(input$run_pf, {
    p <- plotFactor(data = df_pf(), x_col = input$pf_x, y_col = input$pf_y, Lower.se_col = input$pf_low,
                    Upper.se_col = input$pf_up, group_col = if(input$pf_group == "") NULL else input$pf_group,
                    facet_col = if(input$pf_facet == "") NULL else input$pf_facet,
                    letters_col = if(input$pf_letters == "") NULL else input$pf_letters,
                    letters_d = input$pf_letters_d, col_width = input$pf_col_width, err_width = input$pf_err_width,
                    dodge_width = input$pf_dodge_width, fill_colors = if (input$pf_fill_colors == "") NULL else trimws(unlist(strsplit(input$pf_fill_colors, ","))),
                    color = if (input$pf_color == "") NA else input$pf_color, 
                    alpha = input$pf_alpha, 
                    base_size = input$pf_base_size,
                    legend_position = if (input$pf_legend_none) "none" else c(input$pf_legend_x, input$pf_legend_y),
                    removeCalibratorCols = input$pf_removeRows, removeCalibratorText = input$pf_removeText)
    if (nzchar(trimws(input$extra_pf))) {
      tryCatch({ p <- eval(parse(text = paste0("p", input$extra_pf))) }, error = function(e) { showNotification(paste("Plot adjustment error:", e$message), type = "error") })
    }
    p
  })
  output$preview_pf <- renderTable({ req(df_pf()); head(df_pf(), 50) })
  output$plot_pf_main <- renderPlot({ req(pf_plot_obj()); pf_plot_obj() }, width = function() input$pf_w * 72, height = function() input$pf_h * 72)
  
  # Navigation logic
  observeEvent(input$func_tabs, {
    if (input$func_tabs == "Home") updateTabsetPanel(session, "output_tabs", selected = "Introduction")
    else updateTabsetPanel(session, "output_tabs", selected = input$func_tabs)
  })
  
  # Downloads
  output$download_dc <- downloadHandler(filename = "ANOVA_DCt.csv", content = function(f) write.csv(res_dc()$relativeExpression, f, row.names = FALSE))
  output$download_ddct <- downloadHandler(filename = "ANOVA_DDCt.csv", content = function(f) write.csv(res_ddct()$relativeExpression, f, row.names=F))
  output$download_eff <- downloadHandler(filename = "Efficiency.csv", content = function(f) write.csv(res_eff()$Efficiency, f, row.names=F))
  output$download_tt <- downloadHandler(filename = "TTEST.csv", content = function(f) write.csv(res_tt(), f, row.names=F))
  output$download_wx <- downloadHandler(filename = "WILCOX_DDCt.csv", content = function(f) write.csv(res_wx(), f, row.names = FALSE))
  output$download_final_dc <- downloadHandler(filename = function() paste0("Final_Table_DCt_", input$gene_dc, ".csv"), content = function(f) write.csv(res_dc()$perGene[[input$gene_dc]]$Final_data, f, row.names = FALSE))
  output$download_final_ddct <- downloadHandler(filename = function() paste0("Final_Table_DDCt_", input$gene_ddct, ".csv"), content = function(f) write.csv(res_ddct()$perGene[[input$gene_ddct]]$Final_data, f, row.names = FALSE))
  output$download_eff_png <- downloadHandler(filename = "Eff.png", content = function(f) ggsave(f, plot = res_eff()$plot, device = "png", width = input$eff_w, height = input$eff_h))
  output$download_eff_pdf <- downloadHandler(filename = "Eff.pdf", content = function(f) ggsave(f, plot = res_eff()$plot, device = "pdf", width = input$eff_w, height = input$eff_h))
  output$download_pf_plot <- downloadHandler(filename = "qPCR_Plot.png", content = function(f) ggsave(f, plot = pf_plot_obj(), device = "png", width = input$pf_w, height = input$pf_h))
  output$download_pf_pdf <- downloadHandler(filename = "qPCR_Plot.pdf", content = function(f) ggsave(f, plot = pf_plot_obj(), device = "pdf", width = input$pf_w, height = input$pf_h))
  output$download_lm_dc <- downloadHandler(filename = function() paste0("LM_", input$gene_dc, ".rds"), content = function(f) saveRDS(res_dc()$perGene[[input$gene_dc]]$lm, f))
  output$download_lm_ddct <- downloadHandler(filename = function() paste0("LM_", input$gene_ddct, ".rds"), content = function(f) saveRDS(res_ddct()$perGene[[input$gene_ddct]]$lm, f))
}

shinyApp(ui, server)

#' Delta Delta Ct ANOVA analysis with optional model specification
#'
#' Apply Delta Delta Ct (ddCt) analysis to each target gene
#' and performs per-gene statistical analysis.
#'
#' @param x The input data frame containing experimental design columns, replicates (integer), target gene
#' E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
#' columns must be located at the right end of the data frame. See "Input data 
#' structure" in vignettes for details about data structure.
#' @param numOfFactors Integer. Number of experimental factor columns
#' (excluding \code{rep} and optional \code{block}).
#' @param numberOfrefGenes Integer. Number of reference genes.
#' @param block Character. Block column name or \code{NULL}. 
#' When a qPCR experiment is done in multiple qPCR plates, 
#' variation resulting from the plates may interfere with the actual amount of 
#' gene expression. One solution is to conduct each plate as a randomized block 
#' so that at least one replicate of each treatment and control is present 
#' on a plate. Block effect is usually considered as random and its interaction 
#' with any main effect is not considered.
#' @param specs Example: "A", "A|B" or "A|B*C" if A, B and C are name of factor columns in the input data
#' The first name (here A) is the factor for which the relative expression is analysed.
#' @param calibratorLevel NULL or one of the levels of the first selected factor in specs argument. If NULL the first level of that factor is used as calibrator.
#' Optional character vector specifying the order of levels for the main factor.
#' If \code{NULL}, the first observed level is used as the calibrator.
#' If provided, the first element of the vector is used as the calibrator level.
#' @param analyseAllTarget Logical or character.
#' If \code{TRUE} (default), all target genes are analysed.
#' Alternatively, a character vector specifying the names (names of their Efficiency columns) of target genes
#' to be analysed.
#' @param model Optional model formula. If provided, this overrides the automatic formula (factorial CRD or RCBD 
#' based on \code{block} and \code{numOfFactors}). The formula uses 
#' \code{wDCt} as the response variable. 
#' For mixed models, random effects can be defined using \code{lmer} syntax 
#' (e.g., \code{"wDCt ~ Treatment + (1 | id)"}). When using \code{model}, 
#' the \code{block} and \code{numOfFactors} arguments are ignored for model 
#' specification, but still used for data structure identification.
#'   
#' for fixed effects only, the \code{"lm"} (ordinary least squares) is used. 
#' \code{"lmer"} is used for mixed effects models 
#' (requires the \code{lmerTest} package). If a custom formula is provided with 
#' random effects, the function will use \code{lmerTest::lmer()}; otherwise 
#' it will use \code{stats::lm()}. Note that \code{emmeans} supports both 
#' model types and will use appropriate degrees of freedom methods (Satterthwaite by default).
#' @param p.adj
#' Method for p-value adjustment. See \code{\link[stats]{p.adjust}}.
#' @param set_missing_target_Ct_to_40 If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA. 
#' @param se.type Character string specifying how standard error is calculated. 
#' One of \code{"paired.group"}, \code{"two.group"}, or \code{"single.group"}. 
#' \code{"paired.group"} computes SE from paired differences (used when a random 
#' \code{id} effect is present), \code{"two.group"} uses the unpaired two-group 
#' t-test standard error against the reference level, and \code{"single.group"} 
#' computes SE within each level using a one-group t-test.
#' @param modelBased_se Logical. If \code{TRUE} (default), standard errors are  
#' calculated from model-based residuals. If \code{FALSE}, standard errors are calculated directly from the observed 
#' \code{wDCt} values within each treatment group according to the selected \code{se.type}.  
#' For single factor data, both methods are the same. It is recommended to use \code{modelBased_se = TRUE} (default).
#' @param ... Additional arguments. Included for backward compatibility with deprecated \code{mainFactor.column}.
#' 
#' @importFrom stats setNames
#'
#' @details
#' ddCt analysis of variance (ANOVA) is performed for 
#' the main factor (as specified using \code{specs} argument) based on a full model factorial 
#' experiment by default. Analysis of covariance is also possible depending on the user defined model. 
#' If the interaction between the main factor and the covariate is significant, ANCOVA is not appropriate.
#' 
#' All the functions for relative expression analysis (including \code{TTEST_DDCt()}, 
#' \code{WILCOX_DDCt()}, \code{ANOVA_DDCt()}, and \code{ANOVA_DCt()}) return the 
#' relative expression table which include fold change and corresponding 
#' statistics. The output of \code{ANOVA_DDCt()}, 
#' and \code{ANOVA_DCt()} also include lm models, residuals, raw data and ANOVA table 
#' for each gene. 
#' 
#' The expression table returned by \code{TTEST_DDCt()}, 
#' \code{WILCOX_DDCt()}, and \code{ANOVA_DDCt()} functions 
#' include these columns: gene (name of target genes), 
#' contrast (calibrator level and contrasts for which the relative expression is computed), 
#' ddCt (mean of weighted delta delta Ct values), RE (relative expression or 
#' fold change = 2^-ddCt),  log2FC (log(2) of relative expression or fold change), 
#' pvalue, sig (per-gene significance), LCL (95\% lower confidence level), UCL (95\% upper confidence level),
#' se (standard error of mean calculated from the weighted delta Ct values of each of the main factor levels),
#' Lower.se.RE (The lower limit error bar for RE which is 2^(log2(RE) - se)), 
#' Upper.se.RE (The upper limit error bar for RE which is 2^(log2(RE) + se)),
#' Lower.se.log2FC (The lower limit error bar for log2 RE), and 
#' Upper.se.log2FC (The upper limit error bar for log2 RE)
#'
#' @import emmeans
#' @return
#' An object containing expression table, lm model, residuals, raw data and ANOVA table for each gene:
#' \describe{  
#' \item{ddCt expression table along with per-gene statistical comparison outputs}{\code{object$relativeExpression}}
#' \item{ANOVA table}{\code{object$perGene$gene_name$ANOVA_table}}
#' \item{lm ANOVA}{\code{object$perGene$gene_name$lm}}
#' \item{lm_formula}{\code{object$perGene$gene_name$lm_formula}}
#' \item{Residuals}{\code{resid(object$perGene$gene_name$lm)}}
#' }
#' @export
#' 
#' @references
#' LivakKJ, Schmittgen TD (2001).
#' Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR
#' and the Double Delta CT Method.
#' \emph{Methods}, 25(4), 402–408.
#' doi:10.1006/meth.2001.1262
#'
#' Ganger MT, Dietz GD, and Ewing SJ (2017).
#' A common base method for analysis of qPCR data and the application of
#' simple blocking in qPCR experiments.
#' \emph{BMC Bioinformatics}, 18, 1–11.
#' 
#' Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. (2019). 
#' The ultimate qPCR experiment: producing publication quality, reproducible 
#' data the first time. \emph{Trends in Biotechnology}, 37, 761-774. 
#' 
#' Yuan JS, Reed A, Chen F, Stewart N (2006).
#' Statistical Analysis of Real-Time PCR Data.
#' \emph{BMC Bioinformatics}, 7, 85.
#'
#' 
#' @examples
#' data1 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
#' ANOVA_DDCt(data1,
#'            numOfFactors = 2,
#'            numberOfrefGenes = 3,
#'            block = "block",
#'            specs = "Concentration",
#'            p.adj = "none")
#'            
#' data2 <- read.csv(system.file("extdata", "data_1factor_one_ref.csv", package = "rtpcr"))          
#' ANOVA_DDCt(data2,
#'            numOfFactors = 1,
#'            numberOfrefGenes = 1,
#'            block = NULL,
#'            specs = "Condition",
#'            p.adj = "none",
#'            se.type = "single.group")
#'   
#' # Repeated measure analysis
#' a <- ANOVA_DDCt(data_repeated_measure_1,
#'            numOfFactors = 1,
#'            numberOfrefGenes = 1,
#'            block = NULL,
#'            specs = "time",
#'            p.adj = "none", model = wDCt ~ time + (1 | id))
#'
#' a$perGene$Target$ANOVA_table
#'
#'
#' # Repeated measure analysis: split-plot in time
#' a <- ANOVA_DDCt(data_repeated_measure_2,
#'            numOfFactors = 2, numberOfrefGenes = 1,
#'            specs = "time", block = NULL,
#'            model = wDCt ~ treatment * time + (1 | id))
#'    
ANOVA_DDCt <- function(
    x,
    numOfFactors,
    numberOfrefGenes,
    specs,           
    block,           
    calibratorLevel = NULL, 
    p.adj = "none",
    analyseAllTarget = TRUE,
    model = NULL,
    set_missing_target_Ct_to_40 = FALSE,
    se.type = c("single.group", "paired.group", "two.group"),
    modelBased_se = TRUE, ...) {
  
  se.type <- match.arg(se.type)

  
  dots <- list(...)
  deprecated_args <- c("mainFactor.column", "mainFactor.level.order")
  for (arg in deprecated_args) {
    if (arg %in% names(dots)) {
      lifecycle::deprecate_warn(
        when = "2.1.5", 
        what = paste0("ANOVA_DDCt(", arg, ")"), 
        with = "ANOVA_DDCt(specs)") } }
  
  
  n <- ncol(x)
  nDesign <- if (is.null(block)) numOfFactors + 1 else numOfFactors + 2
  designCols <- seq_len(nDesign)
  nRefCols <- 2 * numberOfrefGenes
  refCols <- (n - nRefCols + 1):n
  targetCols <- setdiff(seq_len(n), c(designCols, refCols))
  
  if (length(targetCols) == 0 || length(targetCols) %% 2 != 0) {
    stop("The supplied arguments don't match the input data.")
  }
  
  
  detect_rep_id_random <- function(model, numOfFactors, block, x) {
    if (is.null(model)) return(FALSE)
    ftxt <- paste(deparse(formula(model)), collapse = " ")
    rep_id_col <- if (is.null(block)) numOfFactors + 1 else numOfFactors + 2
    rep_id_name <- colnames(x)[rep_id_col]
    grepl(paste0("\\|\\s*", rep_id_name, "\\b"), ftxt)
  }
  
  
  
  targetPairs <- split(targetCols, ceiling(seq_along(targetCols)/2))
  targetNames <- vapply(targetPairs, function(tc) colnames(x)[tc[1]], character(1))
  
  if (!isTRUE(analyseAllTarget)) {
    keep <- targetNames %in% analyseAllTarget
    if (!any(keep)) stop("None of the specified target genes were found in the data.")
    targetPairs <- targetPairs[keep]
    targetNames <- targetNames[keep]
  }
  
  perGene <- list()
  relativeExpression_list <- list()
  
  for (i in seq_along(targetPairs)) {
    
    tc <- targetPairs[[i]]
    gene_name <- targetNames[i]
    
    gene_df <- x[, c(designCols, tc, refCols), drop = FALSE]
    user_defined_model <- !is.null(model)
    
    
    
    if (is.null(block)) {
      gene_df[seq_len(numOfFactors)] <- lapply(
        gene_df[seq_len(numOfFactors)],
        function(col) factor(col, levels = unique(col)))
    } else {
      gene_df[seq_len(numOfFactors + 1)] <- lapply(
        gene_df[seq_len(numOfFactors + 1)],
        function(col) factor(col, levels = unique(col)))
    }
    
    mainFactor.level.order <- unique(gene_df[,1])
    mainFactor <- trimws(strsplit(specs, "\\|")[[1]][1])
    mainFactor_col_number <- which(names(x) == mainFactor)
    gene_df <- gene_df[, c(mainFactor_col_number, (1:ncol(gene_df))[-mainFactor_col_number])]
    if(is.null(calibratorLevel)){gene_df} else {
      gene_df <- gene_df %>% arrange(gene_df[[1]] != calibratorLevel, gene_df[[1]])
    }
    
    
    gene_df <- compute_wDCt(gene_df, numOfFactors, numberOfrefGenes, block,
                            set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40)
    
    gene_df[] <- lapply(gene_df, function(x) {
      if (is.factor(x)) as.character(x) else x
    })
    
    if (is.null(block)) {
      gene_df[seq_len(numOfFactors)] <- lapply(
        gene_df[seq_len(numOfFactors)],
        function(col) factor(col, levels = unique(col)))
    } else {
      gene_df[seq_len(numOfFactors + 1)] <- lapply(
        gene_df[seq_len(numOfFactors + 1)],
        function(col) factor(col, levels = unique(col)))
    }
    
    factors <- colnames(gene_df)[1:numOfFactors]
    default_model_formula <- NULL
    is_mixed_model <- FALSE
    is_singular <- FALSE
    
    
    if (user_defined_model) {
      if (is.character(model)) {
        formula_str <- paste("wDCt ~", model)
        formula_obj <- as.formula(formula_str)
      } else if (inherits(model, "formula")) {
        formula_obj <- model
      } else {
        stop("model must be either a character string or a formula object")
      }
      
      has_random_effects <- grepl("\\|", paste(deparse(formula_obj), collapse = " "))
      
      if (has_random_effects) {
        is_mixed_model <- TRUE
        lm_fit <- suppressMessages(lmerTest::lmer(formula_obj, data = gene_df, na.action = na.exclude))
        # singularity check
        is_singular <- lme4::isSingular(lm_fit, tol = 1e-4)
      } else {
        lm_fit <- lm(formula_obj, data = gene_df, na.action = na.exclude)
      }
      
      lm_formula <- paste(deparse(formula(lm_fit)), collapse = " ")
      ANOVA_table <- stats::anova(lm_fit)
      
    } else {
      
      if (is.null(block)) {
        formula_ANOVA <- as.formula(
          paste("wDCt ~", paste(factors, collapse = " * "))
        )
        default_model_formula <- deparse(formula_ANOVA)
        lm_fit <- lm(formula_ANOVA, data = gene_df, na.action = na.exclude)
        
      } else {
        formula_ANOVA <- as.formula(
          paste("wDCt ~", block, "+", paste(factors, collapse = " * "))
        )
        default_model_formula <- deparse(formula_ANOVA)
        lm_fit <- lm(formula_ANOVA, data = gene_df, na.action = na.exclude)
      }
      
      lm_formula <- paste(deparse(formula(lm_fit)), collapse = " ")
      ANOVA_table <- stats::anova(lm_fit)
    }
    
    ############################################################################
    specs2 <- stats::as.formula(paste("~ ", specs))
    pp1 <- suppressMessages(emmeans::emmeans(lm_fit, specs2, adjust = p.adj))    # mode = "satterthwaite" data = gene_df,
    cld_results <- multcomp::cld(pp1, alpha = 0.05, Letters = letters)
    if(is.null(calibratorLevel)) {calibratorLevel <- levels(factor(gene_df[[mainFactor]]))[1]}
    stats::confint(graphics::pairs(pp1), ref = calibratorLevel)
    contrast <- emmeans::contrast(pp1, "trt.vs.ctrl", ref = calibratorLevel, type = "response", adjust = "none")
    conf <- stats::confint(contrast)
    
    # bootstrap_model <- function(data, indices) {
    #   d <- data[indices, ]
    #   model_boot <- update(lm_fit, data = d)
    #   return(coef(model_boot))}
    # boot_result <- boot::boot(data = gene_df, statistic = bootstrap_model, R = 1000)
    # b <- boot::boot.ci(boot_result, type = "perc", conf = 0.95)
    
    conf$RE <- 2^(-conf$estimate)
    conf$LCL <- 2^(-conf$upper.CL)
    conf$UCL <- 2^(-conf$lower.CL)

    

    conf <- as.data.frame(conf)
    contrast <- as.data.frame(contrast)
    conf <- data.frame(conf, p.value = contrast$p.value)
    # bsLCL = 2^-b$percent[length(b$percent)],
    # bsUCL = 2^-b$percent[length(b$percent)-1]
    
    
    
    all_cols <- names(conf)
    stat_cols <- c("contrast", "estimate", "SE", "df", "lower.CL", "upper.CL", "RE", "LCL", "UCL", "p.value")    
    grouping_cols <- setdiff(all_cols, stat_cols)
    conf <- conf %>%
      mutate(contrast = as.character(contrast)) %>%
      group_by(across(all_of(grouping_cols))) %>% 
      group_modify(~ {
        ref_name <- sub(".*[ -]", "", .x$contrast[1])
        ref_name <- trimws(ref_name)
        ref_row <- .x[1, ]
        ref_row$contrast <- ref_name
        ref_row$p.value  <- 1
        ref_row$RE       <- 1
        cols_to_zero <- c("estimate", "SE", "df", "lower.CL", "upper.CL", "LCL",  "UCL") 
        existing_numeric <- intersect(cols_to_zero, names(ref_row))
        ref_row[existing_numeric] <- 0
        bind_rows(ref_row, .x)
      }) %>%
      ungroup()
    ############################################################################
    
    
    # 1. Parse specs to separate the primary factor from conditioning factors
    # Split "A | B * C" into primary ("A") and context ("B", "C")
    spec_parts <- strsplit(specs, "\\|")[[1]]
    primary_var <- trimws(all.vars(as.formula(paste("~", spec_parts[1]))))
    context_vars <- if(length(spec_parts) > 1) {
      trimws(all.vars(as.formula(paste("~", spec_parts[2]))))
    } else {
      NULL
    }
    
    if (!modelBased_se) {
      gene_df$residual <- gene_df$wDCt
    } else {
      gene_df$residual <- residuals(lm_fit, type = "response")
    }
    
    
    idRand <- detect_rep_id_random(model = model,
                                   numOfFactors = numOfFactors,
                                   block = block, x = x)
    
    
    # Add ID column if it's a random effect/paired design
    id_col_name <- NULL
    if (idRand || se.type == "paired.group") {
      id_col_idx <- min(tc) - 1
      id_col_name <- "pair_id"
      gene_df[[id_col_name]] <- x[[id_col_idx]]
    }
    # 3. Define the calculation logic per subgroup
    calculate_se <- function(sub_df, type, primary) {
      levs <- levels(factor(sub_df[[primary]]))
      if (length(levs) == 0) return(NULL)
      ref_lev <- levs[1]
      
      if (type == "single.group") {
        return(sub_df %>%
                 group_by(!!sym(primary)) %>%
                 summarise(se = stats::sd(residual, na.rm = TRUE) / sqrt(sum(!is.na(residual))), .groups = "drop"))
      } else {
        # For two.group or paired.group
        ref_vals <- sub_df[sub_df[[primary]] == ref_lev, ]
        
        results <- lapply(levs, function(l) {
          if (l == ref_lev) return(data.frame(primary = l, se = 0))
          comp_vals <- sub_df[sub_df[[primary]] == l, ]
          
          val <- tryCatch({
            if (type == "paired.group") {
              # Strict pairing by ID
              #paired <- merge(ref_vals, comp_vals, by = id_col_name) #####
              #if(nrow(paired) < 2) return(NA_real_) #####
              #stats::sd(paired$residual.x - paired$residual.y, na.rm = TRUE) / sqrt(nrow(paired)) #####
              stats::t.test(comp_vals$residual, ref_vals$residual, paired = TRUE)$stderr
            } else {
              # Independent samples t-test stderr
              stats::t.test(comp_vals$residual, ref_vals$residual, paired = FALSE)$stderr
            }
          }, error = function(e) NA_real_)
          
          data.frame(primary = l, se = val)
        })
        res_df <- do.call(rbind, results)
        colnames(res_df)[1] <- primary
        return(res_df)
      }
    }
    # 4. Execute grouped by context_vars
    if (is.null(context_vars)) {
      # Simple case: No pipe in specs
      se_final <- calculate_se(gene_df, se.type, primary_var)
    } else {
      # Nested case: Calculate within each combination of context_vars
      se_final <- gene_df %>%
        group_by(across(all_of(context_vars))) %>%
        group_modify(~ calculate_se(.x, se.type, primary_var)) %>%
        ungroup()
    }
    
    
    
    if ("pair_id" %in% names(gene_df)) {gene_df$pair_id <- NULL}
    
    
    
    
    sig <- .convert_to_character(conf$p.value)
    post_hoc_test <- data.frame(conf,
                                log2FC = -(conf$estimate),
                                sig = sig,
                                se = se_final$se)
    
    post_hoc_test$RE[post_hoc_test$pvalue == "NaN"] <- 0
    post_hoc_test$log2FC[post_hoc_test$pvalue == "NaN"] <- 0
    post_hoc_test$sig[post_hoc_test$pvalue == "NaN"] <- "ND"
    
    
    
    post_hoc_test$contrast <- sapply(
      strsplit(as.character(post_hoc_test$contrast), " - "),
      function(x) paste(x, collapse = " vs ")
    )
    
    post_hoc_test <- data.frame(
      post_hoc_test,
      Lower.se.RE = 2^(-post_hoc_test$estimate - post_hoc_test$se),
      Upper.se.RE = 2^(-post_hoc_test$estimate + post_hoc_test$se),
      Lower.se.log2FC = -post_hoc_test$estimate - post_hoc_test$se,
      Upper.se.log2FC = -post_hoc_test$estimate + post_hoc_test$se
    )
    
    
    post_hoc_test$gene <- gene_name

    
    res <- list(
      Final_data = gene_df,
      lm = lm_fit,
      ANOVA_table = ANOVA_table,
      Fold_Change = post_hoc_test,
      lm_formula = lm_formula,
      user_defined_model = user_defined_model,
      default_model_formula = default_model_formula,
      is_mixed_model = is_mixed_model,
      singular = is_singular
    )
    
    perGene[[gene_name]] <- res
    relativeExpression_list[[i]] <- post_hoc_test
  }
  
  relativeExpression <- do.call(rbind, relativeExpression_list)
  rownames(relativeExpression) <- NULL
  
  names(relativeExpression)[names(relativeExpression) == "estimate"] <- "ddCt"
  
  # desired_order <- c("gene", "contrast", "ddCt", "RE", "log2FC", "LCL", "UCL", "se",
  #                    "Lower.se.RE", "Upper.se.RE", "Lower.se.log2FC", "Upper.se.log2FC", 
  #                    "p.value", "sig")
  # relativeExpression <- relativeExpression[, desired_order]
  relativeExpression$SE <- NULL   
  relativeExpression$df <- NULL
  relativeExpression$lower.CL <- NULL
  relativeExpression$upper.CL <- NULL
  relativeExpression <- relativeExpression[, c("gene",
                                               setdiff(colnames(relativeExpression), c("gene", "p.value", "sig")),
                                               "p.value", "sig"), drop = FALSE]
  
  
  
  
  for (col in names(relativeExpression)) {
    if (is.numeric(relativeExpression[[col]])) {
      relativeExpression[[col]] <- round(relativeExpression[[col]], 5)
    }
  }
  
  
  
  cat("\nRelative Expression\n")
  print(relativeExpression)
  cat("\n")
  
  first_gene_res <- perGene[[1]]
  calibrator_level <- strsplit(first_gene_res$Fold_Change$contrast[1], " vs ")[[1]][1]
  cat(paste("The", calibrator_level, "level was used as calibrator.\n"))
  
  singular_genes <- names(perGene)[
    vapply(perGene, function(z) isTRUE(z$singular), logical(1))
  ]
  
  if (length(singular_genes) > 0) {
    warning(
      "Singular fit detected for the following genes:\n  ",
      paste(singular_genes, collapse = ", ")
    )
  }
  
  if (is.null(model) && length(perGene) > 0) {
    default_formula <- perGene[[1]]$default_model_formula
    if (!is.null(default_formula)) {
      cat("Note: Using default model for statistical analysis:", default_formula, "\n")
    }
  }
  
  
  invisible(list(
    perGene = perGene,
    relativeExpression = relativeExpression
  ))
}

#' Sample qPCR data: amplification efficiency 1
#'
#' A sample qPCR dataset for demonstrating efficiency calculation.
#'
#' @format A data frame with 21 observations and 4 variables:
#' \describe{
#'   \item{Dilutions}{Dilution factor}
#'   \item{C2H2.26}{Target gene 1}
#'   \item{C2H2.01}{Target gene 2}
#'   \item{GAPDH}{Reference gene}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_efficiency1"



			

#' Sample qPCR data: amplification efficiency 2
#'
#' A sample qPCR dataset for demonstrating efficiency calculation.
#'
#' @format A data frame with 12 observations and 5 variables:
#' \describe{
#'   \item{dilutions}{dilution factor}
#'   \item{ref_control}{reference gene in control condition}
#'   \item{ref_treatment}{reference gene in treatment condition}
#'   \item{target_control}{target gene in control condition}
#'   \item{target_treatment}{Reference gene in treatment condition}
#' }
#'
#' @source Yuan2006PMCBioinf
#' @keywords internal
"data_efficiency_Yuan2006PMCBioinf"



#' Sample data (one factor three levels)
#'
#' A sample dataset. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 9 observations and 6 variables:
#' \describe{
#'   \item{SA}{An experimental factor here called SA}
#'   \item{Rep}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_1factor"


#' Sample data (two factor)
#'
#' A sample dataset. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 18 observations and 7 variables:
#' \describe{
#'   \item{Genotype}{First experimental factor}
#'   \item{Drought}{Second experimental factor}
#'   \item{Rep}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' } 
#'
#' @source Not applicable
#' @keywords internal
"data_2factor"



#' Sample data in (two factor with blocking factor)
#'
#' A sample qPCR data set with blocking factor. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 18 observations and 8 variables:
#' \describe{
#'   \item{Type}{First experimental factor}
#'   \item{Concentration}{Second experimental factor}
#'   \item{block}{Second experimental factor}
#'   \item{Rep}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_2factorBlock"


#' Sample data (three factor)
#'
#' A sample dataset. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 36 observations and 8 variables:
#' \describe{
#'   \item{Type}{First experimental factor}
#'   \item{Conc}{Second experimental factor}
#'   \item{SA}{Third experimental factor}
#'   \item{Replicate}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_3factor"



#' Sample qPCR data (two different conditions)
#'
#' Sample qPCR data (two different conditions)
#'
#' @format A data frame with 6 observations and 10 variables:
#' \describe{
#'   \item{Condition}{Experimental conditions}
#'   \item{Rep}{Biological replicates}
#'   \item{C2H2_26}{amplification efficiency of C2H2_26 gene}
#'   \item{C2H2_26_Ct}{Ct values of C2H2_26 gene. Each is the mean of technical replicates}
#'   \item{C2H2_01}{Amplification efficiency of C2H2_01 gene}
#'   \item{C2H2_01_Ct}{Ct values of C2H2_01 gene. Each is the mean of technical replicates}
#'   \item{C2H2_12}{Amplification efficiency of C2H2_12 gene}
#'   \item{C2H2_12_Ct}{Ct values of C2H2_12 gene}
#'   \item{ref}{Amplification efficiency of ref gene}
#'   \item{ref_Ct}{Ct values of ref gene}
#' }
#'
#' @source University of Kurdistan
#' @keywords internal
"data_1factor_one_ref"


#' Sample qPCR data (two different conditions)
#'
#' Sample qPCR data (two different conditions)
#'
#' @format A data frame with 6 observations and 6 variables:
#' \describe{
#'   \item{Con}{Experimental conditions}
#'   \item{r}{Biological replicates}
#'   \item{target}{Amplification efficiency of target gene}
#'   \item{target_Ct}{Ct values of target gene}
#'   \item{Actin}{Amplification efficiency of reference gene}
#'   \item{Actin_Ct}{Ct values of reference gene}
#' }
#'
#' @source University of Kurdistan
#' @keywords internal
"data_1factor_one_ref_Eff"



#' Sample qPCR data (one target and two reference genes under two different conditions)
#'
#' Sample qPCR data (one target and two reference genes under two different conditions)
#'
#' @format A data frame with 6 observations and 8 variables:
#' \describe{
#'   \item{Condition}{Experimental conditions}
#'   \item{Rep}{Biological replicates}
#'   \item{DER5}{amplification efficiency of DER5 gene}
#'   \item{Ct_DER5}{Ct values of DER5 gene. Each is the mean of technical replicates}
#'   \item{Actin}{Amplification efficiency of Actin gene}
#'   \item{Ct_Actin}{Ct values of Actin gene. Each is the mean of technical replicates}
#'   \item{HPRT}{Amplification efficiency of HPRT gene}
#'   \item{Ct_HPRT}{Ct values of HPRT gene}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_1factor_Two_ref"

#' Sample data (with technical replicates)
#'
#' A sample data for calculating biological replicated. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 18 observations and 9 variables:
#' \describe{
#'   \item{Condition}{experimental factor}
#'   \item{biolrep}{biological replicate}
#'   \item{techrep}{technical replicates}
#'   \item{target}{Amplification efficiency of target gene}
#'   \item{Ct_target}{Ct of target gene}
#'   \item{ref}{Amplification efficiency of reference gene}
#'   \item{Ct_ref}{Ct of reference gene}
#' } 
#'
#' @source Not applicable
#' @keywords internal
"data_withTechRep"


#' Sample data (with technical replicates)
#'
#' A sample data for calculating biological replicated. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 72 observations and 8 variables:
#' \describe{
#'   \item{factor1}{experimental factor}
#'   \item{DS}{DS}
#'   \item{biolRep}{biological replicate}
#'   \item{techRep}{technical replicates}
#'   \item{APOE}{Amplification efficiency of APOE gene}
#'   \item{Ct_APOE}{Ct of APOE gene}
#'   \item{GAPDH}{Amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct of GAPDH gene}
#' }
#'
#' @source Lee et al, (2020) <doi:10.12688/f1000research.23580.2>
#' @keywords internal
"data_Lee_etal2020qPCR"


#' Repeated measure sample data
#'
#' A repeated measure sample data in which 6 individuals have been analysed.  In the "id" column, a unique number is assigned to each individual,  e.g. all the three number 1 indicate one individual.
#' samples are taken or measurements are scored over different time points (time column) from each individual.
#'
#' @format A data frame with 18 observations and 7 variables:
#' \describe{
#'   \item{treatment}{treatment}
#'   \item{time}{time course levels}
#'   \item{id}{experimental factor}
#'   \item{Target}{Amplification efficiency of target gene}
#'   \item{Target_Ct}{Ct of target gene}
#'   \item{Ref}{Amplification efficiency of reference gene}
#'   \item{Ref_Ct}{Ct of reference gene}
#' }
#' 
#' @source NA
#' @keywords internal
"data_repeated_measure_2"


#' Repeated measure sample data
#'
#' A repeated measure sample data in which 3 individuals have been analysed.  In the "id" column, a unique number is assigned to each individual, e.g. all the three number 1 indicate one individual.
#' samples are taken or measurements are scored over different time points (time column) from each individual.
#'
#' @format A data frame with 9 observations and 6 variables:
#' \describe{
#'   \item{time}{time course levels}
#'   \item{id}{experimental factor}
#'   \item{Target}{Amplification efficiency of target gene}
#'   \item{Ct_Target}{Ct of target gene}
#'   \item{Ref}{Amplification efficiency of reference gene}
#'   \item{Ct_Ref}{Ct of reference gene}
#' }
#' 
#' @source NA
#' @keywords internal
"data_repeated_measure_1"


#' Sample data in (two factor with blocking factor)
#'
#' A sample qPCR data set with blocking factor. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 18 observations and 8 variables:
#' \describe{
#'   \item{factor1}{First experimental factor}
#'   \item{factor2}{Second experimental factor}
#'   \item{Rep}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
#'   \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#'   \item{ref2}{Mean amplification efficiency of ref2 gene}
#'   \item{Ct_ref2}{Ct values of ref2 gene. Each is the mean of technical replicates}
#'   \item{ref3}{Mean amplification efficiency of ref3 gene}
#'   \item{Ct_ref3}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_2factor3ref"



#' Sample data in (two factor with blocking factor and 3 reference genes)
#'
#' A sample qPCR data set with blocking factor and 3 reference genes. Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' @format A data frame with 18 observations and 8 variables:
#' \describe{
#'   \item{Type}{First experimental factor}
#'   \item{Concentration}{Second experimental factor}
#'   \item{block}{blocking factor}
#'   \item{Rep}{Biological replicates}
#'   \item{PO}{Mean amplification efficiency of PO gene}
#'   \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
#'   \item{NLM}{Mean amplification efficiency of NLM gene}
#'   \item{Ct_NLM}{Ct values of NLM gene. Each is the mean of technical replicates}
#'   \item{ref1}{Mean amplification efficiency of ref1 gene}
#'   \item{Ct_ref1}{Ct values of ref1 gene. Each is the mean of technical replicates}
#'   \item{ref2}{Mean amplification efficiency of ref2 gene}
#'   \item{Ct_ref2}{Ct values of ref2 gene. Each is the mean of technical replicates}
#'   \item{ref3}{Mean amplification efficiency of ref3 gene}
#'   \item{Ct_ref3}{Ct values of GAPDH gene. Each is the mean of technical replicates}
#' }
#'
#' @source Not applicable
#' @keywords external
"data_2factorBlock3ref"


#' Sample data in (one factor with one reference gene)
#'
#' A sample qPCR data set with one experimental factor (condition)
#' and one reference gene. Each line belongs to a separate individual
#' (non-repeated measure experiment).
#'
#' @format A data frame with 24 observations and 6 variables:
#' \describe{
#'   \item{condition}{Experimental factor with two levels (control, treatment)}
#'   \item{rep}{Biological replicates}
#'   \item{target}{Mean amplification efficiency of target gene}
#'   \item{Ct_target}{Ct values of target gene. Each is the mean of technical replicates}
#'   \item{ref}{Mean amplification efficiency of reference gene}
#'   \item{Ct_ref}{Ct values of reference gene. Each is the mean of technical replicates}
#' }
#'
#' @source Yuan2006PMCBioinf
#' @keywords internal
"data_Yuan2006PMCBioinf"




#' Sample data in (two-group design with multiple target genes)
#'
#' A sample qPCR data set with two experimental groups (Control and Treatment)
#' and 18 target genes normalized to one reference gene (GAPDH).
#' Each line belongs to a separate individual (non-repeated measure experiment).
#' This dataset is suitable for two-sample t-test based expression analysis.
#'
#' @format A data frame with 8 observations and 38 variables:
#' \describe{
#'   \item{Condition}{Experimental group (Control, Treatment)}
#'   \item{Rep}{Biological replicates}
#'
#'   \item{ANGPT1, Ct.ANGPT1}{Efficiency and Ct values of ANGPT1 gene}
#'   \item{ANGPT2, Ct.ANGPT2}{Efficiency and Ct values of ANGPT2 gene}
#'   \item{CCL2, Ct.CCL2}{Efficiency and Ct values of CCL2 gene}
#'   \item{CCL5, Ct.CCL5}{Efficiency and Ct values of CCL5 gene}
#'   \item{CSF2, Ct.CSF2}{Efficiency and Ct values of CSF2 gene}
#'   \item{FGF2, Ct.FGF2}{Efficiency and Ct values of FGF2 gene}
#'   \item{IL1A, Ct.IL1A}{Efficiency and Ct values of IL1A gene}
#'   \item{IL1B, Ct.IL1B}{Efficiency and Ct values of IL1B gene}
#'   \item{IL6, Ct.IL6}{Efficiency and Ct values of IL6 gene}
#'   \item{IL8, Ct.IL8}{Efficiency and Ct values of IL8 gene}
#'   \item{PDGFA, Ct.PDGFA}{Efficiency and Ct values of PDGFA gene}
#'   \item{PDGFB, Ct.PDGFB}{Efficiency and Ct values of PDGFB gene}
#'   \item{TGFA, Ct.TGFA}{Efficiency and Ct values of TGFA gene}
#'   \item{TGFB, Ct.TGFB}{Efficiency and Ct values of TGFB gene}
#'   \item{TNF, Ct.TNF}{Efficiency and Ct values of TNF gene}
#'   \item{VEGFA, Ct.VEGFA}{Efficiency and Ct values of VEGFA gene}
#'   \item{VEGFB, Ct.VEGFB}{Efficiency and Ct values of VEGFB gene}
#'   \item{VEGFC, Ct.VEGFC}{Efficiency and Ct values of VEGFC gene}
#'   \item{GAPDH, Ct.GAPDH}{Efficiency and Ct values of GAPDH reference gene}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_ttest18genes"



#' Sample data in (repeated-measures design with two reference genes)
#'
#' A sample qPCR data set with repeated measurements over time,
#' a blocking factor, and two reference genes. The experiment includes
#' two treatments (untreated and treated) with repeated measures
#' on the same individuals (id).
#'
#' This dataset is suitable for repeated-measures mixed-model analysis
#' and normalization using multiple reference genes.
#'
#' @format A data frame with 18 observations and 10 variables:
#' \describe{
#'   \item{treatment}{Experimental treatment (untreated, treated)}
#'   \item{time}{Time point of measurement}
#'   \item{blk}{Blocking factor}
#'   \item{id}{Subject/individual identifier for repeated measures}
#'
#'   \item{Target}{Mean amplification efficiency of target gene}
#'   \item{Ct_Target}{Ct values of target gene}
#'
#'   \item{Ref1}{Mean amplification efficiency of reference gene 1}
#'   \item{Ct_Ref1}{Ct values of reference gene 1}
#'
#'   \item{Ref2}{Mean amplification efficiency of reference gene 2}
#'   \item{Ct_Ref2}{Ct values of reference gene 2}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_repeated_measure_3bLock"




#' Sample data in (multi-group design with 6 target genes)
#'
#' A sample qPCR data set with four experimental groups
#' (Uninjected, SNConly, DMSORPE, SNCRPE) and six target genes
#' normalized to one reference gene (Gapdh).
#' Each line belongs to a separate individual (non-repeated measure experiment).
#'
#' This dataset is suitable for t-test or one-way ANOVA based
#' expression analysis of multiple genes.
#'
#' @format A data frame with 18 observations and 16 variables:
#' \describe{
#'   \item{Treatment}{Experimental treatment group}
#'   \item{rep}{Biological replicates}
#'
#'   \item{Fn1, Ct.Fn1}{Efficiency and Ct values of Fn1 gene}
#'   \item{Col1a1, Ct.Col1a1}{Efficiency and Ct values of Col1a1 gene}
#'   \item{Acta2, Ct.Acta2}{Efficiency and Ct values of Acta2 gene}
#'   \item{TgfB, Ct.TgfB}{Efficiency and Ct values of TgfB gene}
#'   \item{Tnfa, Ct.Tnfa}{Efficiency and Ct values of Tnfa gene}
#'   \item{Mcp1, Ct.Mcp1}{Efficiency and Ct values of Mcp1 gene}
#'   \item{Gapdh, Ct.Gapdh}{Efficiency and Ct values of Gapdh reference gene}
#' }
#'
#' @source Not applicable
#' @keywords internal
"data_Heffer2020PlosOne"

#' Cleaning data and weighted delta Ct (wDCt) calculation
#'
#' The \code{compute_wDCt} function cleans the data and computes wDCt. This function is 
#' automatically applied to the expression analysis functions like \code{ANOVA_DDCt}, 
#' \code{TTEST_DDCt}, etc. So it should not be applied in advance of expression analysis functions.
#'
#' @param x A data frame containing experimental design columns, replicates (integer), target gene
#'   E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
#'   columns must be located at the end of the data frame. 
#' @param numOfFactors Integer. Number of experimental factor columns
#'   (excluding \code{rep} and optional \code{block}).
#' @param numberOfrefGenes Integer. Number of reference genes.
#' @param block Character or \code{NULL}. Name of the blocking factor column.
#' When a qPCR experiment is done in multiple qPCR plates, each plate is considered as a random block 
#' so that at least one replicate of each treatment and control is present 
#' on a plate.
#' @param set_missing_target_Ct_to_40 If \code{TRUE}, missing target gene Ct 
#'   values become 40; if \code{FALSE} (default), they become NA.
#' 
#' @importFrom stats setNames
#'
#' @details
#' The \code{compute_wDCt} function computes weighted delta Ct (wDCt) for the input data. 
#' Missing data can be denoted by NA in the input data frame. 
#' Values such as '0' and 'undetermined' (for any E and Ct) are
#' automatically converted to NA. For target genes, NA for E or Ct measurements cause returning NA for 
#' the corresponding delta Ct for that replicate (row). 
#' If there are more than one reference gene, NA in the place of the E or the Ct value cause
#' skipping that gene and remaining references are geometrically averaged.
#' The \code{compute_wDCt} function is automatically applied to the expression analysis
#' functions.
#' @return
#' The original data frame along with the weighted delta Ct column.
#' @export
#' 
#' @examples
#'            
#' data <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
#' data
#' compute_wDCt(x = data,
#'              numOfFactors = 2,
#'              numberOfrefGenes = 3,
#'              block = "block")
#' 



compute_wDCt <- function(x, 
                         numOfFactors,
                         numberOfrefGenes, 
                         block,
                         set_missing_target_Ct_to_40 = FALSE) {
  
  if (is.null(block)) {
    x[seq_len(numOfFactors)] <- lapply(
      x[seq_len(numOfFactors)],
      factor
    )
  } else {
    x[seq_len(numOfFactors + 1)] <- lapply(
      x[seq_len(numOfFactors + 1)],
      factor
    )
  }
  
  x <- .cleanup(x = x, numOfFactors = numOfFactors, numberOfrefGenes = numberOfrefGenes, block = block,
                set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40)
  
  stopifnot(numberOfrefGenes >= 1)
  nRef <- numberOfrefGenes
  nc   <- ncol(x)
  
  # Identify columns
  ref_E_cols  <- seq(nc - 2 * nRef + 1, nc, by = 2)
  ref_Ct_cols <- seq(nc - 2 * nRef + 2, nc, by = 2)
  
  target_E_col  <- ref_E_cols[1] - 2
  target_Ct_col <- ref_E_cols[1] - 1
  
  # Target term - handle NA/Inf
  target_term <- log2(x[[target_E_col]]) * x[[target_Ct_col]]
  # Convert Inf/-Inf to NA
  target_term[!is.finite(target_term)] <- NA_real_
  
  # Reference matrices
  E_mat  <- as.matrix(x[, ref_E_cols])
  Ct_mat <- as.matrix(x[, ref_Ct_cols])
  
  # Row-wise geometric mean of reference efficiencies (NA-aware)
  geoMeanE <- apply(E_mat, 1, function(z) {
    k <- sum(!is.na(z))
    if (k > 0) {
      # Check for zeros or negative values
      if (any(z <= 0, na.rm = TRUE)) {
        return(NA_real_)
      }
      prod(z, na.rm = TRUE)^(1 / k)
    } else {
      NA_real_
    }
  })
  
  # Reference term (Excel-consistent)
  ref_term <- numeric(nrow(x))
  for (r in seq_len(nrow(x))) {
    
    # Check if geoMeanE is valid (positive and finite)
    if (is.na(geoMeanE[r]) || geoMeanE[r] <= 0 || !is.finite(geoMeanE[r])) {
      ref_term[r] <- NA_real_
      next
    }
    
    logE <- log2(geoMeanE[r])
    
    # Check if logE is finite
    if (!is.finite(logE)) {
      ref_term[r] <- NA_real_
      next
    }
    
    tmp <- logE * Ct_mat[r, ]
    
    # Remove NA values
    tmp <- tmp[!is.na(tmp)]
    k   <- length(tmp)
    
    if (k > 0) {
      # Check if any tmp values are infinite
      if (any(!is.finite(tmp))) {
        ref_term[r] <- NA_real_
      } else {
        # Calculate geometric mean
        result <- prod(tmp)^(1 / k)
        # Check if result is finite
        ref_term[r] <- if (is.finite(result)) result else NA_real_
      }
    } else {
      ref_term[r] <- NA_real_
    }
  }
  
  # Final wDCt - handle cases where either term is NA
  x$wDCt <- target_term - ref_term
  # Convert any remaining Inf/-Inf to NA
  x$wDCt[!is.finite(x$wDCt)] <- NA_real_
  
  x
}

#' @title Delta Delta Ct pairwise comparisons using a fitted model
#'
#' @description
#' Performs ddCt expression analysis using a fitted model object produced by
#' \code{ANOVA_DCt()} or \code{ANOVA_DDCt()}.
#'
#' @details
#' The \code{Means_DDCt} function performs pairwise comparisons of relative expression values fo all combinations using
#' estimated marginal means derived from a fitted model.
#' For ANOVA models, relative expression values can be obtained for main effects,
#' interactions, and sliced (simple) effects.
#' For ANCOVA models returned by the \pkg{rtpcr} package, only simple
#' effects are supported.
#'
#' Internally, this function relies on the \pkg{emmeans} package to
#' compute marginal means and contrasts, which are then back-transformed
#' to fold change values using the ddCt framework.
#'
#' @author
#' Ghader Mirzaghaderi
#'
#' @export
#'
#' @import emmeans
#'
#' @param model
#' A fitted model object (typically an \code{lmer} or \code{lm} object)
#' created by \code{ANOVA_DCt()}, \code{ANOVA_DDCt()}.
#'
#' @param specs
#' A character string or character vector specifying the predictors or
#' combinations of predictors over which relative expression values are desired.
#' This argument follows the specification syntax used by
#' \code{emmeans::emmeans()} (e.g., \code{"Factor"},
#' \code{"Factor1 | Factor2"}).
#'
#' @param p.adj
#' Character string specifying the method for adjusting p-values.
#' See \code{\link[stats]{p.adjust}} for available options.
#'
#' @return
#' A data frame containing estimated relative expression values, confidence
#' intervals, p-values, and significance levels derived from the fitted
#' model.
#'
#' @examples
#' 
#' data <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))
#'
#' # Obtain a fitted model from ANOVA_DDCt
#' res <- ANOVA_DDCt(
#'   data,
#'   numOfFactors = 3,
#'   numberOfrefGenes = 1,
#'   specs = "Type",
#'   block = NULL)
#'
#' # Relative expression values for Type main effect
#' lm <- res$perGene$PO$lm
#' Means_DDCt(lm, specs = "Type")
#'
#' # Relative expression values for Concentration main effect
#' Means_DDCt(lm, specs = "Conc")
#'
#' # Relative expression values for Concentration sliced by Type
#' Means_DDCt(lm, specs = "Conc | Type")
#'
#' # Relative expression values for Concentration sliced by Type and SA
#' Means_DDCt(lm, specs = "Conc | Type * SA")
#' 


Means_DDCt <- function(model, 
                       specs, 
                       p.adj = "none"){
  
  if(any(c("lmerModLmerTest", "lm") %in% class(model)) == FALSE){
    stop(deparse(substitute(model)), " is not an accepted model object created by ANOVA_DDCt, qpcrANOVARE, or REPEATED_DDCt functions!")
  }
  
  f <- stats::as.formula(paste("pairwise ~", specs))
  base::suppressMessages(Pv <- emmeans(model, f, adjust = p.adj))
  base::suppressMessages(emm2 <- stats::confint(emmeans(model, f, adjust = p.adj)))
  emm2$contrasts$p.value <- as.data.frame(Pv$contrasts)$p.value
  emm2 <- as.data.frame(emm2$contrasts)
  emm2$estimate <- 2^emm2$estimate
  emm2$lower.CL <- 2^emm2$lower.CL
  emm2$upper.CL <- 2^emm2$upper.CL
  emm2$contrast <- as.character(emm2$contrast)
  emm2$contrast <- sapply(strsplit(emm2$contrast, " - "), function(x) paste(rev(x), collapse = " vs "))
  colnames(emm2)[which(names(emm2) == "lower.CL")] <- "LCL"
  colnames(emm2)[which(names(emm2) == "upper.CL")] <- "UCL"
  emm2$sig <- .convert_to_character(emm2$p.value)
  colnames(emm2)[which(names(emm2) == "estimate")] <- "RE"
  colnames(emm2)[which(names(emm2) == "FC")] <- "RE"
  return(emm2)
}

.convert_to_character <- function(numbers) {
  characters <- rep(" ", length(numbers))
  ok <- !is.na(numbers)
  characters[ok & numbers < 0.001] <- "***"
  characters[ok & numbers >= 0.001 & numbers < 0.01] <- "**"
  characters[ok & numbers >= 0.01  & numbers < 0.05] <- "*"
  characters[ok & numbers >= 0.05  & numbers < 0.1 ] <- "."
  return(characters)
}




.cleanup <- function(
    x,
    numOfFactors,
    numberOfrefGenes,
    block = NULL,
    set_missing_target_Ct_to_40 = FALSE  # Default FALSE = NA, TRUE = 40
) {
  
  x <- as.data.frame(x)
  
  # Set missing value based on parameter
  missing_value <- if (set_missing_target_Ct_to_40) 40 else NA
  
  # Number of non-gene columns
  n_meta <- numOfFactors + if (!is.null(block)) 2 else 1
  
  # Total gene columns
  gene_cols <- (n_meta + 1):ncol(x)
  
  if (length(gene_cols) %% 2 != 0) stop("Gene columns must be in E/Ct pairs")
  
  # Split into pairs
  gene_pairs <- split(gene_cols, ceiling(seq_along(gene_cols) / 2))
  
  # Reference vs target genes
  ref_pairs <- tail(gene_pairs, numberOfrefGenes)
  target_pairs <- utils::head(gene_pairs, length(gene_pairs) - numberOfrefGenes)
  
  # Helper: check invalid (NA or any string/non-numeric)
  is_invalid <- function(v) {
    is.na(v) | is.na(suppressWarnings(as.numeric(v)))
  }
  
  # Reference genes: NA for invalid values 
  for (pair in ref_pairs) {
    E_col <- pair[1]
    Ct_col <- pair[2]
    
    bad_E <- is_invalid(x[[E_col]])
    x[[E_col]][bad_E] <- NA
    x[[E_col]] <- as.numeric(x[[E_col]])
    
    bad_Ct <- is_invalid(x[[Ct_col]])
    x[[Ct_col]][bad_Ct] <- NA  # Always NA for reference genes
    x[[Ct_col]] <- as.numeric(x[[Ct_col]])
  }
  
  # Target genes - use parameter choice ----
  for (pair in target_pairs) {
    E_col  <- pair[1]
    Ct_col <- pair[2]
    
    bad_E <- is_invalid(x[[E_col]])
    x[[E_col]][bad_E] <- NA
    x[[E_col]] <- as.numeric(x[[E_col]])
    
    bad_Ct <- is_invalid(x[[Ct_col]])
    x[[Ct_col]][bad_Ct] <- missing_value  # NA or 40 based on parameter
    x[[Ct_col]] <- as.numeric(x[[Ct_col]])
  }
  
  return(x)
}






.wide_to_long <- function(df) {
  
  if (ncol(df) < 6) {
    stop("Data frame must contain at least 6 columns.")
  }
  
  # metadata (first two columns)
  meta <- df[, 1:2, drop = FALSE]
  
  # remaining columns (paired by position)
  data_cols <- df[, -(1:2), drop = FALSE]
  
  if (ncol(data_cols) %% 2 != 0) {
    stop("After the first two columns, remaining columns must be in pairs.")
  }
  
  n_pairs <- ncol(data_cols) / 2
  
  out <- do.call(
    rbind,
    lapply(seq_len(n_pairs), function(i) {
      
      e_col  <- data_cols[, 2*i - 1]
      ct_col <- data_cols[, 2*i]
      
      gene_name <- colnames(data_cols)[2*i - 1]
      
      data.frame(
        Condition = meta[[1]],
        Gene = gene_name,
        E = e_col,
        Ct = ct_col,
        stringsAsFactors = FALSE
      )
    })
  )
  
  rownames(out) <- NULL
  out[[1]] <- factor(out[[1]])
  out
}



.long_to_wide <- function(df) {
  
  if (ncol(df) < 4) {
    stop("Data frame must contain at least 4 columns: Condition, Gene, E, Ct")
  }
  
  # standardize first 4 columns internally
  tmp <- data.frame(
    Condition = df[[1]],
    Gene      = df[[2]],
    E         = df[[3]],
    Ct        = df[[4]],
    stringsAsFactors = FALSE
  )
  
  # replicate number within Condition C Gene
  tmp$Rep <- ave(
    seq_len(nrow(tmp)),
    tmp$Condition,
    tmp$Gene,
    FUN = seq_along
  )
  
  # reshape to wide
  wide <- reshape(
    tmp,
    idvar   = c("Condition", "Rep"),
    timevar = "Gene",
    direction = "wide"
  )
  wide[[1]] <- factor(wide[[1]])
  .cleanup(wide)
}



.geom_pub_cols <- function(col_width = 0.8,
                           err_width = 0.15,
                           fill_colors = NULL,
                           dodge_width = 0.8,
                           alpha = 1,
                           ...) {
  
  pos <- position_dodge(width = dodge_width)
  
  layers <- list(
    geom_col(width = col_width, position = pos, alpha = alpha, ...),
    geom_errorbar(aes(ymin = ymin, ymax = ymax), width = err_width, position = pos)
  )
  
  if (!is.null(fill_colors)) {
    layers <- c(layers, scale_fill_manual(values = fill_colors))
  }
  
  layers
}


.theme_pub <- function(base_size = 12,
                       base_family = "sans",
                       legend_position = "right",
                       ...) {
  
  theme_bw(base_size = base_size, base_family = base_family) %+replace%
    theme(
      panel.border = element_blank(),
      axis.line.x = element_line(color = "black"),
      axis.line.y = element_line(color = "black"),
      legend.position = legend_position,
      ...
    )
}






.ANOVA_DDCtx <- function(
    x,
    numOfFactors,
    numberOfrefGenes,
    mainFactor.column,
    block,
    mainFactor.level.order = NULL,
    p.adj = "none",
    analyseAllTarget = TRUE,
    model = NULL,
    set_missing_target_Ct_to_40 = FALSE,
    se.type = c("paired.group", "two.group", "single.group"),
    modelBased_se = TRUE
) {
  
  
  se.type <- match.arg(se.type)
  
  n <- ncol(x)
  nDesign <- if (is.null(block)) numOfFactors + 1 else numOfFactors + 2
  designCols <- seq_len(nDesign)
  nRefCols <- 2 * numberOfrefGenes
  refCols <- (n - nRefCols + 1):n
  targetCols <- setdiff(seq_len(n), c(designCols, refCols))
  
  if (length(targetCols) == 0 || length(targetCols) %% 2 != 0) {
    stop("The supplied arguments don't match the input data.")
  }
  
  
  detect_rep_id_random <- function(model, numOfFactors, block, x) {
    if (is.null(model)) return(FALSE)
    ftxt <- paste(deparse(formula(model)), collapse = " ")
    rep_id_col <- if (is.null(block)) numOfFactors + 1 else numOfFactors + 2
    rep_id_name <- colnames(x)[rep_id_col]
    grepl(paste0("\\|\\s*", rep_id_name, "\\b"), ftxt)
  }
  
  
  
  targetPairs <- split(targetCols, ceiling(seq_along(targetCols)/2))
  targetNames <- vapply(targetPairs, function(tc) colnames(x)[tc[1]], character(1))
  
  if (!isTRUE(analyseAllTarget)) {
    keep <- targetNames %in% analyseAllTarget
    if (!any(keep)) stop("None of the specified target genes were found in the data.")
    targetPairs <- targetPairs[keep]
    targetNames <- targetNames[keep]
  }
  
  perGene <- list()
  relativeExpression_list <- list()
  
  for (i in seq_along(targetPairs)) {
    
    tc <- targetPairs[[i]]
    gene_name <- targetNames[i]
    
    gene_df <- x[, c(designCols, tc, refCols), drop = FALSE]
    user_defined_model <- !is.null(model)
    
    gene_df <- gene_df[, c(mainFactor.column, (1:ncol(gene_df))[-mainFactor.column])]
    
    if (is.null(block)) {
      gene_df[seq_len(numOfFactors)] <- lapply(
        gene_df[seq_len(numOfFactors)],
        function(col) factor(col, levels = unique(col)))
    } else {
      gene_df[seq_len(numOfFactors + 1)] <- lapply(
        gene_df[seq_len(numOfFactors + 1)],
        function(col) factor(col, levels = unique(col)))
    }
    
    if (is.null(mainFactor.level.order)) {
      mainFactor.level.order <- unique(gene_df[,1])
      calibrator <- gene_df[,1][1]
    } else if (any(is.na(match(unique(gene_df[,1]), mainFactor.level.order)))) {
      stop("The `mainFactor.level.order` doesn't match main factor levels.")
    } else {
      gene_df <- gene_df[order(match(gene_df[,1], mainFactor.level.order)), ]
      calibrator <- gene_df[,1][1]
    }
    
    
    gene_df <- compute_wDCt(gene_df, numOfFactors, numberOfrefGenes, block,
                            set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40)
    
    gene_df[] <- lapply(gene_df, function(x) {
      if (is.factor(x)) as.character(x) else x
    })
    
    if (is.null(block)) {
      gene_df[seq_len(numOfFactors)] <- lapply(
        gene_df[seq_len(numOfFactors)],
        function(col) factor(col, levels = unique(col)))
    } else {
      gene_df[seq_len(numOfFactors + 1)] <- lapply(
        gene_df[seq_len(numOfFactors + 1)],
        function(col) factor(col, levels = unique(col)))
    }
    
    factors <- colnames(gene_df)[1:numOfFactors]
    default_model_formula <- NULL
    
    is_mixed_model <- FALSE
    is_singular <- FALSE
    
    
    if (user_defined_model) {
      
      if (is.character(model)) {
        formula_str <- paste("wDCt ~", model)
        formula_obj <- as.formula(formula_str)
      } else if (inherits(model, "formula")) {
        formula_obj <- model
      } else {
        stop("model must be either a character string or a formula object")
      }
      
      # has_random_effects <- grepl("\\|", as.character(formula_obj)[3])
      has_random_effects <- grepl("\\|", paste(deparse(formula_obj), collapse = " "))
      
      if (has_random_effects) {
        
        is_mixed_model <- TRUE
        lm_fit <- suppressMessages(lmerTest::lmer(formula_obj, data = gene_df, na.action = na.exclude))
        
        # singularity check
        is_singular <- lme4::isSingular(lm_fit, tol = 1e-4)
        
      } else {
        lm_fit <- lm(formula_obj, data = gene_df, na.action = na.exclude)
      }
      
      lm_formula <- paste(deparse(formula(lm_fit)), collapse = " ")
      ANOVA_table <- stats::anova(lm_fit)
      
    } else {
      
      if (is.null(block)) {
        formula_ANOVA <- as.formula(
          paste("wDCt ~", paste(factors, collapse = " * "))
        )
        default_model_formula <- deparse(formula_ANOVA)
        lm_fit <- lm(formula_ANOVA, data = gene_df, na.action = na.exclude)
        
      } else {
        formula_ANOVA <- as.formula(
          paste("wDCt ~", block, "+", paste(factors, collapse = " * "))
        )
        default_model_formula <- deparse(formula_ANOVA)
        lm_fit <- lm(formula_ANOVA, data = gene_df, na.action = na.exclude)
      }
      
      lm_formula <- paste(deparse(formula(lm_fit)), collapse = " ")
      ANOVA_table <- stats::anova(lm_fit)
    }
    
    
    pp1 <- suppressMessages(
      emmeans::emmeans(lm_fit, colnames(gene_df)[1],
                       adjust = p.adj))    # mode = "satterthwaite"      data = gene_df, 
    
    pp2 <- as.data.frame(graphics::pairs(pp1), adjust = p.adj)
    pp3 <- pp2[1:length(mainFactor.level.order)-1,]
    ci  <- as.data.frame(stats::confint(graphics::pairs(pp1)),   # , na.action = stats::na.pass
                         adjust = p.adj)[1:length(unique(gene_df[,1]))-1,]
    pp  <- cbind(pp3, lower.CL = ci$lower.CL, upper.CL = ci$upper.CL)
    
    
    
    
    
    
    
    
    if (!modelBased_se) {
      bwDCt <- gene_df$wDCt
    } else {
      bwDCt <- residuals(lm_fit, type = "response")
    }
    
    
    
    
    idRand <- detect_rep_id_random(model = model,
                                   numOfFactors = numOfFactors,
                                   block = block, x = x)
    
    se.type <- match.arg(se.type, c("single.group", "two.group", "paired.group"))
    
    grp_levels <- levels(factor(gene_df[[1]]))
    n_groups  <- length(grp_levels)
    ref_level <- grp_levels[1]
    
    
    #if (idRand) 
      if (se.type == "paired.group") {
        warning("paired se is only appropriate for paired groups or repeated measure experiments!")  
      # id column is immediately before E/Ct gene columns
        df_se <- data.frame(
          factor = gene_df[[1]],
          wDCt   = bwDCt
        )
        
        ref_vals <- df_se$wDCt[df_se$factor == ref_level]
        
        se <- data.frame(
          factor = grp_levels,
          se = sapply(grp_levels, function(g) {
            if (g == ref_level) return(0)
            grp_vals <- df_se$wDCt[df_se$factor == g]
            
            tryCatch(
              stats::t.test(grp_vals, ref_vals, paired = TRUE)$stderr,
              error = function(e) NA_real_
            )
          })
        )
      
    } else if (se.type == "single.group") {
        df_se <- data.frame(
          factor = gene_df[[1]],
          wDCt   = bwDCt
        )
        
        se <- dplyr::summarise(
          dplyr::group_by(df_se, factor),
          se = tryCatch(
            stats::t.test(wDCt)$stderr,
            error = function(e) NA_real_
          ),
          .groups = "drop"
        )
        
      } else if (se.type == "two.group") {
        df_se <- data.frame(
          factor = gene_df[[1]],
          wDCt   = bwDCt
        )
        
        ref_vals <- df_se$wDCt[df_se$factor == ref_level]
        
        se <- data.frame(
          factor = grp_levels,
          se = sapply(grp_levels, function(g) {
            if (g == ref_level) return(0)
            grp_vals <- df_se$wDCt[df_se$factor == g]
            
            tryCatch(
              stats::t.test(grp_vals, ref_vals, paired = FALSE)$stderr,
              error = function(e) NA_real_
            )
          })
        )
      }
    
    
    sig <- .convert_to_character(pp$p.value)
    post_hoc_test <- data.frame(
      contrast = pp$contrast,
      ddCt = - pp$estimate,
      RE = 1/(2^-(pp$estimate)),
      log2FC = log2(1/(2^-(pp$estimate))),
      pvalue = pp$p.value,
      sig = sig,
      LCL = 1/(2^-pp$lower.CL),
      UCL = 1/(2^-pp$upper.CL),
      se = se$se[-1]
    )
    #post_hoc_test$sig[post_hoc_test$RE < 0.001] <- "ND"
    post_hoc_test$RE[post_hoc_test$pvalue == "NaN"] <- 0
    post_hoc_test$log2FC[post_hoc_test$pvalue == "NaN"] <- 0
    post_hoc_test$sig[post_hoc_test$pvalue == "NaN"] <- "ND"
    
    
    reference <- data.frame(
      contrast = mainFactor.level.order[1],
      ddCt = 0, RE = 1, log2FC = 0,
      pvalue = 1, sig = " ",
      LCL = 0, UCL = 0,
      se = se$se[1])
    
    tableC <- rbind(reference, post_hoc_test)
    
    tableC$contrast <- sapply(
      strsplit(as.character(tableC$contrast), " - "),
      function(x) paste(rev(x), collapse = " vs ")
    )
    
    tableC <- data.frame(
      tableC,
      Lower.se.RE = 2^(log2(tableC$RE) - tableC$se),
      Upper.se.RE = 2^(log2(tableC$RE) + tableC$se),
      Lower.se.log2FC = log2(tableC$RE) - tableC$se,
      Upper.se.log2FC = log2(tableC$RE) + tableC$se
    )
    
    
    tableC$gene <- gene_name
    
    res <- list(
      Final_data = gene_df,
      lm = lm_fit,
      ANOVA_table = ANOVA_table,
      Fold_Change = tableC,
      lm_formula = lm_formula,
      user_defined_model = user_defined_model,
      default_model_formula = default_model_formula,
      is_mixed_model = is_mixed_model,
      singular = is_singular
    )
    
    perGene[[gene_name]] <- res
    relativeExpression_list[[i]] <- tableC
  }
  
  relativeExpression <- do.call(rbind, relativeExpression_list)
  rownames(relativeExpression) <- NULL
  
  relativeExpression <- relativeExpression[, c(
    "gene","contrast","ddCt","RE","log2FC",
    "LCL","UCL","se",
    "Lower.se.RE","Upper.se.RE",
    "Lower.se.log2FC","Upper.se.log2FC",
    "pvalue","sig"
  )]
  
  
  for (col in names(relativeExpression)) {
    if (is.numeric(relativeExpression[[col]])) {
      relativeExpression[[col]] <- round(relativeExpression[[col]], 5)
    }
  }
  
  
  
  
  # cat("\nRelative Expression\n")
  # print(relativeExpression)
  # cat("\n")
  
  first_gene_res <- perGene[[1]]
  calibrator_level <- strsplit(first_gene_res$Fold_Change$contrast[1], " vs ")[[1]][1]
  cat(paste("The", calibrator_level, "level was used as calibrator.\n"))
  
  singular_genes <- names(perGene)[
    vapply(perGene, function(z) isTRUE(z$singular), logical(1))
  ]
  
  if (length(singular_genes) > 0) {
    warning(
      "Singular fit detected for the following genes:\n  ",
      paste(singular_genes, collapse = ", ")
    )
  }
  
  if (is.null(model) && length(perGene) > 0) {
    default_formula <- perGene[[1]]$default_model_formula
    if (!is.null(default_formula)) {
      cat("Note: Using default model for statistical analysis:", default_formula, "\n")
    }
  }
  
  
  invisible(list(
    perGene = perGene,
    relativeExpression = relativeExpression
  ))
}

#' @title Amplification efficiency statistics and standard curves
#'
#' @description
#' The \code{efficiency} function calculates amplification efficiency (E)
#' and related statistics, including slope and coefficient of determination
#' (R\eqn{^2}), and generates standard curves for qPCR assays.
#'
#' @details
#' Amplification efficiency is estimated from standard curves generated by
#' regressing Ct values against the logarithm of template dilution.
#' For each gene, the function reports the slope of the standard curve,
#' amplification efficiency (E), and R\eqn{^2} as a measure of goodness of fit.
#' The function also provides graphical visualization of the standard curves.
#' 
#' @import emmeans
#'
#' @author Ghader Mirzaghaderi
#'
#' @export
#'
#' @import tidyr
#' @import dplyr
#' @import reshape2
#' @import ggplot2
#' @import purrr
#'
#' @param df
#' A data frame containing dilution series and corresponding Ct values.
#' The first column should represent dilution levels, and the remaining
#' columns should contain Ct values for different genes.
#' @param base_size font size
#' @param legend_position legend position
#' @param ... Additional ggplot2 layer arguments
#'
#' @return
#' A list with the following components:
#' \describe{
#'   \item{efficiency}{A data frame containing slope, amplification efficiency (E),
#'   and R\eqn{^2} statistics for each gene.}
#'   \item{Slope_compare}{A table comparing slopes between genes.}
#'   \item{plot}{A \code{ggplot2} object showing standard curves for all genes.}
#' }
#'
#' @examples
#' 
#' # Load example efficiency data
#' data <- read.csv(system.file("extdata", "data_efficiency1.csv", package = "rtpcr"))
#'
#' # Calculate amplification efficiency and generate standard curves
#' efficiency(data)
#' 
#' 
#' ef <- read.csv(system.file("extdata", "data_efficiency_Yuan2006PMCBioinf.csv", package = "rtpcr"))
#' efficiency(ef)
#' 


efficiency <- function(df,
                       base_size = 12,
                       legend_position = c(0.2, 0.2),
                       ...) {

  
  # renaming the first column
  colnames(df)[1] <- "dilutions"
  dilutions <- df$dilutions
  # Fit the linear regressions and extract the slope and R-squared
  results <- df %>%
    select(-dilutions) %>%
    map_df(~{
      model <- lm(. ~ log10(dilutions))
      Slope <- coef(model)[2]
      R2 <- summary(model)$r.squared
      E <- 10^(-1/coef(model)[2])
      data.frame(Slope, R2, E)
    })
  
  # Add the column names to the results data frame
  results <- cbind(gene = colnames(df)[2:ncol(df)], results)
  colnames(results) <- c("Gene", "Slope", "R2", "E")
  rownames(results) <- NULL
  
  Ct <- df[,2]
  p <- ggplot(data = df) + 
    geom_point(aes(y = Ct, x = log10(dilutions))) +
    geom_smooth(data = df,aes(x = log10(dilutions), y = Ct), formula = y ~ x,
                method = "lm", se = F)

  

  
  # COMPAIRING SLOPES
  # making a long format data
  if(ncol(df) > 2){
    e <- melt(df, id = "dilutions")
    
    dilutions <- e$dilutions
    value <- e$value
    
    lm <- lm(value ~ log10(dilutions) * variable, data = e)
    slopes <- emtrends(lm, pairwise ~ variable, var = "log10(dilutions)")
    

    fits <- lapply(df[,-1], function(x) lm(x ~ log10(df[,1])))
    mdat <- melt(df, id = "dilutions")
    Ct <- mdat$value
    Gene <- mdat$variable
    variable <- mdat$Gene
    p <- ggplot(data = mdat) + 
      geom_smooth(data = mdat,aes(x = log10(dilutions), y = Ct, color = Gene), formula = y ~ x,
                  method = "lm", se = F) +
      geom_point(aes(y = Ct, x = log10(dilutions), colour = Gene, shape = Gene), size = 2) +
      theme_bw(base_size = base_size) +
      theme(
          panel.border = element_rect(color = "black"),
          axis.line.x = element_line(color = "black"),
          axis.line.y = element_line(color = "black"),
          legend.background = element_rect(fill = "transparent"),
          legend.position = legend_position,
          axis.text.x = element_text(size = base_size, color = "black"),
          axis.text.y = element_text(size = base_size,color = "black"),
          legend.text = element_text(colour = "black", size = base_size),
          ...
        )
    }
  


  if(ncol(df) == 2){
    res <- list(Efficiency = results, plot = p)
  } else {
    res <- list(Efficiency = results, Slope_compare = slopes, plot = p)
  }
   return(res)
}

#' @title Bar plot of gene(s) expression for 1-, 2-, or 3-factor experiments
#'
#' @description
#' Creates a bar plot of relative gene expression (fold change) values
#' from 1-, 2-, or 3-factor experiments, including error bars and statistical
#' significance annotations.
#'
#' @author
#' Ghader Mirzaghaderi
#'
#' @export
#'
#' @import tidyr
#' @import dplyr
#' @import reshape2
#' @import ggplot2
#'
#' @param data Data frame containing expression results
#' @param x_col Character. Column name for x-axis
#' @param y_col Character. Column name for bar height
#' @param Lower.se_col Character. Column name for lower SE
#' @param Upper.se_col Character. Column name for upper SE
#' @param group_col Character. Column name for grouping bars (optional)
#' @param facet_col Character. Column name for faceting (optional)
#' @param letters_col Character. Column name for significance letters (optional)
#' @param letters_d Numeric. Vertical offset for letters (default \code{0.2})
#' @param col_width Numeric. Width of bars (default \code{0.8})
#' @param err_width Numeric. Width of error bars (default \code{0.15})
#' @param dodge_width Numeric. Width of dodge for grouped bars (default \code{0.8})
#' @param fill_colors Optional vector of fill colors to change the default colors
#' @param color Optional color for the bar outline
#' @param alpha Numeric. Transparency of bars (default \code{1})
#' @param base_size Numeric. Base font size for theme (default \code{12})
#' @param legend_position Character or numeric vector. Legend position (default \code{right})
#' @param removeCalibratorCols = NULL or remove Calibrator Cols
#' @param removeCalibratorText = NULL or remove Calibrator text
#' @param ... Additional ggplot2 layer arguments
#' 
#' @return ggplot2 plot object
#' 
#' @examples
#' data <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
#' 
#' res <- ANOVA_DDCt(x = data,
#'   numOfFactors = 2,
#'   numberOfrefGenes = 3,
#'   block = "block",
#'   specs = "Concentration",
#'   p.adj = "none")
#' 
#' df <- res$relativeExpression
#' 
#' p1 <- plotFactor(
#'   data = df,
#'   x_col = "contrast",
#'   y_col = "RE",
#'   group_col = "gene",
#'   facet_col = "gene",
#'   Lower.se_col = "Lower.se.RE",
#'   Upper.se_col = "Upper.se.RE",
#'   letters_col = "sig",
#'   letters_d = 0.2,
#'   alpha = 1,
#'   col_width = 0.7,
#'   dodge_width = 0.7,
#'   base_size = 14, 
#'   legend_position = "none")
#' 
#' p1
#' 
#' 
#' data2 <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))
#' #Perform analysis first
#' res <- ANOVA_DCt(
#'   data2,
#'   numOfFactors = 3,
#'   numberOfrefGenes = 1,
#'   block = NULL)
#'   
#' df <- res$relativeExpression
#' # Generate three-factor bar plot
#'  p <- plotFactor(
#'   df,
#'   x_col = "SA",       
#'   y_col = "log2FC",       
#'   group_col = "Type",   
#'   facet_col = "Conc",    
#'   Lower.se_col = "Lower.se.log2FC",
#'   Upper.se_col = "Upper.se.log2FC",
#'   letters_col = "sig",
#'   letters_d = 0.3,
#'   col_width = 0.7, 
#'   dodge_width = 0.7,
#'   #fill_colors = c("blue", "brown"),
#'   color = "black",
#'   base_size = 14, 
#'   alpha = 1,
#'   legend_position = c(0.1, 0.2))
#' p
#'   
plotFactor <- function(data,
                       x_col,
                       y_col,
                       Lower.se_col,
                       Upper.se_col,
                       group_col = NULL,
                       facet_col = NULL,
                       letters_col = NULL,
                       letters_d = 0.2,
                       col_width = 0.8,
                       err_width = 0.15,
                       dodge_width = 0.8,
                       fill_colors = NULL,
                       color = NA,
                       alpha = 1,
                       base_size = 12,
                       legend_position = "right",
                       removeCalibratorCols = FALSE,
                       removeCalibratorText = FALSE,
                       ...) {
  

  if (removeCalibratorCols) {
    if ("contrast" %in% colnames(data)) {
      data <- data[grepl(" ", data$contrast), ]
    }
  }
  
  if (removeCalibratorText) {
    if ("contrast" %in% colnames(data)) {
      data$contrast <- gsub("\\s.*", "", as.character(data$contrast))
    }
  }
  
  # required columns
  required_cols <- c(x_col, y_col, Lower.se_col, Upper.se_col)
  if (!is.null(group_col)) required_cols <- c(required_cols, group_col)
  if (!is.null(facet_col)) required_cols <- c(required_cols, facet_col)
  
  if (!all(required_cols %in% colnames(data))) {
    stop("One or more specified columns do not exist in `data`.")
  }
  
  if (!is.null(letters_col) && !letters_col %in% colnames(data)) {
    stop("`letters_col` does not exist in `data`.")
  }
  
  # add error columns
  data$ymin <- data[[Lower.se_col]]
  data$ymax <- data[[Upper.se_col]]
  
  if (!is.null(letters_col)) {
    data[[letters_col]] <- as.character(data[[letters_col]])
  }
  
  
  # To factor:
  data[] <- lapply(data, function(data) {
    if (is.character(data)) factor(data, levels = unique(data)) else data
  })

  
  # 1-factor plot
  if (is.null(group_col) && is.null(facet_col)) {
    p <- ggplot(data, aes(x = .data[[x_col]], y = .data[[y_col]])) +
      geom_col(width = col_width, fill = fill_colors[1] %||% "grey40", alpha = alpha, color = color %||% NULL, ...)
    p <- p + geom_errorbar(aes(ymin = ymin, ymax = ymax), width = err_width)
    
    if (!is.null(letters_col)) {
      p <- p + geom_text(aes(
        label = .data[[letters_col]],
        y = ifelse(.data[[y_col]] < 0, ymin - letters_d, ymax + letters_d)
      ))
    }
    
  } else {
    # 2- or 3-factor plot
    p <- ggplot(data, aes(
      x = .data[[x_col]],
      y = .data[[y_col]],
      fill = .data[[group_col]]
    )) +
      .geom_pub_cols(
        col_width = col_width,
        err_width = err_width,
        fill_colors = fill_colors,
        dodge_width = dodge_width,
        alpha = alpha,
        color = color,
        ...
      )
    
    if (!is.null(facet_col)) {
      p <- p + facet_wrap(vars(.data[[facet_col]]))
    }
    
    if (!is.null(letters_col)) {
      pos <- position_dodge(width = dodge_width)
      p <- p + geom_text(aes(
        label = .data[[letters_col]],
        y = ifelse(.data[[y_col]] < 0, ymin - letters_d, ymax + letters_d)
      ), position = pos, ...)
    }
  }
  
  if(y_col == "log2FC" && any(data$log2FC < 0)){
    p <- p + scale_y_continuous(expand = expansion(mult = c(0.05, 0.05)))
  } else {
    p <- p + scale_y_continuous(expand = expansion(mult = c(0, 0.05)))
  }
  p + 
    .theme_pub(base_size = base_size, legend_position = legend_position) +
    xlab(NULL) +
    theme(axis.text.x = element_text(size = base_size, color = "black", angle = 45, hjust = 1),
          axis.text.y = element_text(size = base_size,color = "black", angle = 0),
          panel.border = element_rect(color = "black", linewidth = 1),
          axis.line.y = element_line(linewidth = 0),
          axis.line.x = element_line(linewidth = 0),
          legend.text = element_text(colour = "black", size = base_size),
          legend.background = element_rect(fill = "transparent"),
          strip.background = element_blank(),            # removes the faceting gray background
          strip.text = element_text(size = base_size))   # keeps the text visible
}

#' @title 
#' Converts a 4-column qPCR long data format to wide format
#' @description
#' Converts a 4-column (Condition, gene, Efficiency, Ct) qPCR long data format to wide format
#' @details
#' Converts a 4-column (Condition, gene, Efficiency, Ct) qPCR long data format to wide format
#' @author
#' Ghader Mirzaghaderi
#' @export
#' @param x 
#' a 4-column (Condition, gene, Efficiency, Ct) qPCR long data
#' @return
#'  A wide qPCR data frame
#'  
#' @examples 
#' 
#' df <- read.table(header = TRUE, text = "
#' Condition	Gene	E	Ct
#' control	C2H2-26	1.8	31.26
#' control	C2H2-26	1.8	31.01
#' control	C2H2-26	1.8	30.97
#' treatment	C2H2-26	1.8	32.65
#' treatment	C2H2-26	1.8	32.03
#' treatment	C2H2-26	1.8	32.4
#' control	C2H2-01	1.75	31.06
#' control	C2H2-01	1.75	30.41
#' control	C2H2-01	1.75	30.97
#' treatment	C2H2-01	1.75	28.85
#' treatment	C2H2-01	1.75	28.93
#' treatment	C2H2-01	1.75	28.9
#' control	C2H2-12	2	28.5
#' control	C2H2-12	2	28.4
#' control	C2H2-12	2	28.8
#' treatment	C2H2-12	2	27.9
#' treatment	C2H2-12	2	28
#' treatment	C2H2-12	2	27.9
#' control	ref	1.9	28.87
#' control	ref	1.9	28.42
#' control	ref	1.9	28.53
#' treatment	ref	1.9	28.31
#' treatment	ref	1.9	29.14
#' treatment	ref	1.9	28.63")
#' 
#' long_to_wide(df)


long_to_wide <- function(x) {
  
  numOfFactors <- 1
  block <- NULL
  df <- x
  
  if (ncol(df) < 4) {
    stop("Data frame must contain at least 4 columns: Condition, Gene, E, Ct")
  }
  
  # standardize first 4 columns internally
  tmp <- data.frame(
    Condition = df[[1]],
    Gene      = df[[2]],
    E         = df[[3]],
    Ct        = df[[4]],
    stringsAsFactors = FALSE
  )
  
  # replicate number within Condition × Gene
  tmp$Rep <- ave(
    seq_len(nrow(tmp)),
    tmp$Condition,
    tmp$Gene,
    FUN = seq_along
  )
  
  # reshape to wide
  wide <- reshape(
    tmp,
    idvar   = c("Condition", "Rep"),
    timevar = "Gene",
    direction = "wide"
  )
  wide[[1]] <- factor(wide[[1]])
  x <- wide
  
  # define factor columns
  if (is.null(block)) {
    x[seq_len(numOfFactors)] <- lapply(x[seq_len(numOfFactors)], factor)
  } else {
    x[seq_len(numOfFactors + 1)] <- lapply(x[seq_len(numOfFactors + 1)], factor)
  }
  
  # pairwise cleanup (from last to first)
  n <- ncol(x)
  
  for (i in seq(from = n, to = 2, by = -2)) {
    
    colA <- x[[i - 1]]
    colB <- x[[i]]
    
    # skip if either column is a factor
    if (is.factor(colA) || is.factor(colB)) next
    
    # identify invalid cells
    badA <- suppressWarnings(is.na(as.numeric(colA))) | colA == 0
    badB <- suppressWarnings(is.na(as.numeric(colB))) | colB == 0
    
    # cross-invalidate within the pair
    colA[badB] <- NA
    colB[badA] <- NA
    
    x[[i - 1]] <- colA
    x[[i]]     <- colB
  }
  
  # column-wise cleanup (original behavior)
  x[] <- lapply(x, function(col) {
    
    if (is.factor(col)) return(col)
    
    if (is.character(col)) {
      col[col %in% c("Undetermined", "undetermined")] <- NA
      suppressWarnings(col <- as.numeric(col))
    }
    
    if (is.numeric(col)) {
      col[col == 0] <- NA
    }
    
    col
  })
  
  x
}

#' @title Combine multiple ggplot objects into a single layout
#'
#' @description
#' The \code{multiplot} function arranges multiple \code{ggplot2} objects
#' into a single plotting layout with a specified number of columns.
#'
#' @details
#' Multiple \code{ggplot2} objects can be provided either as separate
#' arguments via \code{...}.
#' The function uses the \code{grid} package to control the layout.
#'
#' @author
#' Pedro J. (adapted from \url{https://gist.github.com/pedroj/ffe89c67282f82c1813d})
#'
#' @export
#'
#' @import grid
#'
#' @param ...
#' One or more \code{ggplot2} objects.
#'
#'
#' @param cols
#' Integer specifying the number of columns in the layout.
#'
#' @return
#' A grid object displaying multiple plots arranged in the specified layout.
#'
#' @examples
#'
#' # Example using output from TTEST_DDCt
#' data1 <- read.csv(system.file("extdata", "data_ttest18genes.csv", package = "rtpcr"))
#' out <- TTEST_DDCt(
#'   data1,
#'   paired = FALSE,
#'   var.equal = TRUE,
#'   numberOfrefGenes = 1)
#'   
#' p1 <- plotFactor(out, 
#'   x_col = "gene",
#'   y_col = "log2FC", 
#'   Lower.se_col = "Lower.se.log2FC",
#'   Upper.se_col = "Upper.se.log2FC", 
#'   letters_col = "sig")
#'   
#' p2 <- plotFactor(out, 
#'   x_col = "gene",
#'   y_col = "RE", 
#'   Lower.se_col = "Lower.se.RE",
#'   Upper.se_col = "Upper.se.RE", 
#'   letters_col = "sig")
#'   
#'   
#'
#' # Example using output from ANOVA_DCt
#' data2 <- read.csv(system.file("extdata", "data_1factor.csv", package = "rtpcr"))
#' out2 <- ANOVA_DCt(
#'   data2,
#'   numOfFactors = 1,
#'   numberOfrefGenes = 1,
#'   block = NULL)
#'   
#' df <- out2$relativeExpression
#'
#' p3 <- plotFactor(
#'   df,
#'   x_col = "SA",
#'   y_col = "RE",
#'   Lower.se_col = "Lower.se.RE",
#'   Upper.se_col = "Upper.se.RE",
#'   letters_col = "sig",
#'   letters_d = 0.1,
#'   col_width = 0.7,
#'   err_width = 0.15,
#'   fill_colors = "skyblue",
#'   alpha = 1,
#'   base_size = 14)
#'
#' # Combine plots into a single layout
#' multiplot(p1, p2, cols = 2)
#'
#' multiplot(p1, p3, cols = 2)



multiplot <- function(..., cols=1) {
  
  # Make a list from the ... arguments
  plots <- c(list(...))
  
  numPlots = length(plots)
  
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols), byrow = TRUE)
  
  if (numPlots == 1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

#' Delta Ct ANOVA analysis with optional model specification
#'
#' Performs Delta Ct (dCt) analysis of the data from a factorial experiment 
#' with support for both fixed- and mixed-effect models. Per-gene statistical 
#' analysis and grouping is also performed.
#'
#' @details
#' The function performs ANOVA analysis on weighted delta Ct (wDCt) values and returns 
#' variance components along with an expression table containing:
#' \itemize{
#' \item \code{gene}: Name of target genes
#' \item Factor columns: Experimental design factors
#' \item \code{dCt}: Mean weighted delta Ct for each treatment combination
#' \item \code{RE}: Relative expression = fold change = 2^-dCt
#' \item \code{log2FC}: log2 of RE
#' \item \code{LCL}: 95\% lower confidence level
#' \item \code{UCL}: 95\% upper confidence level
#' \item \code{se}: Standard error
#' \item \code{Lower.se.RE}: Lower limit error bar for RE
#' \item \code{Upper.se.RE}: Upper limit error bar for RE
#' \item \code{Lower.se.log2FC}: Lower limit error bar for log2FC
#' \item \code{Upper.se.log2FC}: Upper limit error bar for log2FC
#' \item \code{sig}: Per-gene significance grouping letters
#' }
#'
#'
#' @param x The input data frame containing experimental design columns, target gene
#'   E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
#'   columns must be located at the end of the data frame. See "Input data 
#'   structure" in vignettes for details about data structure.
#' @param numOfFactors Integer. Number of experimental factor columns 
#'   (excluding \code{rep} and optional \code{block}).
#' @param numberOfrefGenes Integer. Number of reference genes. Each reference gene
#'   must be represented by two columns (E and Ct).
#' @param block Character. Block column name or \code{NULL}. When a qPCR experiment 
#'   is done in multiple qPCR plates, variation resulting from the plates may 
#'   interfere with the actual amount of gene expression. One solution is to 
#'   conduct each plate as a randomized block so that at least one replicate of 
#'   each treatment and control is present on a plate. Block effect is usually 
#'   considered as random and its interaction with any main effect is not considered.
#'   Note: This parameter is ignored if \code{model} is provided.
#' @param alpha Statistical level for comparisons (default: 0.05).
#' @param p.adj Method for p-value adjustment. See \code{\link[stats]{p.adjust}}.
#' @param analyseAllTarget Logical or character. If \code{TRUE} (default), all 
#'   detected target genes are analysed. Alternatively, a character vector 
#'   specifying the names (names of their Efficiency columns) of target genes 
#'   to be analysed.
#' @param model Optional model formula. If provided, this overrides the automatic formula (CRD or RCBD 
#'   based on \code{block} and \code{numOfFactors}). The formula uses 
#'   \code{wDCt} as the response variable. 
#'   For mixed models, random effects can be defined using \code{lmer} syntax 
#'   (e.g., \code{"wDCt ~ Treatment + (1|Block)"}). When using \code{model}, 
#'   the \code{block} and \code{numOfFactors} arguments are ignored for model 
#'   specification, but still used for data structure identification.
#' @param modelBased_se Logical. If \code{TRUE} (default), standard errors are  
#' calculated from model-based residuals. If \code{FALSE}, standard errors are calculated directly from the observed 
#' \code{wDCt} values within each treatment group according to the selected \code{se.type}.  
#' For single factor data, both methods are the same. It is recommended to use \code{modelBased_se = TRUE} (default).
#' @param set_missing_target_Ct_to_40 If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA. 
#'
#' @return
#' An object containing expression tables, lm/lmer models, ANOVA tables, 
#' residuals, and raw data for each gene:
#' \describe{
#' \item{\code{relativeExpression}}{dCt expression table for all treatment 
#'   combinations along with per-gene statistical grouping}
#' \item{\code{perGene}}{Nested list containing detailed results for each target gene:
#'   \itemize{
#'   \item \code{ANOVA_table}: Full factorial ANOVA table
#'   \item \code{lm}: lm/lmer model for factorial design
#'   \item \code{Final_data}: Processed data with wDCt values
#'   \item \code{resid(object$perGene$gene_name$lm)}: Residuals
#'   }}
#' }
#' 
#' @examples
#' # Default usage with fixed effects
#' result <- ANOVA_DCt(data_2factorBlock3ref, numOfFactors = 2, numberOfrefGenes = 3, 
#'                     block = "block")
#'
#' # Mixed model with random block effect
#' result <- ANOVA_DCt(data_2factorBlock3ref, numOfFactors = 2, numberOfrefGenes = 3,
#'                           block = "block")
#'
#' # Custom mixed model formula with nested random effects
#' result_custom <- ANOVA_DCt(data_repeated_measure_2, numOfFactors = 2, numberOfrefGenes = 1,
#'                             block = NULL,
#'                             model = wDCt ~ treatment * time + (1 | id))
#'
#'
#' @export

ANOVA_DCt <- function(
    x,
    numOfFactors,
    numberOfrefGenes,
    block = NULL,
    alpha = 0.05,
    p.adj = "none",
    analyseAllTarget = TRUE,
    model = NULL,
    modelBased_se = TRUE,
    set_missing_target_Ct_to_40 = FALSE
) {
  
  default_model_formula <- NULL
  
  ## Model specification
  if (!is.null(model)) {
    if (!inherits(model, "formula")) model <- as.formula(model)
    message("Using user defined formula.")
  } else {
    factors <- colnames(x)[1:numOfFactors]
    rhs <- paste(factors, collapse = " * ")
    default_model_formula <- if (is.null(block)) {
      paste("wDCt ~", rhs)
    } else {
      paste("wDCt ~", block, "+", rhs)
    }
  }
  
  n <- ncol(x)
  nDesign <- numOfFactors + if (is.null(block)) 1 else 2
  if (nDesign >= n) stop("Not enough columns for target and reference genes")
  
  designCols <- seq_len(nDesign)
  refCols <- (n - 2 * numberOfrefGenes + 1):n
  targetCols <- setdiff(seq_len(n), c(designCols, refCols))
  
  if (length(targetCols) == 0 || length(targetCols) %% 2 != 0) {
    stop("Target genes must be supplied as E/Ct column pairs")
  }
  
  targetPairs <- split(targetCols, ceiling(seq_along(targetCols) / 2))
  targetNames <- vapply(targetPairs, function(tc) colnames(x)[tc[1]], character(1))
  
  if (!isTRUE(analyseAllTarget)) {
    keep <- targetNames %in% analyseAllTarget
    if (!any(keep)) stop("None of the specified target genes were found in the data.")
    targetPairs <- targetPairs[keep]
    targetNames <- targetNames[keep]
  }
  
  perGene <- lapply(seq_along(targetPairs), function(i) {
    
    gene_name <- targetNames[i]
    gene_df <- x[, c(designCols, targetPairs[[i]], refCols), drop = FALSE]
    
    gene_df <- compute_wDCt(
      gene_df,
      numOfFactors,
      numberOfrefGenes,
      block,
      set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40
    )
    
    gene_df[] <- lapply(gene_df, function(z)
      if (is.factor(z)) as.character(z) else z)
    
    nFac <- if (is.null(block)) numOfFactors else numOfFactors + 1
    gene_df[seq_len(nFac)] <- lapply(
      gene_df[seq_len(nFac)],
      function(col) factor(col, levels = unique(col))
    )
    
    factors <- colnames(gene_df)[1:numOfFactors]
    
    ## Treatment ID (single source of truth)
    gene_df$T <- do.call(paste, c(gene_df[factors], sep = ":"))
    
    ## Model
    if (is.null(model)) {
      rhs <- paste(factors, collapse = " * ")
      model_i <- if (is.null(block)) {
        as.formula(paste("wDCt ~", rhs))
      } else {
        as.formula(paste("wDCt ~", block, "+", rhs))
      }
    } else {
      model_i <- model
    }
    
    has_random_effects <- grepl("\\|", paste(deparse(model_i), collapse = " "))
    
    is_singular <- FALSE
    
    lm <- if (has_random_effects) {
      fit <- suppressMessages(lmerTest::lmer(model_i, data = gene_df, na.action = na.exclude))
      is_singular <- lme4::isSingular(fit)
      fit
    } else {
      stats::lm(model_i, data = gene_df, na.action = na.exclude)
    }
    
    # Add residuals to the data frame for the perGene output
    gene_df$residuals <- residuals(lm, type = "response")
    
    ANOVA_table <- stats::anova(lm)
    lm_formula <- paste(deparse(formula(lm)), collapse = " ")
    
    ## Estimated marginal means
    emm_formula <- as.formula(
      paste("pairwise ~", paste(factors, collapse = " * "))
    )
    
    emm_obj <- suppressMessages(
      emmeans::emmeans(
        lm,
        specs = emm_formula,
        adjust = p.adj,
        mode = "satterthwaite"
      )
    )[[1]]
    
    emm_df <- as.data.frame(emm_obj)
    
    # Choose basis for SE calculation
    if (!modelBased_se) {
      gene_df$val_for_se <- gene_df$wDCt
    } else {
      gene_df$val_for_se <- gene_df$residuals
    }
    
    ## Raw means and se (Calculated per group, not pooled)
    obs_df <- stats::aggregate(
      cbind(wDCt, val_for_se) ~ T,
      data = gene_df,
      FUN = function(z) c(
        mean = mean(z, na.rm = TRUE),
        se   = stats::sd(z, na.rm = TRUE) / sqrt(sum(!is.na(z)))
      )
    )
    
    obs_df_final <- data.frame(
      T = obs_df$T,
      dCt = obs_df$wDCt[, "mean"],
      se = obs_df$val_for_se[, "se"]
    )
    
    ## Treatment ID for emmeans
    emm_df$T <- do.call(paste, c(emm_df[factors], sep = ":"))
    
    merged <- merge(
      emm_df,
      obs_df_final,
      by = "T",
      all.x = TRUE,
      sort = FALSE
    )
    
    ## Fold change
    RE <- 2^(-merged$dCt)
    log2FC <- -merged$dCt
    RE_LCL <- 2^(-merged$upper.CL)
    RE_UCL <- 2^(-merged$lower.CL)
    
    
    ## Compact letter display
    cld_df <- as.data.frame(
      multcomp::cld(
        emm_obj,
        adjust = p.adj,
        alpha = alpha,
        reversed = FALSE,
        Letters = letters
      )
    )
    
    cld_df$T <- do.call(paste, c(cld_df[factors], sep = ":"))
    cld_df$.group <- trimws(cld_df$.group)
    
    merged <- merge(merged,
                    cld_df[, c("T", ".group")],
                    by = "T",
                    all.x = TRUE,
                    sort = FALSE)
    
    merged$sig <- merged$.group
    merged$.group <- NULL
    
    Results <- merged[, c("T", factors, "dCt")]
    Results$RE <- RE
    Results$log2FC <- log2FC
    Results$LCL <- RE_LCL
    Results$UCL <- RE_UCL
    Results$se <- merged$se
    Results$sig <- merged$sig
    Results$Lower.se.RE <- 2^(log2FC - Results$se)
    Results$Upper.se.RE <- 2^(log2FC + Results$se)
    Results$Lower.se.log2FC <- log2FC - Results$se
    Results$Upper.se.log2FC <- log2FC + Results$se

    
    Results <- Results %>%
      dplyr::mutate_if(is.numeric, ~ round(., 5))
    
    Results$gene <- gene_name
    
    list(
      Final_data = gene_df[, !colnames(gene_df) %in% c("T", "val_for_se")],
      lm = lm,
      lm_formula = lm_formula,
      ANOVA_table = ANOVA_table,
      Results = Results,
      is_singular = is_singular
    )
  })
  
  relativeExpression <- do.call(rbind, lapply(perGene, `[[`, "Results"))
  rownames(relativeExpression) <- NULL
  
  relativeExpression <- relativeExpression[, c("gene",
                                               setdiff(colnames(relativeExpression), c("gene", "T", "sig")),
                                               "sig"), drop = FALSE]
  
  cat("\nRelative Expression\n\n")
  print(relativeExpression)
  
  singular_vec <- vapply(perGene, `[[`, logical(1), "is_singular")
  singular_genes <- targetNames[singular_vec]
  
  if (any(singular_vec)) {
    warning(
      "Singular fit detected for the following genes:\n  ",
      paste(singular_genes, collapse = ", ")
    )
  }
  
  if (is.null(model) && !is.null(default_model_formula)) {
    cat("\nNote: Using default model for statistical analysis:",
        default_model_formula, "\n")
  }
  
  invisible(list(
    relativeExpression = relativeExpression,
    perGene = setNames(perGene, targetNames)
  ))
}

#' Internal global variables
#'
#' These objects are declared to avoid R CMD check notes about
#' non-standard evaluation (e.g., in ggplot2).
#'
#' @keywords internal
#' @name globalVariables
NULL

utils::globalVariables(c(
  "ymin", "ymax", "lower", "upper", "sig", "d",
  "contrast", "RE", "log2FC", "Lower.se", "Upper.se", 
  "Lower.se.RE", "Upper.se.RE", "factors", "as.formula", "tail", "Gene",
  "se", "pvalue", "ave", "reshape", "lm", "formula", "anova", "wDCt", 
  "na.exclude", "residuals", "block", "bwDCt", "residual", "p.value", "ddCt"
))

#' @title Delta Delta Ct method t.test analysis
#'
#' @description
#' The \code{TTEST_DDCt} function performs fold change expression analysis based on
#' the \eqn{\Delta \Delta C_T} method using Student's t.test. It supports analysis
#' of one or more target genes evaluated under two experimental conditions
#' (e.g. control vs treatment).
#'
#' @details
#' Relative expression values are computed using one or more reference genes for normalization.
#' Both paired and unpaired experimental designs are supported.
#'
#' Paired samples in quantitative PCR refer to measurements collected from the same
#' individuals under two different conditions (e.g. before vs after treatment),
#' whereas unpaired samples originate from different individuals in each condition.
#' Paired designs allow within-individual comparisons and typically reduce
#' inter-individual variability.
#'
#' The function returns numerical summaries as well as bar plots based on either
#' relative expression (RE) or log2 fold change (log2FC).
#'
#' @author Ghader Mirzaghaderi
#'
#' @export
#'
#' @import tidyr
#' @import dplyr
#' @import reshape2
#' @import ggplot2
#'
#' @param x
#' A data frame containing experimental conditions, biological replicates, and
#' amplification efficiency and Ct values for target and reference genes.
#' The number of biological replicates must be equal across genes. If this 
#' is not true, or there are \code{NA} values use \code{ANODA_DDCt} function 
#' for independent samples or \code{REPEATED_DDCt} for paired samples.
#' See the package vignette for details on the required data structure.
#'
#' @param paired
#' Logical; if \code{TRUE}, a paired t-test is performed.
#'
#' @param var.equal
#' Logical; if \code{TRUE}, equal variances are assumed and a pooled variance
#' estimate is used. Otherwise, Welch's t-test is applied.
#'
#' @param numberOfrefGenes
#' Integer specifying the number of reference genes used for normalization.
#'
#' @param Factor.level.order
#' Optional character vector specifying the order of factor levels.
#' If \code{NULL}, the first level of the factor column is used as the calibrator. 
#'
#' @param p.adj
#' Method for p-value adjustment. One of
#' \code{"holm"}, \code{"hochberg"}, \code{"hommel"}, \code{"bonferroni"},
#' \code{"BH"}, \code{"BY"}, \code{"fdr"}, or \code{"none"}. See \code{\link[stats]{p.adjust}}.
#' 
#' @param set_missing_target_Ct_to_40 If \code{TRUE}, missing target gene Ct 
#'   values become 40; if \code{FALSE} (default), they become NA.
#'
#' 
#' @details
#' All the functions for relative expression analysis (including `TTEST_DDCt()`, 
#' `WILCOX_DDCt()`, `ANOVA_DDCt()`, `ANCOVA_DDCt()`, `REPEATED_DDCt()`, and `ANOVA_DCt()`) return the 
#' relative expression table which include fold change and corresponding 
#' statistics. The output of `ANOVA_DDCt()`, `ANCOVA_DDCt()`, `ANCOVA_DDCt()`, `REPEATED_DDCt()`, 
#' and `ANOVA_DCt()` also include lm models, residuals, raw data and ANOVA table 
#' for each gene. 
#' 
#' The expression table returned by `TTEST_DDCt()`, 
#' `WILCOX_DDCt()`, `ANOVA_DDCt()`, `ANCOVA_DDCt()`, and `REPEATED_DDCt()` functions 
#' include these columns: gene (name of target genes), 
#' contrast (calibrator level and contrasts for which the relative expression is computed), 
#' RE (relative expression or fold change),  log2FC (log(2) of relative expression or fold change), 
#' pvalue, sig  (per-gene significance), LCL (95\% lower confidence level), UCL (95\% upper confidence level),
#' se (standard error of mean calculated from the weighted delta Ct values of each of the main factor levels),
#' Lower.se.RE (The lower limit error bar for RE which is 2^(log2(RE) - se)), 
#' Upper.se.RE (The upper limit error bar for RE which is 2^(log2(RE) + se)),
#' Lower.se.log2FC (The lower limit error bar for log2 RE), and 
#' Upper.se.log2FC (The upper limit error bar for log2 RE)
#' 
#'
#' @return
#' A list with the following components:
#' \describe{
#'   \item{Result}{Table containing RE values, log2FC, p-values, significance codes,
#'   confidence intervals, standard errors, and lower/upper SE limits.}
#' }
#'
#' @references
#' LivakKJ, Schmittgen TD (2001).
#' Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR
#' and the Double Delta CT Method.
#' \emph{Methods}, 25(4), 402–408.
#' doi:10.1006/meth.2001.1262
#'
#' Ganger MT, Dietz GD, and Ewing SJ (2017).
#' A common base method for analysis of qPCR data and the application of
#' simple blocking in qPCR experiments.
#' \emph{BMC Bioinformatics}, 18, 1–11.
#' 
#' Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. (2019). 
#' The ultimate qPCR experiment: producing publication quality, reproducible 
#' data the first time. \emph{Trends in Biotechnology}, 37, 761-774. 
#' 
#' Yuan JS, Reed A, Chen F, Stewart N (2006).
#' Statistical Analysis of Real-Time PCR Data.
#' \emph{BMC Bioinformatics}, 7, 85.
#'
#' @examples
#' # Example data structure
#' data1 <- read.csv(system.file("extdata", "data_ttest18genes.csv", package = "rtpcr"))
#'
#' # Unpaired t-test
#' TTEST_DDCt(
#'   data1,
#'   paired = FALSE,
#'   var.equal = TRUE,
#'   numberOfrefGenes = 1)
#'
#' # With amplification efficiencies
#' data2 <- read.csv(system.file("extdata", "data_1factor_one_ref_Eff.csv", package = "rtpcr"))
#'
#' TTEST_DDCt(
#'   data2,
#'   numberOfrefGenes = 1)
#'
#' # Two reference genes
#' data3 <- read.csv(system.file("extdata", "data_1factor_Two_ref.csv", package = "rtpcr"))
#' TTEST_DDCt(
#'   data3,
#'   numberOfrefGenes = 2)




TTEST_DDCt <- function(x,
                       numberOfrefGenes,
                       Factor.level.order = NULL,
                       paired = FALSE,
                       var.equal = TRUE,
                       p.adj = "none",
                       set_missing_target_Ct_to_40 = FALSE) {
  
  stopifnot(is.data.frame(x))
  stopifnot(numberOfrefGenes >= 1)
  
  
  # Factor handling
  if (is.null(Factor.level.order)) {
    x[, 1] <- factor(x[, 1], levels = unique(x[, 1]))
    calibrator <- x[, 1][1]
  } else {
    x[, 1] <- factor(x[, 1], levels = Factor.level.order)
    calibrator <- Factor.level.order[1]
  }
  
  on.exit(cat(sprintf(
    "*** The %s level was used as calibrator.\n",
    calibrator
  )))
  
  
  # Column structure
  nc <- ncol(x)
  
  nTargets <- (nc - 2 - numberOfrefGenes * 2) / 2
  if (nTargets < 1) stop("No target genes detected")
  
  cat(sprintf(
    "*** %s target(s) using %s reference gene(s) was analysed!\n",
    nTargets, numberOfrefGenes
  ))
  
  target_start <- 3
  target_end   <- target_start + nTargets * 2 - 1
  ref_start    <- target_end + 1
  
  gene_names <- sub("_E$", "", colnames(x)[seq(target_start, target_end, by = 2)])
  
  # Replicates per condition
  r <- table(x[, 1])[1]
  
  
  # Result container
  res <- matrix(NA, nrow = nTargets, ncol = 7)
  colnames(res) <- c("gene", "ddCt", "RE", "LCL", "UCL", "pvalue", "se")
  
  
  # Loop over targets
  for (i in seq_len(nTargets)) {
    
    E_col  <- target_start + (i - 1) * 2
    Ct_col <- E_col + 1
    
    ## Build per-target wide table
    tmp <- x[, c(1, 2, E_col, Ct_col, ref_start:nc)]
    
    ## Compute wDCt using helper
    tmp <- compute_wDCt(
      x = tmp,
      numOfFactors = 1, 
      numberOfrefGenes = numberOfrefGenes, 
      set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40,
      block = NULL
    )
    
    wDCt <- tmp$wDCt
    
    cal_vals <- wDCt[tmp[, 1] == calibrator]
    trt_vals <- wDCt[tmp[, 1] != calibrator]
    
    tt <- stats::t.test(
      trt_vals,
      cal_vals,
      paired = paired,
      var.equal = var.equal
    )
    
    dif <- mean(trt_vals, na.rm = TRUE) - mean(cal_vals, na.rm = TRUE)
    
    res[i, ] <- c(
      gene_names[i],
      dif,
      2^(-dif),
      2^(-tt$conf.int[2]),
      2^(-tt$conf.int[1]),
      tt$p.value,
      tt$stderr
    )
  }
  
  
  # Result table
  res <- as.data.frame(res)
  num_cols <- c("ddCt", "RE", "LCL", "UCL", "pvalue", "se")
  res[num_cols] <- lapply(res[num_cols], as.numeric)
  
  # res$dif <- NULL
  
  res <- transform(
    res,
    log2FC = -ddCt,
    Lower.se.RE = 2^(-ddCt - se),
    Upper.se.RE = 2^(-ddCt + se),
    Lower.se.log2FC = -ddCt - se,
    Upper.se.log2FC = -ddCt + se,
    pvalue = stats::p.adjust(pvalue, method = p.adj)
  )
  
  res$sig <- cut(
    res$pvalue,
    breaks = c(-Inf, 0.001, 0.01, 0.05, 0.1, Inf),
    labels = c("***", "**", "*", ".", " ")
  )


  desired_order <- c("gene", "ddCt", "RE", "log2FC", "LCL", "UCL", "se",
                     "Lower.se.RE", "Upper.se.RE", "Lower.se.log2FC", "Upper.se.log2FC", 
                     "pvalue", "sig")
  res <- res[, desired_order]
  
  
  Result <- res %>%
    dplyr::mutate_if(is.numeric, ~ round(., 5))
  
  Result
}

#' @title Computing the mean of technical replicates
#'
#' @description
#' Computes the arithmetic mean of technical replicates for each sample or group.
#' This is often performed before ANOVA or other statistical analyses to simplify
#' comparisons between experimental groups.
#'
#' @details
#' The \code{meanTech} function calculates the mean of technical replicates
#' based on one or more grouping columns. This reduces the dataset to a single
#' representative value per group, facilitating downstream analysis such as
#' fold change calculation or ANOVA.
#'
#' @author
#' Ghader Mirzaghaderi
#'
#' @export
#'
#' @import tidyr
#' @import dplyr
#' @import reshape2
#'
#' @param x A raw data frame containing technical replicates.
#' @param numOfFactors Integer. Number of experimental factor columns
#' @param block Character. Block column name or \code{NULL}. 
#'
#' @param groups
#' An integer specifying the number of columns before the technical replicate column.
#' 
#' @param set_missing_target_Ct_to_40 If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA. 
#' @param numberOfrefGenes Integer. Number of reference genes.
#'
#' @return
#' A data frame with the mean of technical replicates for each group.
#'
#' @examples
#'
#' # Example input data frame with technical replicates
#' data1 <- read.csv(system.file("extdata", "data_withTechRep.csv", package = "rtpcr"))
#'
#' # Calculate mean of technical replicates using first four columns as groups
#' meanTech(data1,
#'          groups = 2,
#'          numOfFactors = 1,
#'          numberOfrefGenes = 1,
#'          block = NULL)
#' 
#' # Another example using different dataset and grouping columns
#' data2 <- read.csv(system.file("extdata", "data_Lee_etal2020qPCR.csv", package = "rtpcr"))
#' meanTech(data2, groups = 3,
#'          numOfFactors = 2,
#'          numberOfrefGenes = 1,
#'          block = NULL)


meanTech <- function(x, groups, numOfFactors, numberOfrefGenes, block,
                     set_missing_target_Ct_to_40 = FALSE){

  x <- .cleanup(x = x, numOfFactors = numOfFactors + 1, numberOfrefGenes = numberOfrefGenes, block = block,
                set_missing_target_Ct_to_40 = set_missing_target_Ct_to_40)
  
  g <- colnames(x)[1:groups]

  # Concatenate the columns using paste0
  x$T <- do.call(paste, c(x[1:groups], sep = ":"))

  data_meanBiolRep <- x %>%
    group_by(across(all_of(g))) %>%
    summarise(across(where(is.numeric), mean, na.rm = TRUE))

  y <- data_meanBiolRep
  m <- ncol(y)-4
  databiol <- y[,-m]
  return(as.data.frame(databiol))
}

---
title: "Manual"
output: github_document
vignette: >
  %\VignetteIndexEntry{Manual}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE, fig.align='center', warning = F, message=F}
options(tinytex.verbose = TRUE)
knitr::opts_chunk$set(echo = TRUE)
library(rtpcr)
```



# Overview

rtpcr is a tool for analysis of RT-qPCR gene expression data using delta Ct (dCt) 
and delta delta Ct (ddCt) methods, including t-tests and ANOVA, repeated-measures models, and publication-ready visualizations. The package implements a general calculation method adopted from Ganger et al. (2017) and Taylor et al. (2019), covering both the Livak and Pfaffl methods. See the [calculation method](Method.html) for details.


![Figure 1: rtpcr is now available as shiny_rtpcr, a web application developed using R/Shiny for interactive analysis of qPCR data at https://mirzaghaderi.shinyapps.io/rtpcr/](../man/figures/shiny_rtpcr.png){.center width="100%"}


## Running the shiny version of the rtpcr

If you have problem with connecting to https://mirzaghaderi.shinyapps.io/rtpcr/, you can follow the following steps in Rstudio to run the shiny version of the rtpcr.

```{r eval= F}
# install.packages("rtpcr")
# install.packages("shiny")
library(shiny)
library(rtpcr)
# Run the following code in Rstudio
runApp(system.file("shinyapp/app.R", package = "rtpcr"))
```



# Functions
In the rtpcr package, functions with _DDCt at the end of their name (`ANOVA_DDCt()`, `TTEST_DDCt()`, `WILCOX_DDCt()`) perform expression analysis based on the delta delta Ct (ddCt) method, while `ANOVA_DCt()` function analyze gene expression using the delta Ct (dCt) method. The ANOVA prefix indicates that the function uses analysis of variance (using a default full factorial model or a user defined model) for statistical analysis, and mean comparisons. Mean comparisons is actually performed by the `emmeans()` function using the resulting model from the ANOVA analysis. 

| Function            | Description                                                  |
|---------------------|--------------------------------------------------------------|
| `ANOVA_DCt()`     | dCt expression analysis for all the level combinations of factor(s). |
| `ANOVA_DDCt()`    | ddCt expression analysis for levels of a factor (generally or per levels of another factors(s)), specified by the `specs` argument.  |
| `TTEST_DDCt()`    | ddCt method *t*.test analysis for paired or unpaired samples. |
| `WILCOX_DDCt()`         | ddCt method wilcox.test analysis for paired or unpaired samples.    |
| `plotFactor()`        | Bar plot of gene expression for one-, two- or three-factor experiments
| `Means_DDCt()`        | Pairwise comparison of RE values for any user-specified effect |
| `efficiency()`        | Amplification efficiency statistics and standard curves     |
| `meanTech()`          | Calculate mean of technical replicates. This is used if your data needs averaging over biological replicates.     |
| `multiplot()`         | Combine multiple ggplot objects into a single layout   |
| `compute_wDCt()`	 | Cleaning data and weighted delta Ct (wDCt) calculation using the geometric mean of reference gene(s).   |
| `long_to_wide()`	 | Converts a 4-column qPCR long data format to wide format    |


# Quick start
### Installing and loading

The `rtpcr` package can be installed by running the following code in R:

from CRAN:

```{r eval = F}
# Installing from CRAN
install.packages("rtpcr")

# Loading the package
library(rtpcr)
```

Or from from GitHub (developing version):

```{r eval = F}
devtools::install_github("mirzaghaderi/rtpcr", build_vignettes = TRUE)
```


# Input data structure 

For relative expression analysis (using `TTEST_DDCt()`, `WILCOX_DDCt()`, `ANOVA_DCt()`, and `ANOVA_DDCt()` functions), the amplification efficiency (`E`) and `Ct` or `Cq` values (the mean of technical replicates) is used for the input table. If the `E` values are  not available you should use '2' instead representing the complete primer amplification efficiency. The input data table should include the following columns from left to wright:


1.  Experimental condition columns (and one block if
    available [NOTE 1](#note-1))
2.  Replicates information (biological replicates or subjects; see [NOTE
    2](#note-2), and [NOTE 3](#note-3))  
3.  Target genes efficiency and Ct values (a pair column for each gene).
4.  Reference genes efficiency and Ct values (a pair column for each
    gene) [NOTE 4](#note-4).

The package supports **one or more target or reference gene(s)**, supplied as efficiency–Ct column pairs. Reference gene columns must always appear last. Two sample input data sets are presented below.



![Figure 2: A sample input data with one experimetal factor, replicate column and E/Ct information of target and reference genes](../man/figures/sampleData1.png){.center width="80%"}

<br>

If there is no blocking factor, the block column should be omitted. However, a column for biological replicates (which may be named "Rep", "id" or similar) is always required.

<br>

![Figure 3: A sample input data with two experimetal factors, blocking factor, replicate column and E/Ct information of target and reference genes](../man/figures/dataStructure1.png){.center width="100%"}






#### NOTE 1

When a qPCR experiment is done in multiple qPCR plates, variation resulting from the plates may interfere with the actual amount of gene expression. One solution is to conduct each plate as a randomized block so that at least one replicate of each treatment and control is present on a plate. Block effect is usually considered as random and its interaction with any main effect is not considered. 

#### NOTE 2

For `TTEST_DDCt()` and `WILCOX_DDCt()` (independent groups), `ANOVA_DCt()`, and `ANOVA_DDCt()` each row is from a separate and unique biological replicate. For example, a data frame with
12 rows has come from an experiment with 12 individuals. The
repeated measure models are intended for experiments with repeated
observations (e.g. time-course data). In repeated measure experiments the replicate
column contains identifiers for each individual (id or subject). For
example, all rows with a `r1` at Rep column correspond to a single
individual, all rows with a `r2` correspond to another individual, and so
on, which have been sampled at specific time points.

#### NOTE 3

Your data table may also include a column of technical replicates. For example, 
using one target and one reference genes, if you want to have 4 biological 
and 3 technical replicates under Control and Treatment conditions,
there would be a table of 24 rows containing both biological 
replicates and technical replicate columns in the data. 
In this case, the `meanTech()` function should be
applied first to calculate the mean of the technical replicates. The
resulting collapsed table is then used as the input for expression analysis. To
use the `meanTech()` function correctly, the technical replicate column
must appear immediately after the biological replicate column (see [Mean
of technical replicates](#mean-of-technical-replicates) for an example).

![Figure 4: An experimental design with four biological replicates for both Control and Treatment conditions, assuming a single sampling time point, where cDNA samples were analyzed by qPCR. The diagram details the initial dataset containing technical replicates. Three technical replicates shown for biological replicate 1 under Control, with example amplification efficiencies (E) and cycle threshold (Ct) values for both target and reference genes. Technical replicates is averaged to get a condensed dataset, comprising eight rows (one per biological replicate). This final data structure is used for the downstream relative expression analysis. Green and yellow circles are control samples.](../man/figures/base.png){.center width="100%"}

#### NOTE 4

Complete amplification efficiency (E) in the input data is denoted by 2. This 
means that 2 indicates 100%, and 1.85 and 1.70 indicate 0.85% and 0.70%
amplification efficiencies. 


# Handling missing data

Missing Ct values for target genes is Handled using the `set_missing_target_Ct_to_40` argument. If `TRUE`, missing target gene Ct values become 40; if `FALSE` (default), they become NA. missing Ct values of reference genes are always converted to NA.
If there are more than one reference gene, NA in the place of the E or the Ct value of cause skipping that gene and remaining references are geometrically averaged in that 
replicate. 


# Data Analysis 

### Amplification Efficiency 

The `efficiency()` function calculates the amplification efficiency (E), slope, and R² statistics for genes, and performs pairwise comparisons of slopes. It takes a data frame in which the first column contains the dilution ratios, followed by the Ct value columns for each gene.

```{r eval= F}
# Applying the efficiency function
data <- read.csv(system.file("extdata", "data_efficiency.csv", package = "rtpcr"))
data
# dilutions	Gene1	Gene2	Gene3
# 1.00	25.58	24.25	22.61
# 1.00	25.54	24.13	22.68
# 1.00	25.50	24.04	22.63
# 0.50	26.71	25.56	23.67
# 0.50	26.73	25.43	23.65
# 0.50	26.87	26.01	23.70
# 0.20	28.17	27.37	25.11
# 0.20	28.07	26.94	25.12
# 0.20	28.11	27.14	25.11
# 0.10	29.20	28.05	26.17
# 0.10	29.49	28.89	26.15
# 0.10	29.07	28.32	26.15
# 0.05	30.17	29.50	27.12
# 0.05	30.14	29.93	27.14
# 0.05	30.12	29.71	27.16
# 0.02	31.35	30.69	28.52
# 0.02	31.35	30.54	28.57
# 0.02	31.35	30.04	28.53
# 0.01	32.55	31.12	29.49
# 0.01	32.45	31.29	29.48
# 0.01	32.28	31.15	29.26

# Analysis
efficiency(data)

# $Efficiency
#    Gene     Slope        R2        E
# 1 Gene1 -3.388094 0.9965504 1.973110
# 2 Gene2 -3.528125 0.9713914 1.920599
# 3 Gene3 -3.414551 0.9990278 1.962747
# 
# $Slope_compare
# $contrasts
#  contrast          estimate    SE df t.ratio p.value
#  C2H2.26 - C2H2.01   0.1400 0.121 57   1.157  0.4837
#  C2H2.26 - GAPDH     0.0265 0.121 57   0.219  0.9740
#  C2H2.01 - GAPDH    -0.1136 0.121 57  -0.938  0.6186
```


![Figure 5: Standard curve plot displaying the relationship between the logarithm of cDNA dilution factors (ranging from -2.0 to 0.0) and their corresponding qPCR cycle threshold (Ct) values for three genes: C2H2.26, C2H2.01, and GAPDH. The accompanying table provides the precise Ct measurements, which is used to determine the amplification efficiency for each gene](../man/figures/standCur.png){.center width="67%"}


### Relative expression

**Single factor experiment with two levels (e.g. Control and Treatment):** `TTEST_DDCt()` function is used for relative expression analysis in treatment condition compared to the control group. Both paired and unpaired experimental designs are supported. if the data doesn't follow t.test assumptions, the `WILCOX_DDCt()` function can be used instead. 


**Single- or multi-factor experiments:** In these cases, `ANOVA_DDCt()` and `ANOVA_DCt()` functions are used for the analysis of qPCR data. By default, statistical analysis is performed based on uni- or multi-factorial Completely Randomized Design (CRD) or Randomized Complete Block Design (RCBD) design based on `numOfFactors` and the availability of `block`. However, optional custom model formula as a character string can be supplied to these functions. If provided, this overrides the default formula (full factorial CRD or RCBD design). The formula uses `wDCt` as the response variable (wDCt is automatically created by the function).  For mixed models, include random effects using `lmer` syntax (e.g., `wDCt ~ Treatment + (1 | id)`). Below are a sample of most common models that can be used. 

| Example models may be used in `ANOVA_DCt()` or `ANOVA_DDCt()` functions | Experimental design            |
|---------------------------------------------------|--------------------------------------------------|
| wDCt ~ Condition                                  | Completely Randomized Design (CRD). Can also be used for t.test with two independent groups. (**default**) |
| wDCt ~ Factor1 * Factor2 * Factor3                | Factorial under Completely Randomized Design (RCBD) (**default**)    |
| wDCt ~ block + Factor1 * Factor2                  | Factorial under Randomized Complete Block Design (**default**) |
| wDCt ~ time + (1 \| id)                            | Repeated measure analysis: different time points. Also can be used for t.test with two paired groups. |
| wDCt ~ Condition * time + (1 \| id)                | Repeated measure analysis: split-plot in time  |
| wDCt ~ wDCt ~ Condition * time + (1 \| block) + (1 \| id)  | Repeated measure analysis: split-plot in time   |
| wDCt ~ Type + Concentration                       | Analysis of Covariance: Type is covariate       |
| wDCt ~ block + Type + Concentration               | Analysis of Covariance with blocking factor: block and Type are covariates      |

#### NOTE

By default, data are analysed according to a full factorial CRD (if there is no blocking factor) or RCBD (if there is a blocking factor) model, so you do not need to explicitly define a model. The package automatically selects an appropriate model based on the provided arguments. If no model is specified, the default used model is printed along with the output expression table.

#### NOTE

Sometime groups are paired (e.g., in repeated measure experiments). Examples: 1) Analyzing gene expression in different time points, or before and after treatment in each biological replicate;  2) Analyzing gene expression between two tissue types within the same organism. For such analysis types, if there are only two levels or time points, we can use the `TTEST_DDCt()` with the argument `paired = TRUE`; or `ANOVA_DDCt()` (if there are two or more time points) with a repeated measure model such as `wDCt ~ Treatment + ( 1 | id)`  or `wDCt ~ Treatment + ( 1 | Rep)`.

![Figure 6: A) Basic structure of independent group- or paired group-experiments. Data of paired group-experiments are analysed using the `TTEST_DDCt()` function with the argument paired = TRUE; or using the `ANOVA_DDCt()` function with a repeated measure model such as wDCt ~ Treatment + ( 1 | Rep). B) The standard error (`se`) in the `ANOVA_DDCt()` function is calculated from from weighted delta Ct (wDCt) values or model residuals (default, `modelBased_se = TRUE`) for the experimental groups according to the selected `se.type` (One of `"paired.group"`, `"two.group"`, or `"single.group"`). Note that in the `"paired.group"` method,`se` is computed from differences (wddCt or equivalent values from residuals) which resembles the standard error of a paired t.test, while `"two.group"` and `"single.group"` methods use wdCt or resudual values. In the `"two.group"` method, `se` is computed for each non-reference group from that group and the reference (calibrator) group which resembles the standard error of an unpaired t.test. `"single.group"` computes `se` within each reference or non-reference group. If a model for a repeated‐measure or a paired-group design is specified, `se.type` should be set to `"paired.group"` in the `ANOVA_DDCt()` function. In method 4, the 95% confidence interval (CI) is calculated and printed in the output expression table under LCL and UCL columns. CI uses the pooled standard error (SE) and can be used as another way of error presentation. It can be used for any experimental models as it is derived from a fitted model.](../man/figures/repeated_measure.png){.center width="100%"}

<br>

### Examples

Relative expression analysis can be done using ddCt or dCt methods through different functions (i.e. `TTEST_DDCt()`, `WILCOX_DDCt()`, `ANOVA_DDCt()`, and `ANOVA_DCt()`). Below are some examples of expression analysis using ddCt method.


```{r eval= F}
data <- read.csv(system.file("extdata", "data_Yuan2006PMCBioinf.csv", package = "rtpcr"))

# Anova analysis
ANOVA_DDCt(
  data,
  specs = "condition",
  numOfFactors = 1,
  numberOfrefGenes = 1,
  block = NULL)

# An example of a properly arranged dataset from a repeated-measures experiment.
data <- read.csv(system.file("extdata", "data_repeated_measure_1.csv", package = "rtpcr"))

# time	id  E_Target	Ct_target   E_Ref      Ct_Ref
#    1	 1	   	2	    18.92	2	32.77
#    1	 2	   	2	    15.82	2	32.45
#    1	 3	   	2	    19.84	2	31.62
#    2	 1	  	2	    19.46	2	33.03
#    2	 2	   	2	    17.56	2	33.24
#    2	 3	   	2	    19.74	2	32.08
#    3	 1	  	2	    15.73	2	32.95
#    3	 2	  	2	    17.21	2	33.64
#    3	 3	   	2	    18.09	2	33.40

# Repeated measure analysis
res <- ANOVA_DDCt(
  data,
  numOfFactors = 1,
  numberOfrefGenes = 1,
  specs = "time",
  block = NULL, model = wDCt ~ time + (1 | id))


# Paired t.test (equivalent to repeated measure analysis, but not
# always the same results, due to different calculation methods!)
TTEST_DDCt(
  data[1:6,], 
  numberOfrefGenes = 1, 
  paired = T)


# Anova analysis
data3 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))

res <- ANOVA_DDCt(
  x = data3,
  specs = "Type | Concentration",
  numOfFactors = 2,
  numberOfrefGenes = 3,
  block = "block",
  analyseAllTarget = TRUE)
```


# Output
## Data output
All the functions for relative expression analysis (including `TTEST_DDCt()`, `WILCOX_DDCt()`, `ANOVA_DDCt()`, and `ANOVA_DCt()`) return the relative expression table which include fold change and corresponding statistics. The output of `ANOVA_DDCt()`, and `ANOVA_DCt()` also include lm model, residuals, raw data and ANOVA table for each gene. These outputs can be obtained as follow:

| Per_gene Output    | Code                                                  |
|--------------------|-------------------------------------------------------|
| expression table   | `res$relativeExpression`                              |
| ANOVA table        | `res$perGene$gene_name$ANOVA_table`                   |
| ANOVA lm           | `res$perGene$gene_name$lm`                      |
| ANOVA lm formula           | `res$perGene$gene_name$lm_formula`                      |
| Residuals          | `resid(res$perGene$gene_name$lm)`               |


```{r eval= F}
# Relative expression table for the specified column in the input data:
data3 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))

res <- ANOVA_DDCt(
  x = data3,
  specs = "Concentration",
  numOfFactors = 2,
  numberOfrefGenes = 3,
  block = "block",
  analyseAllTarget = TRUE)

# Relative Expression
# gene contrast    ddCt     RE  log2FC    LCL    UCL      se L.se.RE U.se.RE L.se.log2FC U.se.log2FC  pvalue sig
#   PO       L1  0.0000 1.0000  0.0000 0.0000 0.0000 0.13940 0.90790 1.10144     0.00000     0.00000 1.00000    
#   PO L2 vs L1 -0.9461 1.9266  0.9461 1.2586 2.9493 0.14499 1.74245 2.13036     0.85564     1.04613 0.00116  **
#   PO L3 vs L1 -2.1919 4.5693  2.1919 3.0806 6.7772 0.29402 3.72685 5.60221     1.78783     2.68748 0.00000 ***
#  NLM       L1  0.0000 1.0000  0.0000 0.0000 0.0000 0.91809 0.52921 1.88962     0.00000     0.00000 1.00000    
#  NLM L2 vs L1  0.8656 0.5487 -0.8656 0.3983 0.7561 0.36616 0.42577 0.70734    -1.11579    -0.67163 0.00018 ***
#  NLM L3 vs L1 -1.4434 2.7196  1.4434 1.9467 3.7994 0.17132 2.41511 3.06256     1.28179     1.62542 0.00000 ***
# 
# The L1 level was used as calibrator.
# Note: Using default model for statistical analysis: wDCt ~ block + Concentration * Type 


ANOVA_table <- res$perGene$PO$ANOVA_table
ANOVA_table

lm <- res$perGene$PO$lm
lm

lm_formula <- res$perGene$gene_name$lm_formula
lm_formula

residuals <- resid(res$perGene$gene_name$lm)
residuals
```



![Figure 7: A) All the functions for relative expression analysis (including `TTEST_DDCt()`, `WILCOX_DDCt()`, `ANOVA_DDCt()`, and `ANOVA_DCt()`) return the relative expression table which include fold change and corresponding statistics. B) The output of `ANOVA_DDCt()`, and `ANOVA_DCt()` also include lm model, residuals, raw data and ANOVA table for each gene. ](../man/figures/out.png){.center width="100%"}




## Plot output
A single function of `plotFactor()` is used to produce bar plots for one- to three-factor expression tables. 

## Plot output: example 1

```{r eval= F, warning = F, fig.height = 7, fig.width = 12.5, fig.align = 'center', warning = F}
data <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))
#Perform analysis first
res <- ANOVA_DCt(
  data,
  numOfFactors = 3,
  numberOfrefGenes = 1,
  block = NULL)
  
df <- res$relativeExpression
df
 # Generate three-factor bar plot
plotFactor(
  df,
  x_col = "SA",       
  y_col = "log2FC",       
  group_col = "Type",   
  facet_col = "Conc",    
  Lower.se_col = "Lower.se.log2FC",
  Upper.se_col = "Upper.se.log2FC",
  letters_col = "sig",
  letters_d = 0.3,
  col_width = 0.7, 
  dodge_width = 0.7,
  fill_colors = c("palegreen3", "skyblue"),
  color = "black",
  base_size = 14, 
  alpha = 1,
  legend_position = c(0.1, 0.2))
```



![Figure 8: The bar plots of the log2 fold change expression expression of a gene produced by the `plotFactor()` function.](../man/figures/Rplot02.png){.center height="400px"}


# How to edit ouptput plots?
The plot can further be edited by adding new layers (see examples bellow):

| Task | Example Code |
|------|--------------|
| Change y-axis label | `p + ylab("Relative expression ($\Delta\Delta Ct$ method)")` |
| Add a horizontal reference line | `p + geom_hline(yintercept = 0, linetype = "dashed")` |
| Change y-axis limits | `p + scale_y_continuous(expand = expansion(mult = c(0, 0.1)))` |
| Relabel x-axis | `p + scale_x_discrete(labels = c("A" = "Control", "B" = "Treatment"))` |
| Change fill colors | `p + scale_fill_brewer(palette = "Set2")` |
| x.axis line width | `p + theme(axis.line.x = element_line(linewidth = 0))` |
| y.axis line width | `p + theme(axis.line.y = element_line(linewidth = 0))` |
| panel line width | `theme(panel.border = element_rect(color = "black", linewidth = 1))` |


### Plot output: example 2
```{r eval= F, fig.height = 7, fig.width = 12.5, fig.align = 'center', warning = F}
data <- read.csv(system.file("extdata", "data_2factorBlock.csv", package = "rtpcr"))
res <- ANOVA_DCt(data, 
      numOfFactors = 2,
      block = "block",
      numberOfrefGenes = 1)

df <- res$relativeExpression

plotFactor(
  data = df,
  x_col = "Concentration",
  y_col = "RE",
  group_col = "Type",
  Lower.se_col = "Lower.se.RE",
  Upper.se_col = "Upper.se.RE",
  letters_col = "sig",
  letters_d = 0.2,
  fill_colors = c("aquamarine4", "gold2"),
  color = "black",
  alpha = 1,
  col_width = 0.7,
  dodge_width = 0.7,
  base_size = 16, 
  legend_position = c(0.2, 0.8))
```



![Figure 9: The bar plots of the relative expression expression and log2 fold change of a gene produced by the `plotFactor()` function.](../man/figures/Rplot01.png){.center height="300px"}


### Plot output: example 3

```{r eval= F, warning = F}
# Using data from Heffer et al., 2020, PlosOne
library(dplyr)

res <- ANOVA_DDCt(
  data_Heffer2020PlosOne,
  numOfFactors = 1,
  specs = "Treatment",
  numberOfrefGenes = 1,
  block = NULL)

data <- res$relativeExpression

# Selecting only the first words in 'contrast' column to be used as the x-axis labels.
data$contrast <- sub(" .*", "", data$contrast)

plotFactor(
  data = data,
  x_col = "contrast",
  y_col = "RE",
  group_col = "contrast",
  facet_col = "gene",
  Lower.se_col = "Lower.se.RE",
  Upper.se_col = "Upper.se.RE",
  letters_col = "sig",
  letters_d = 0.2,
  alpha = 1,
  fill_colors = palette.colors(4, recycle = TRUE),
  color = "black",
  col_width = 0.5,
  dodge_width = 0.5,
  base_size = 16, 
  legend_position = "none")
```



![Figure 10: The bar plot of the fold change expression of 6 genes produced by the `plotFactor()` function.](../man/figures/Rplot03.png){.center height="600px"}


# Post-hoc analysis
Although all the expression analysis functions perform statistical comparisons for the levels of the analysed factor, further post-hoc analysis is still possible. 
The `Means_DDCt()` function performs post-hoc comparisons using a fitted model object produced by `ANOVA_DCt()` and `ANOVA_DDCt()`. It applies pairwise statistical comparisons of relative expression (RE) values for user-specified effects via the `specs` argument. Supported effects include simple effects, interactions, and slicing, provided the underlying model is an ANOVA. For ANCOVA models, only simple effects are returned.

```{r eval= F}
res <- ANOVA_DDCt(
  data_3factor,
  numOfFactors = 3,
  numberOfrefGenes = 1,
  specs = "Conc",
  block = NULL)

model <- res$perGene$E_PO$lm
# Relative expression values for Concentration main effect
Means_DDCt(model, specs = "Conc")

# contrast        RE        SE df       LCL       UCL p.value sig
# L vs H   0.1703610 0.2208988 24 0.1242014 0.2336757 <0.0001 ***
# M vs H   0.2227247 0.2208988 24 0.1623772 0.3055004 <0.0001 ***
# M vs L   1.3073692 0.2208988 24 0.9531359 1.7932535  0.0928 .  
#
#Results are averaged over the levels of: Type, SA 
#Confidence level used: 0.95 

# Relative expression values for Concentration sliced by Type
Means_DDCt(model, specs = "Conc | Type")

# Type = R:
#  contrast       RE        SE df       LCL      UCL p.value sig
#  L vs H   0.103187 0.3123981 24 0.0659984 0.161331 <0.0001 ***
#  M vs H   0.339151 0.3123981 24 0.2169210 0.530255 <0.0001 ***
#  M vs L   3.286761 0.3123981 24 2.1022126 5.138776 <0.0001 ***
#
# Type = S:
#  contrast       RE        SE df       LCL      UCL p.value sig
#  L vs H   0.281265 0.3123981 24 0.1798969 0.439751 <0.0001 ***
#  M vs H   0.146266 0.3123981 24 0.0935518 0.228684 <0.0001 ***
#  M vs L   0.520030 0.3123981 24 0.3326112 0.813055  0.0059 ** 
#
# Results are averaged over the levels of: SA 
# Confidence level used: 0.95 

# Relative expression values for Concentration sliced by Type and SA
Means_DDCt(model, specs = "Conc | Type * SA")
```

# Checking normality of residuals

If the residuals from a `t.test` or an `lm` object are not normally distributed, the significance results might be violated. In such cases, non-parametric tests can be used. For example, the Mann–Whitney test - also known as the Wilcoxon rank-sum test, (implemented via `WILCOX_DDCt()` in the rtpcr package), is an alternative to t.test, and `kruskal.test()` is an alternative to one-way analysis of variance. These tests assess differences between population medians using independent groups. However, the rtpcr `TTEST_DDCt()` function includes the `var.equal` argument. When set to `FALSE`, performs `t.test` under the unequal variances hypothesis. Residuals from `ANOVA_DCt()` and `ANOVA_DDCt()` functions can be extracted from `lm`and plotted as follow:

```{r eval= F}
data <- read.csv(system.file("extdata", "data_repeated_measure_1.csv", package = "rtpcr"))
res3 <- ANOVA_DDCt(
  data,
  numOfFactors = 1,
  numberOfrefGenes = 1,
  specs = "time",
  block = NULL,
  model = wDCt ~ time + (1 | id))

residuals <- resid(res3$perGene$Target$lm)
shapiro.test(residuals) 
par(mfrow = c(1,2))
plot(residuals)
qqnorm(residuals)
qqline(residuals, col = "red")
```

# Mean of technical replicates
Calculating the mean of technical replicates and generating an output table suitable for subsequent expression analysis can be accomplished using the `meanTech()` function. The input dataset must follow the column structure illustrated in the example data below. Columns used for grouping should be explicitly specified via the `groups` argument of the `meanTech()` function.

```{r eval= F}
# Example input data frame with technical replicates
data1 <- read.csv(system.file("extdata", "data_withTechRep.csv", package = "rtpcr"))

# Calculate mean of technical replicates using first four columns as groups
meanTech(data1,
         groups = 2,
         numOfFactors = 1,
         block = NULL)
```


![Figure 11: The mean of technical replicates can be computed using the `meanTech()` function. ](../man/figures/techrep.png){.center height="380px"}


# Contact 
Email: gh.mirzaghaderi at uok.ac.ir

# Citation
```md
citation("rtpcr")

To cite the package ‘rtpcr’ in publications, please use:

  Ghader Mirzaghaderi (2025). rtpcr: a package for statistical analysis and graphical
  presentation of qPCR data in R. PeerJ 13:e20185. https://doi.org/10.7717/peerj.20185

A BibTeX entry for LaTeX users is

  @Article{,
    title = {rtpcr: A package for statistical analysis and graphical presentation of qPCR data in R},
    author = {Ghader Mirzaghaderi},
    journal = {PeerJ},
    volume = {13},
    pages = {e20185},
    year = {2025},
    doi = {10.7717/peerj.20185},
  }
```



# References
Livak, Kenneth J, and Thomas D Schmittgen. 2001. Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR and the Double Delta CT Method. Methods 25 (4). <a href="https://doi.org/10.1006/meth.2001.1262">doi.org/10.1006/meth.2001.1262</a>.

Ganger, MT, Dietz GD, Ewing SJ. 2017. A common base method for analysis of qPCR data and the application of simple blocking in qPCR experiments. BMC bioinformatics 18, 1-11. <a href="https://doi.org/10.1186/s12859-017-1949-5">doi.org/10.1186/s12859-017-1949-5</a>.

Mirzaghaderi G. 2025. rtpcr: a package for statistical analysis and graphical presentation of qPCR data in R. PeerJ 13, e20185. <a href="https://doi.org/10.7717/peerj.20185">doi.org/10.7717/peerj.20185</a>.

Pfaffl MW, Horgan GW, Dempfle L. 2002. Relative expression software tool (REST©) for group-wise comparison and statistical analysis of relative expression results in real-time PCR. Nucleic acids research 30, e36-e36. <a href="https://doi.org/10.1093/nar/30.9.e36">doi.org/10.1093/nar/30.9.e36</a>.

Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. 2019. The ultimate qPCR experiment: producing publication quality, reproducible data the first time. Trends in Biotechnology, 37(7), 761-774. <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">doi.org/10.1016/j.tibtech.2018.12.002</a>.

Yuan, JS, Ann Reed, Feng Chen, and Neal Stewart. 2006. Statistical Analysis of Real-Time PCR Data. BMC Bioinformatics 7 (85). <a href="https://doi.org/10.1186/1471-2105-7-85">doi.org/10.1186/1471-2105-7-85</a>.

---
title: "rtpcr: a package for statistical analysis of real-time PCR data in R"
author: Ghader Mirzaghaderi
output:
  html_document: 
    toc: true
    keep_md: true
    output: rmarkdown::html_vignette
    df_print: default
  pdf_document: 
    toc: true
    latex_engine: lualatex
  word_document:
    toc: No
vignette: |
  %\VignetteIndexEntry{Sending Messages With Gmailr}
  %\usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::knitr}
editor_options:
  markdown:
    wrap: 90
  chunk_output_type: console
---


```{r setup, include = FALSE, fig.align='center', warning = F, message=F}
options(tinytex.verbose = TRUE)
knitr::opts_chunk$set(echo = TRUE)
```

# Overview


Real-time polymerase chain reaction (real-time PCR) is widely used in biological studies. Various analysis methods are employed on the real-time PCR data to measure the mRNA levels under different experimental conditions. 
‘rtpcr’ package was developed for amplification efficiency calculation, statistical analysis and bar plot representation of real-time PCR data in R. By accounting for up to two reference genes and amplification efficiency values, a general calculation methodology described by <a href="https://doi.org/10.1186/s12859-017-1949-5">Ganger et al. (2017)</a>
and <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">Taylor et al. (2019)</a>, matching both <a href="https://doi.org/10.1006/meth.2001.1262">Livak and Schmittgen (2001)</a> and <a href="https://doi.org/10.1093/nar/30.9.e36">Pfaffl et al. (2002) </a> methods  was used. Based on the experimental conditions, the functions of the ‘rtpcr’ package use t-test (for experiments with a two-level factor), analysis of variance, analysis of covariance (ANCOVA) or analysis of repeated measure data to calculate the fold change (FC, ${\Delta\Delta C_t}$ method) or relative expression (RE, ${\Delta C_t}$ method). The functions further provide standard errors and confidence interval for means, apply statistical mean comparisons and present significance. To facilitate function application, different data sets were used as examples and the outputs were explained. An outstanding feature of ‘rtpcr’ package is providing publication-ready bar plots with various controlling arguments for experiments with up to three different factors which are further editable by ggplot2 functions.

# Calculation methods
The basic method for expression estimation of a gene between conditions relies on the calculation of fold differences by applying the PCR amplification efficiency (E) and the threshold cycle (syn. crossing point or Ct). Among the various approaches developed for data analysis in real-time PCR, the Livak approach, also known as the $2^{-\Delta\Delta C_t}$ method, stands out for its simplicity and widespread use where the fold change (FC) exoression $(2^{-\Delta\Delta C_t})$ in Treatment (Tr) compared to Control (Co) condition is calculated according to equation:


$$\begin{align*}
\text{Fold change} & = 2^{-\Delta\Delta C_t} \\
& = \frac{2^{-(C_{t_{\text{target}}}-C_{t_{\text{ref}}})_{Tr}}}
{2^{-(C_{t_{\text{target}}}-C_{t_{\text{ref}}})_{Co}}} \\ 
& =2^{-[(C_{t_{\text{target}}}-C_{t_{\text{ref}}})_{\text{Tr}}-
{(C_{t_{\text{target}}}-C_{t_{\text{ref}}})}_{\text{Co}}]} \\ 
& = 2^{-[{(\Delta C_t)_{Tr} - (\Delta C_t)_{Co}}]}
\end{align*}$$


Here, $\Delta C_t$ is the difference between target Ct and reference Ct values for a given sample. Livak method assumes that both the target and reference genes are amplified with efficiencies close to 100%, allowing for the relative quantification of gene expression levels.

On the other hand, the Pfaffl method offers a more flexible approach by accounting for differences in amplification efficiencies between the target and reference genes. This method adjusts the calculated expression ratio by incorporating the specific amplification efficiencies, thus providing a more accurate representation of the relative gene expression levels.

$$\text{Fold change} = \frac{E^{-(C_{t_{\text{Tr}}}-C_{t_{\text{Co}}})_{target}}}
{E^{-(C_{t_{\text{Tr}}}-C_{t_{\text{Co}}})_{ref}}}$$

# A generalized calculation method
The `rtpcr` package was developed for the R environment in the major operating systems. The package functions are mainly based on the calculation of efficiency-weighted $\Delta C_t$ $(w\Delta C_t)$ values from target and reference gene Ct (equation 3). $w\Delta C_t$  values are weighted for the amplification efficiencies as described by Ganger et al. (2017) except that log2 is used instead of log10:


$$w\Delta Ct =\log_{2}(E_{target}).Ct_{target}-\log_{2}(E_{ref}).Ct_{ref}$$


The relative expression of the target gene normalized to that of reference gene(s) within the same sample or condition is called relative expression (RE). From the mean $w\Delta C_t$ values over biological replicates, RE of a target gene can be calculated for each condition according to the equation 

$$\text{Relative Expression} = 2^{-\overline{w\Delta Ct}}$$
Relative expression is only calibrated for the reference gene(s) and not for a control condition. However, often one condition is considered as calibrator and the fold change (FC) expression in other conditions is calculated relative to the calibrator. Examples are Treatment versus Control where Control is served as the calibrator, or time 0 versus time 1 (e.g. after 1 hour) and time 2 (e.g. after 2 hours) where time 0 is served as the reference or calibrator level. So, calibrator is the reference level or sample that all others are compared to. The fold change (FC) expression of a target gene for the reference or calibrator level is 1 because it is not changed compared to itself. The fold change expression of a target gene due to the treatment can be calculated as follows: 

$$\text{Fold Change due to Treatment}=2^{-(\overline{w\Delta Ct}_{\text{Tr}}-{\overline{w\Delta Ct}_{\text{Co}}})}$$

`qpcrTTEST` and `qpcrTTESTplot` functions calculate FC for multi-genes-two conditional cases, `qpcrANOVAFC` represents FC for single-gene-factorial (single- or multi-factor) experiments, and `qpcrREPEATED` calculates FC for the repeated measure data. If $w \Delta C_t$ values is calculated from the E values, the calculations match the formula of Pfaffl while if 2 (complete efficiency) be used instead, the result match the $2^{-\Delta\Delta C_t}$ method. In any case we called these as Fold Change in the outputs of `rtpcr`. Under factorial experiments where the calculation of the expression of the target gene relative to the reference gene (called Relative Expression) in each condition is desired, `qpcrANOVARE`, `oneFACTORplot`, `twoFACTORplot` and `threeFACTORplot` functions were developed for ANOVA analysis, and representing the plots from single, double or triple factor experiments, respectively. The last three functions generate `ggplot2`-derived graphs based on the output of the `qpcrANOVARE` function. If available, the blocking factor can also be handled by `qpcrANOVARE`, `qpcrANOVAFC` and `qpcrREPEATED` functions. Standard error of the FC and RE means is calculated according to <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">Taylor et al. (2019)</a> in `rtpcr` package.
Here, a brief methodology is presented but details about the $w\Delta C_t$  calculations and statistical analysis are available in <a href="https://doi.org/10.1186/s12859-017-1949-5">Ganger et al. (2017)</a>. Importantly, because both the RE or FC gene expression values follow a lognormal distribution, a normal distribution is expected for the $w \Delta C_t$ values making it possible to apply t-test or analysis of variance to them. Following analysis, $w\Delta C_t$ values are statistically compared and standard deviations and confidence interval are calculated, but the transformation $y = 2^{-x}$ is applied in the final step in order to report the results.





# Installing and loading

```{r eval= T, include= F, message=FALSE, warning = FALSE}
library(rtpcr)
library(multcomp)
library(dplyr)
library(reshape2)
library(tidyr)
library(ggplot2)
library(grid)
```

The `rtpcr` package can be installed and loaded using:
```r
install.packages("rtpcr")
library(rtpcr)
```

Alternatively, the `rtpcr` with the latest changes can be installed by running the following code in your R software: 

```{r eval= F, include= T, message=FALSE, warning = FALSE}

# install `rtpcr` from github (under development)

devtools::install_github("mirzaghaderi/rtpcr")

# I strongly recommend to install the package with the vignette as it contains information about how to use the 'rtpcr' package. Through the following code, Vignette is installed as well.

devtools::install_github("mirzaghaderi/rtpcr", build_vignettes = TRUE)
```



# Data structure and column arrangement

To use the functions, input data should be prepared in the right format with appropriate column arrangement. The correct column arrangement is shown in Table 1 and Table 2. For `qpcrANOVAFC` or `qpcrANOVARE` analysis, ensure that each line in the data set belongs to a separate individual or biological replicate reflecting a non-repeated measure experiment.

*Table 1. Data structure and column arrangement required for ‘rtpcr’ package.  rep: technical replicate; targetE and refE: amplification efficiency columns for target and reference genes, respectively. targetCt and refCt: target gene and reference gene Ct columns, respectively. factors (up to three factors is allowed): experimental factors.*

 | Experiment type     |  Column arrangement of the input data   | Example in the package                   |
 |:---------------------|:---------------------------------------|:------------------------------------------|
 |Amplification efficiency             |Dilutions - geneCt ... | data_efficiency |
 |t-test (accepts multiple genes)      |condition (put the control level first) - gene (put reference gene(s) last.)- efficiency - Ct  | data_ttest |
 |Factorial (Up to three factors)      |factor1 - rep - targetE - targetCt - refE - refCt | data_1factor |
 |                                     |factor1 - factor2 - rep - targetE - targetCt - refE - refCt | data_2factor |
 |                                     |factor1 - factor2 - factor3 - rep - targetE - targetCt - refE - refCt | data_3factor |
 |Factorial with blocking              |factor1 - block - rep - targetE - targetCt - refE - refCt | |
 |                                     |factor1 - factor2 - block - rep - targetE - targetCt - refE - refCt	 | data_2factorBlock |
 |                                     |factor1 - factor2 - factor3 - block - rep - targetE - targetCt - refE - refCt | |
 |Two reference genes                  |. . . . . .  rep - targetE - targetCt - ref1E - ref1Ct - ref2E - ref2Ct | |
 |calculating biological replicated    |. . . . . .  biologicalRep - techcicalRep - Etarget - targetCt - Eref - refCt  | data_withTechRep |
 |                |. . . . . .  biologicalRep - techcicalRep - Etarget - targetCt - ref1E - ref1Ct - ref2E - ref2Ct  | |
 

NOTE: For `qpcrANOVAFC` or `qpcrANOVARE` analysis, each line in the input data set belongs to a separate individual or biological replicate reflecting a non-repeated measure experiment.




*Table 2. Repeated measure data structure and column arrangement required for the `qpcrREPEATED` function.  targetE and refE: amplification efficiency columns for target and reference genes, respectively. targetCt and refCt: Ct columns for target and reference genes, respectively. In the "id" column, a unique number is assigned to each individual, e.g. all the three number 1 indicate a single individual.*

 | Experiment type      |  Column arrangement of the input data   | Example in the package                   |
 |:---------------------|:----------------------------------------|:------------------------------------------|
 |Repeated measure      | id - time - targetE - targetCt - ref1E - ref1Ct | data_repeated_measure_1 |
 |                      | id - time - targetE - targetCt - ref1E - ref1Ct - ref2E - ref2Ct | |
  |Repeated measure     | id - treatment - time - targetE - targetCt - ref1E - ref1Ct | data_repeated_measure_2 |
 |                      | id - treatment - time - targetE - targetCt - ref1E - ref1Ct - ref2E - ref2Ct | |



To see list of data in the `rtpcr` package run `data(package = "rtpcr")`. 
Example data sets can be presented by running the name of each data set. A description of the columns names in  each data set is called by "?" followed by the names of the data set, for example `?data_1factor`

# functions usage

To simplify `rtpcr` usage, examples for using the functions are presented below.


           
*Table 3. Functions and examples for using them.*

 | function             |                 Analysis | Example (see package help for more arguments) |
 |:---------------------|:-----------------------------------|:----------------------------------|
 | efficiency           | Efficiency, standard curves and related statistics | efficiency(data_efficiency) |
 | meanTech             | Calculating the mean of technical replicates | meanTech(data_withTechRep, groups = 1:4) |
 | qpcrANOVAFC          | FC and bar plot of the target gene (one or multi-factorial experiments) | qpcrANOVAFC(data_1factor, numberOfrefGenes = 1, mainFactor.column = 1, mainFactor.level.order = c("L1", "L2", "L3") |
 |  oneFACTORplot       | Bar plot of the relative gene expression from a one-factor experiment | out <- qpcrANOVARE(data_1factor, numberOfrefGenes = 1)\$Result;  oneFACTORplot(out, errorbar = "se") |
 |  qpcrANOVARE         | Analysis of Variance of the qpcr data  | qpcrANOVARE(data_3factor, numberOfrefGenes = 1) |
 | qpcrTTEST            | Computing the fold change and related statistics | qpcrTTEST(data_ttest,  numberOfrefGenes = 1, paired = FALSE, var.equal = TRUE, p.adj = "none") |
 | qpcrTTESTplot        | Bar plot of the average fold change of the target genes	 | qpcrTTESTplot(data_ttest,  numberOfrefGenes = 1, order = c("C2H2-01", "C2H2-12", "C2H2-26")) |
 |  threeFACTORplot     | Bar plot of the relative gene expression from a three-factor experiment | res <- qpcrANOVARE(data_3factor,  numberOfrefGenes = 1)\$Result; threeFACTORplot(res, 
 arrangement = c(3, 1, 2), errorbar = "se") |
 | twoFACTORplot        | Bar plot of the relative gene expression from a two-factor experiment | res <- qpcrANOVARE(data_2factor, numberOfrefGenes = 1)\$Result; twoFACTORplot(res, x.axis.factor = Genotype, group.factor = Drought, errorbar = "se") | 
 | qpcrREPEATED        | Bar plot of the fold change expression for repeated measure observations (taken over time from each individual) |  qpcrREPEATED(data_repeated_measure_2, numberOfrefGenes = 1), factor = "time" |
 

 
*see package help for more arguments including the number of reference genes, levels arrangement, blocking, and arguments for adjusting the bar plots.*
 

# Amplification efficiency data analysis
## Sample data of amplification efficiency

To calculate the amplification efficiencies of a target and a reference gene, a data frame should be prepared with 3 columns of dilutions, target gene Ct values, and reference gene Ct values, respectively,  as shown below.

```{r eval= T}
data_efficiency
```


	



## Calculating amplification efficiency

Amplification efficiency in PCR can be either defined as percentage (from 0 to 1) or as time of PCR product increase per cycle (from 1 to 2).  in the `rtpcr` package, the amplification efficiency (E) has been referred to times of PCR product increase (1 to 2). A complete efficiency is equal to 2.  If dilutions and Ct values are available for a number of genes, the `efficiency` function calculates the amplification efficiency of genes and presents the related standard curves along with the Slope, Efficiency, and R2 statistics. The function also does pairwise comparisons of the slopes for the genes. For this, a regression line is fitted forst using the $\Delta C_t$ values of each pair of genes.



```{r eval = T, , fig.height = 3, fig.width = 5, fig.align = 'center', fig.cap = 'Standard curve and the amplification efficiency analysis of genes. Required iput data include dilutions and Ct value columns for different genes.', warning = FALSE, message = FALSE}
efficiency(data_efficiency)
```


Note: It is advised that the amplification efficiency be calculated for each cDNA sample because the amplification efficiency not only depends on the PCR mic and primer characteristics, but also varies among different cDNA samples.


# Expression data analysis

## Target genes in two conditions (t-test)

### Example data
When a target gene is assessed under two different conditions (for example Control and treatment), it is possible to calculate the average fold change expression $({\Delta \Delta C_t}$ method) of the target gene in treatment relative to control conditions. For this, the data should be prepared according to the following data set consisting of 4 columns belonging to condition levels, E (efficiency), genes and Ct values, respectively. Each Ct value is the mean of technical replicates. Complete amplification efficiencies of 2 have been assumed here for all wells but the calculated efficiencies can be used instead. 

```{r eval= T, fig.height = 3, fig.width = 5, fig.align = 'center'}
data_ttest
```

### Data analysis under two conditions

Here, the above data set was used for the Fold Change expression analysis of the target genes using the `qpcrTTEST` function. This function performs a t-test-based analysis of any number of genes that have been evaluated under control and treatment conditions. The output is a table of target gene names, fold changes confidence limits, and the t.test derived p-values. The `qpcrTTEST` function includes the `var.equal` argument. When set to `FALSE`, `t.test` is performed under the unequal variances hypothesis. Furthermore, the samples in qPCR may be unpaired or paired so the analysis can be done for unpaired or paired conditions. Paired samples refer to a situation where the measurements are made on the same set of subjects or individuals. This could occur if the data is acquired from the same set of individuals before and after a treatment. In such cases, the paired t-test is used for statistical comparisons by setting the t-test `paired` argument to `TRUE`.

```{r eval= T}
qpcrTTEST(data_ttest, 
          numberOfrefGenes = 1,
          paired = F, 
          var.equal = T,
          p.adj = "none")
```


### Generating plot
The `qpcrTTESTplot` function generates a bar plot of fold changes (FC) and confidence intervals for the target genes. the `qpcrTTESTplot` function accepts any number of genes and any replicates. The `qpcrTTESTplot` function automatically puts appropriate signs of **, * on top of the plot columns based on the output p-values.


```{r eval= T, fig.height=3, fig.width=8, fig.align='center', fig.cap = "Average Fold changes of three target genes relative to the control condition computed by unpaired t-tests via `qpcrTTESTplot` function. Confidence interval (ci) and standard error (se) has been used as error bar in 'A' and 'B', respectively.", warning = F, message = F}

# Producing the plot
t1 <- qpcrTTESTplot(data_ttest,
              numberOfrefGenes = 1,
              fontsizePvalue = 4,
              errorbar = "ci",
              p.adj = "none")

# Producing the plot: specifying gene order
t2 <- qpcrTTESTplot(data_ttest,
              numberOfrefGenes = 1,
              order = c("C2H2-01", "C2H2-12", "C2H2-26"),
              paired = FALSE,
              var.equal = TRUE,
              width = 0.5,
              fill = "palegreen",
              y.axis.adjust = 3,
              y.axis.by = 2,
              ylab = "Average Fold Change",
              xlab = "none",
              fontsizePvalue = 4)

multiplot(t1, t2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
```

## Uni- or multi-factorial experiments: Fold Change (FC) analysis

In the rtpcr package, the fold change analysis of a target gene is applied based on analysis of variance; ANOVA (or analysis of covariance; ANCOVA) using the `qpcrANOVAFC` function. the default argument is `anova` but can be changes to `ancova` as well. the ancova analysis is suitable when the levels of a factor are also affected by an uncontrolled quantitative covariate. For example, suppose that wDCt of a target gene in a plant is affected by temperature. The gene may also be affected by drought. since we already know that temperature affects the target gene, we are interesting now if the gene expression is also altered by the drought levels. We can design an experiment to understand the gene behavior at both temperature and drought levels at the same time. The drought is another factor (the covariate) that may affect the expression of our gene under the levels of the first factor i.e. temperature. The data of such an experiment can be analyzed by both ANCOVA or factorial ANOVA using `qpcrANOVAFC` function. The function also works for one factor experiment as well. Bar plot of fold changes (FC values) along with the 95\% confidence interval is also returned by the `qpcrANOVAFC` function. There is also a function called `oneFACTORplot` which returns FC values and related plot for a one-factor-experiment with more than two levels. 

```{r eval = T, fig.height = 3, fig.width = 5, fig.align='center', fig.cap = "Statistical table and figure of the Fold change expression of a gene in three different levels of Drough stress relative to the D0 as reference or calibrator level produced by the `qpcrANOVAFC` function. The other factor i.e. Genotype has been concidered as covariate."}
# See sample data
data_2factor

qpcrANOVAFC(data_2factor, 
           numberOfrefGenes = 1, 
           block = NULL,
           analysisType = "ancova",
           mainFactor.column = 2,
           fontsizePvalue = 4,
           x.axis.labels.rename = "none")
```


## Uni- or multi-factorial experiments: Relative Expression (RE) analysis

the `qpcrANOVARE` function performs Relative Expression (RE) analysis for uni- or multi-factorial experiments in which all factor level combinations are used as treatments. The input data set should be prepared as shown in table 1. Factor columns should be presented first followed by blocking factor (if available), biological replicates and efficiency and Ct values of target and reference gene(s). The example data set below (`data_3factor`) represents amplification efficiency and Ct values for target and reference genes under three grouping factors (two different cultivars, three drought levels, and the presence or absence of bacteria). Here, the efficiency value of 2 has been used for all wells, but the calculated efficiencies can be used instead.


```{r eval= T}
# See a sample dataset
data_3factor
```



### Output table of the analysis

The `qpcrANOVARE` function produces the main analysis output including mean wDCt, LCL, UCL, grouping letters, and standard deviations for relative expression values. The standard deviation for each treatment is derived from the biological replicates of back-transformed wDCt data.

```{r eval= T, fig.height = 3, fig.width = 5}
# If the data include technical replicates, means of technical replicates
# should be calculated first using meanTech function.

# Applying ANOVA analysis
res <- qpcrANOVARE(data_2factor,
                   numberOfrefGenes = 1,
                   block = NULL)
res$Result
res$Post_hoc_Test
```



```{r eval= T, fig.height = 4, fig.width = 9, fig.align = 'center', fig.cap = "A: bar plot representing Relative expression of a gene under three levels of a factor generated using `oneFACTORplot` function, B: Plot of the Fold change expression produced by the `qpcrANOVAFC` function from the same data used for 'A'. The first element in the `mainFactor.level.order` argument (here L1) is served as the Reference level, although the x-axis names have later been renamed by the `x.axis.labels.rename` argument. Error bars represent 95% confidence interval in A and standard error in B."}

# Before plotting, the result needs to be extracted as below:
out2 <- qpcrANOVARE(data_1factor, numberOfrefGenes = 1, block = NULL)$Result

f1 <- oneFACTORplot(out2,
              width = 0.2,
              fill = "skyblue",
              y.axis.adjust = 0.5,
              y.axis.by = 1,
              errorbar = "ci",
              show.letters = TRUE,
              letter.position.adjust = 0.1,
              ylab = "Relative Expression",
              xlab = "Factor Levels",
              fontsize = 12,
              fontsizePvalue = 4)


addline_format <- function(x,...){
    gsub('\\s','\n',x)
}
f2 <- qpcrANOVAFC(data_1factor,
                 numberOfrefGenes = 1,
                 mainFactor.column = 1,
                 block = NULL,
                 mainFactor.level.order = c("L1","L2","L3"),
                 width = 0.5,
                 fill = c("skyblue","#79CDCD"),
                 y.axis.by = 1,
                 letter.position.adjust = 0,
                 y.axis.adjust = 1,
                 ylab = "Fold Change",
                 fontsize = 12, plot = F,
                 x.axis.labels.rename = addline_format(c("Control", 
                                                       "Treatment_1 vs Control", 
                                                       "Treatment_2 vs Control")))


multiplot(f1, f2$FC_Plot_of_the_main_factor_levels, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
```




### Barplot with the (1-alpha)% confidence interval as error bars

```{r eval= T, include = T, fig.height = 4, fig.width = 9, fig.align = 'center', fig.cap = "Relative expression of a target gene under two different factors of genotype (with two levels) and drought (with three levels). Error bars represent standard error. Means (columns) lacking letters in common have significant difference at alpha = 0.05 as resulted from a `LSD.test`."}

# Before plotting, the result needs to be extracted as below:
res <- qpcrANOVARE(data_2factor, numberOfrefGenes = 1, block = NULL)$Result
Final_data <- qpcrANOVARE(data_2factor, numberOfrefGenes = 1, block = NULL)$Final_data

# Plot of the 'res' data with 'Genotype' as grouping factor
q1 <- twoFACTORplot(res,
   x.axis.factor = Drought,
   group.factor = Genotype,
   errorbar = "se",
   width = 0.5,
   fill = "Greens",
   y.axis.adjust = 0.5,
   y.axis.by = 2,
   ylab = "Relative Expression",
   xlab = "Drought Levels",
   legend.position = c(0.15, 0.8),
   show.letters = TRUE,
   fontsizePvalue = 4)

# Plotting the same data with 'Drought' as grouping factor
q2 <- twoFACTORplot(res,
   x.axis.factor = Genotype,
   group.factor = Drought,
   errorbar = "se",
   xlab = "Genotype",
   fill = "Blues",
   legend.position = c(0.15, 0.8),
   show.letters = FALSE,
   show.errorbars = T,
   fontsizePvalue = 4)

multiplot(q1, q2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
```



### A three-factorial experiment example

```{r, fig.height = 5, fig.width = 11, fig.align = 'center', fig.cap = "A and B) Relative expression (RE) of a target gene from a three-factorial experiment data produced by  `threeFACTORplot`  function. Error bars represent standard error (A), although can be set to confidence interval (B). Means (columns) lacking letters in common have significant differences at alpha = 0.05 as resulted from an ‘LSD.test’."}
# Before plotting, the result needs to be extracted as below:
res <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Result
res

# releveling a factor levels first
res$Conc <- factor(res$Conc, levels = c("L","M","H"))
res$Type <- factor(res$Type, levels = c("S","R"))

# Arrange the first three colunms of the result table.
# This determines the columns order and shapes the plot output.
p1 <- threeFACTORplot(res,
    arrangement = c(3, 1, 2),
    errorbar = "se",
    legend.position = c(0.2, 0.85),
    xlab = "condition",
    fontsizePvalue = 4)


# When using ci as error, increase y.axis.adjust to see the plot correctly!
p2 <- threeFACTORplot(res,
   arrangement = c(2, 3, 1),
   bar.width = 0.8,
   fill = "Greens",
   xlab = "Drought",
   ylab = "Relative Expression",
   errorbar = "ci",
   y.axis.adjust = 2,
   y.axis.by = 2,
   letter.position.adjust = 0.6,
   legend.title = "Genotype",
   fontsize = 12,
   legend.position = c(0.2, 0.8),
   show.letters = TRUE,
   fontsizePvalue = 4)

multiplot(p1, p2, cols = 2)
grid.text("A", x = 0.02, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
grid.text("B", x = 0.52, y = 1, just = c("right", "top"), gp=gpar(fontsize=16))
```


## Repeated measure data in qpcr

Fold change (FC) analysis of observations repeatedly taken over time `qpcrREPEATED` function, for Repeated measure analysis of uni- or multi-factorial experiment data. The bar plot of the fold changes (FC) values along with the standard error (se) or confidence interval (ci) is also returned by the `qpcrREPEATED` function.

```{r, eval=T, fig.height = 4, fig.width = 7, fig.align = 'center', fig.cap = "Fold change expression (FC) of a target gene from a one and a two factorial experiment data produced by  `qpcrREPEATED`  function. Error bars represent standard error (A), although can be set to confidence interval."}

a <- qpcrREPEATED(data_repeated_measure_1,
                  numberOfrefGenes = 1,
                  block = NULL,
                  fill = c("#778899", "#BCD2EE"),
                  factor = "time",
                  axis.text.x.angle = 45,
                  axis.text.x.hjust = 1,
                  plot = F)

b <- qpcrREPEATED(data_repeated_measure_2,
                  numberOfrefGenes = 1,
                  factor = "time",
                  block = NULL,
                  axis.text.x.angle = 45,
                  axis.text.x.hjust = 1,
                  plot = F)

multiplot(a, b, cols = 2)
```



## Fold change analysis using a model

The `qpcrMeans` performs fold change ${\Delta \Delta C_t}$ mwthid analysis using a model produced by the
`qpcrANOVAFC` or `qpcrREPEATED.` The values can be returned for any effects in the model including simple effects,
interactions and slicing if an ANOVA model is used, but ANCOVA models returned by rtpcr package only include simple effects.

```{r, eval=T}
# Returning fold change values from a fitted model.
# Firstly, result of `qpcrANOVAFC` or `qpcrREPEATED` is 
# acquired which includes a model object:
res <- qpcrANOVAFC(data_3factor, numberOfrefGenes = 1, mainFactor.column = 1, block = NULL)

# Returning fold change values of Conc levels from a fitted model:
qpcrMeans(res$lm_ANOVA, specs = "Conc")


# Returning fold change values of Conc levels sliced by Type*SA:
qpcrMeans(res$lm_ANOVA, specs = "Conc | (Type*SA)")

# Returning fold change values of Conc
qpcrMeans(res$lm_ANOVA, specs = "Conc * Type")

# Returning fold change values of Conc levels sliced by Type:
res2 <- qpcrMeans(res$lm_ANOVA, specs = "Conc | Type")
twoFACTORplot(res2, x.axis.factor = contrast, ylab = "Fold Change",
              group.factor = Type, errorbar = "ci")
```





# An example of showing point on the plot
```{r, eval=T, include = T, fig.height = 4, fig.width = 6, fig.align = 'center'}

library(ggplot2)
b <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Result
a <- qpcrANOVARE(data_3factor, numberOfrefGenes = 1, block = NULL)$Final_data

# Arrange factor levels to your desired order:
b$Conc <- factor(b$Conc, levels = c("L","M","H"))
a$Conc <- factor(a$Conc, levels = c("L","M","H"))

# Generating plot
ggplot(b, aes(x = Type, y = RE, fill = factor(Conc))) +
  geom_bar(stat = "identity", position = "dodge") +
  facet_wrap(~ SA) +
  scale_fill_brewer(palette = "Reds") +
  xlab("Type") +
  ylab("Relative Expression") +
  geom_point(data = a, aes(x = Type, y = (2^(-wDCt)), fill = factor(Conc)), 
             position = position_dodge(width = 0.9), color = "black") +
  ylab("ylab") +
  xlab("xlab") +
  theme_bw() +
  theme(axis.text.x = element_text(size = 12, color = "black", angle = 0, hjust = 0.5),
        axis.text.y = element_text(size = 12, color = "black", angle = 0, hjust = 0.5),
        axis.title  = element_text(size = 12),
        legend.text = element_text(size = 12)) +
  theme(legend.position  = c(0.2, 0.7)) +
  theme(legend.title = element_text(size = 12, color = "black")) +
  scale_y_continuous(breaks = seq(0, max(b$RE) + max(b$se) + 0.1, by = 5), 
                     limits = c(0, max(b$RE) + max(b$se) + 0.1), expand = c(0, 0)) 
```



# Checking normality of residuals

If the residuals from a `t.test` or an `lm` or and `lmer` object are not normally distributed, the significance results might be violated. In such cases, one could use non-parametric tests such as the Mann-Whitney test (also known as the Wilcoxon rank-sum test), `wilcox.test()`, which is an alternative to `t.test`, or the `kruskal.test()` test which alternative to one-way analysis of variance, to test the difference between medians of the populations using independent samples. However, the `t.test` function (along with the `qpcrTTEST` function described above) includes the `var.equal` argument. When set to `FALSE`, perform `t.test` under the unequal variances hypothesis. Residuals for `lm` (from `qpcrANOVARE` and `qpcrANOVAFC` functions) and `lmer` (from `qpcrREPEATED` function) objects can be extracted and plotted as follow:

```{r eval= T, eval= T, fig.height = 5, fig.width = 10, fig.align = 'center', fig.cap = "QQ-plot for the normality assessment of the residuals derived from `t.test` or `lm` functions."}

residuals <- qpcrANOVARE(data_1factor, numberOfrefGenes = 1, block = NULL)$lmCRD$residuals
shapiro.test(residuals) 

par(mfrow = c(1,2))
plot(residuals)
qqnorm(residuals)
qqline(residuals, col = "red")
```


For the repeated measure models, residulas can be extracted by `residuals(a$lm)` and plotted by `plot(residuals(a$lm))` where 'a' is an object created by the `qpcrREPEATED` function.

```{r eval= T, eval= T, fig.height = 4, fig.width = 4, fig.align = 'center', fig.cap = "QQ-plot for the normality assessment of the residuals derived from `t.test` or `lm` functions."}

a <- qpcrREPEATED(data_repeated_measure_2, 
                  numberOfrefGenes = 1, 
                  factor = "time", 
                  block = NULL,
                  y.axis.adjust = 1.5)


residuals(a$lm)
plot(residuals(a$lm))
qqnorm(residuals(a$lm))
qqline(residuals(a$lm), col = "red")
```

# Mean of technical replicates
Calculating the mean of technical replicates and getting an output table appropriate for subsequent ANOVA analysis can be done using the `meanTech` function. For this, the input data set should follow the column arrangement of the following example data. Grouping columns must be specified under the `groups` argument of the `meanTech` function.

```{r eval= T}
# See example input data frame:
data_withTechRep

# Calculating mean of technical replicates
meanTech(data_withTechRep, groups = 1:4)
```


# Combining FC results of different genes

`qpcrANOVAFC` and `twoFACTORplot` functions give FC results for one gene each time. you can combine FC tables of different genes and present their bar plot by `twoFACTORplot` function. An example has been shown bellow for two genes, however it works for any number of genes.

```{r eval= T, eval= T, , fig.height = 4, fig.width = 5, fig.align = 'center', fig.cap = "Fold change expression of two different genes. FC tables of any number of genes can be combined and used as input data frame for `twoFACTORplot` function."}

a <- qpcrREPEATED(data_repeated_measure_1,
             numberOfrefGenes = 1,
             factor = "time", block = NULL)

b <- qpcrREPEATED(data_repeated_measure_2,
                  factor = "time",
                  numberOfrefGenes = 1, block = NULL)


a1 <- a$FC_statistics_of_the_main_factor
b1 <- b$FC_statistics_of_the_main_factor

c <- rbind(a1, b1)
c$gene <- factor(c(1,1,1,2,2,2))
c

twoFACTORplot(c, x.axis.factor = contrast, 
              group.factor = gene, fill = 'Reds', errorbar = "se",
              ylab = "FC", axis.text.x.angle = 45, y.axis.adjust = 1.5,
              axis.text.x.hjust = 1, legend.position = c(0.2, 0.8))
```


# How to edit ouptput graphs?

the rtpcr graphical functions create a list containing the ggplot object so for editing or adding new layers to the graph output, you need to extract the ggplot object first:

```{r eval= F, include = T, fig.height = 4, fig.width = 5}

b <- qpcrANOVAFC(data_2factor, 
            numberOfrefGenes = 1,
            mainFactor.column = 1,
            block = NULL,
            mainFactor.level.order = c("S", "R"),
            fill = c("#CDC673", "#EEDD82"),
            analysisType = "ancova",
            fontsizePvalue = 7,
            y.axis.adjust = 0.1, width = 0.35)



library(ggplot2)
p2 <- b$FC_Plot_of_the_main_factor_levels
p2 + theme_set(theme_classic(base_size = 20))

```


# Citation

```{r eval= F}
citation("rtpcr")
```


# Contact 
Email: gh.mirzaghaderi at uok.ac.ir


# References
Livak, Kenneth J, and Thomas D Schmittgen. 2001. Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR and the Double Delta CT Method. Methods 25 (4). <a href="https://doi.org/10.1006/meth.2001.1262">doi.org/10.1006/meth.2001.1262</a>.

Ganger, MT, Dietz GD, Ewing SJ. 2017. A common base method for analysis of qPCR data and the application of simple blocking in qPCR experiments. BMC bioinformatics 18, 1-11. <a href="https://doi.org/10.1186/s12859-017-1949-5">doi.org/10.1186/s12859-017-1949-5</a>.

Pfaffl MW, Horgan GW, Dempfle L. 2002. Relative expression software tool (REST©) for group-wise comparison and statistical analysis of relative expression results in real-time PCR. Nucleic acids research 30, e36-e36. <a href="https://doi.org/10.1093/nar/30.9.e36">doi.org/10.1093/nar/30.9.e36</a>.

Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. 2019. The ultimate qPCR experiment: producing publication quality, reproducible data the first time. Trends in Biotechnology, 37(7), 761-774<a href="https://doi.org/10.1016/j.tibtech.2018.12.002">doi.org/10.1016/j.tibtech.2018.12.002</a>.

Yuan, JS, Ann Reed, Feng Chen, and Neal Stewart. 2006. Statistical Analysis of Real-Time PCR Data. BMC Bioinformatics 7 (85). <a href="https://doi.org/10.1186/1471-2105-7-85">doi.org/10.1186/1471-2105-7-85</a>.


.

---
title: "Calculation method"
output: html_vignette
link-citations: true
vignette: >
  %\VignetteIndexEntry{Calculation method}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include = FALSE, fig.align='center', warning = F, message=F}
options(tinytex.verbose = TRUE)
knitr::opts_chunk$set(echo = TRUE)
library(rtpcr)
```



Real-time polymerase chain reaction (real-time PCR) is widely used in biological studies. Various analysis methods are employed on the real-time PCR data to measure the mRNA levels under different experimental conditions. 


‘rtpcr’ package was developed for amplification efficiency calculation, statistical analysis and bar plot representation of real-time PCR data in R. By accounting for up to two reference genes and amplification efficiency values, a general calculation methodology described by <a href="https://doi.org/10.1186/s12859-017-1949-5">Ganger et al. (2017)</a>
  and <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">Taylor et al. (2019)</a>, matching both <a href="https://doi.org/10.1006/meth.2001.1262">Livak and Schmittgen (2001)</a> and <a href="https://doi.org/10.1093/nar/30.9.e36">Pfaffl et al. (2002) </a> methods  was used. Based on the experimental conditions, the functions of the ‘rtpcr’ package use t-test (for experiments with a two-level factor), analysis of variance, analysis of covariance (ANCOVA) or analysis of repeated measure data to calculate the ${\Delta\Delta C_t}$ method or ${\Delta C_t}$ method. The functions further provide standard errors and confidence interval for means, apply statistical mean comparisons and present significance. To facilitate function application, different data sets were used as examples and the outputs were explained. An outstanding feature of ‘rtpcr’ package is providing publication-ready bar plots with various controlling arguments for experiments with up to three different factors which are further editable by ggplot2 functions.
  
# Calculations
  
The basic method for estimating gene expression between conditions relies on calculating fold differences using the PCR amplification efficiency ($E$) and the threshold cycle (also called the crossing point, $C_t$). Among the various approaches for analyzing real-time PCR data, the Livak method—also known as the $2^{-\Delta\Delta C_t}$ method—is widely used for its simplicity and reliability. In this method, the fold change (FC) expression, $2^{-\Delta\Delta C_t}$, in a Treatment (Tr) group relative to a Control (Co) group is calculated as follows:
    
    



$$
  \begin{align*}
\text{Fold change}
&= 2^{-\Delta\Delta C_t} \\
&= \frac{2^{-(C_{t,\text{target}} - C_{t,\text{ref}})_{\text{Tr}}}}
{2^{-(C_{t,\text{target}} - C_{t,\text{ref}})_{\text{Co}}}} \\
&= 2^{-[(C_{t,\text{target}} - C_{t,\text{ref}})_{\text{Tr}}
        - (C_{t,\text{target}} - C_{t,\text{ref}})_{\text{Co}}]} \\
&= 2^{-[(\Delta C_t)_{\text{Tr}} - (\Delta C_t)_{\text{Co}}]}
\end{align*}
$$
  
  
  

Here, $\Delta C_t$ is the difference between target Ct and reference Ct values for a given sample. Livak method assumes that both the target and reference genes are amplified with efficiencies close to 100%, allowing for the relative quantification of gene expression levels.

On the other hand, the Pfaffl method offers a more flexible approach by accounting for differences in amplification efficiencies between the target and reference genes. This method adjusts the calculated expression ratio by incorporating the specific amplification efficiencies, thus providing a more accurate representation of the relative gene expression levels.

$$
  \text{Fold change} =
  \frac{E_{\text{target}}^{-(C_{t,\text{Tr}} - C_{t,\text{Co}})}}
{E_{\text{ref}}^{-(C_{t,\text{Tr}} - C_{t,\text{Co}})}}
$$
  
  
# A generalized calculation method
  
The `rtpcr` package was developed for the R environment in the major operating systems. The package functions are mainly based on the calculation of efficiency-weighted $\Delta C_t$ $(w\Delta C_t)$ values from target and reference gene Ct (equation 3). $w\Delta C_t$  values are weighted for the amplification efficiencies as described by Ganger et al. (2017) except that log2 is used instead of log10:
  
  
$$
  w\Delta C_t =
  \log_2(E_{\text{target}}) \cdot C_{t,\text{target}}
-
  \log_2(E_{\text{ref}}) \cdot C_{t,\text{ref}}
$$
and for more than one reference gene, the log2E.Ct term of the references, and the whole wDCt  is defined using the geometric mean as follow:

$$
wCt =
\log_{2}\!\left(E_{\text{target}}\right)\cdot Ct_{\text{target}}
-
\left(
\prod_{i=1}^{n}
\left[
\log_{2}\!\left(\text{geoMeanE}\right)\cdot Ct_{\text{ref}_i}
\right]
\right)^{1/n}
$$
where the geometric mean of the efficiency values of the reference genes is:

$$
\text{geoMeanE}
=
\left(
\prod_{i=1}^{n} E_{\text{ref}_i}
\right)^{1/n}
$$


The relative expression of the target gene normalized to that of reference gene(s) within the same sample or condition is called relative expression (RE). From the mean $w\Delta C_t$ values over biological replicates, RE of a target gene can be calculated for each condition according to the equation 

$$
  \text{Relative Expression} = 2^{-\overline{w\Delta C_t}}
$$
  
Relative expression is only calibrated for the reference gene(s) and not for a control condition. However, often one condition is considered as calibrator and the fold change (FC) expression in other conditions is calculated relative to the calibrator. Examples are Treatment versus Control where Control is served as the calibrator, or time 0 versus time 1 (e.g. after 1 hour) and time 2 (e.g. after 2 hours) where time 0 is served as the reference or calibrator level. So, calibrator is the reference level or sample that all others are compared to. The fold change (FC) expression of a target gene for the reference or calibrator level is 1 because it is not changed compared to itself. The fold change expression of a target gene due to the treatment can be calculated as follows: 
  
$$
  \text{Fold Change due to Treatment} =
  2^{-(\overline{w\Delta C_t}_{\text{Tr}} - \overline{w\Delta C_t}_{\text{Co}})}
$$
  
  
Standard error of the FC and RE means is calculated according to <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">Taylor et al. (2019)</a> in `rtpcr` package.
Here, a brief methodology is presented but details about the $w\Delta C_t$  calculations and statistical analysis are available in <a href="https://doi.org/10.1186/s12859-017-1949-5">Ganger et al. (2017)</a>. Importantly, because relative expression values follow a lognormal distribution, a normal distribution is expected for the $w \Delta C_t$ values making it possible to apply t-test or analysis of variance. Following analysis, $w\Delta C_t$ values are statistically compared and standard deviations and confidence interval are calculated, but the transformation $y = 2^{-x}$ is applied in the final step in order to report the results.


# References

Livak, Kenneth J, and Thomas D Schmittgen. 2001. Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR and the Double Delta CT Method. Methods 25 (4). <a href="https://doi.org/10.1006/meth.2001.1262">doi.org/10.1006/meth.2001.1262</a>.

Ganger, MT, Dietz GD, Ewing SJ. 2017. A common base method for analysis of qPCR data and the application of simple blocking in qPCR experiments. BMC bioinformatics 18, 1-11. <a href="https://doi.org/10.1186/s12859-017-1949-5">doi.org/10.1186/s12859-017-1949-5</a>.

Mirzaghaderi G. 2025. rtpcr: a package for statistical analysis and graphical presentation of qPCR data in R. PeerJ 13, e20185. <a href="https://doi.org/10.7717/peerj.20185">doi.org/10.7717/peerj.20185</a>.

Pfaffl MW, Horgan GW, Dempfle L. 2002. Relative expression software tool (REST©) for group-wise comparison and statistical analysis of relative expression results in real-time PCR. Nucleic acids research 30, e36-e36. <a href="https://doi.org/10.1093/nar/30.9.e36">doi.org/10.1093/nar/30.9.e36</a>.

Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. 2019. The ultimate qPCR experiment: producing publication quality, reproducible data the first time. Trends in Biotechnology, 37(7), 761-774. <a href="https://doi.org/10.1016/j.tibtech.2018.12.002">doi.org/10.1016/j.tibtech.2018.12.002</a>.

Yuan, JS, Ann Reed, Feng Chen, and Neal Stewart. 2006. Statistical Analysis of Real-Time PCR Data. BMC Bioinformatics 7 (85). <a href="https://doi.org/10.1186/1471-2105-7-85">doi.org/10.1186/1471-2105-7-85</a>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ANOVA_DDCt.R
\name{ANOVA_DDCt}
\alias{ANOVA_DDCt}
\title{Delta Delta Ct ANOVA analysis with optional model specification}
\usage{
ANOVA_DDCt(
  x,
  numOfFactors,
  numberOfrefGenes,
  specs,
  block,
  calibratorLevel = NULL,
  p.adj = "none",
  analyseAllTarget = TRUE,
  model = NULL,
  set_missing_target_Ct_to_40 = FALSE,
  se.type = c("single.group", "paired.group", "two.group"),
  modelBased_se = TRUE,
  ...
)
}
\arguments{
\item{x}{The input data frame containing experimental design columns, replicates (integer), target gene
E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
columns must be located at the right end of the data frame. See "Input data 
structure" in vignettes for details about data structure.}

\item{numOfFactors}{Integer. Number of experimental factor columns
(excluding \code{rep} and optional \code{block}).}

\item{numberOfrefGenes}{Integer. Number of reference genes.}

\item{specs}{Example: "A", "A|B" or "A|B*C" if A, B and C are name of factor columns in the input data
The first name (here A) is the factor for which the relative expression is analysed.}

\item{block}{Character. Block column name or \code{NULL}. 
When a qPCR experiment is done in multiple qPCR plates, 
variation resulting from the plates may interfere with the actual amount of 
gene expression. One solution is to conduct each plate as a randomized block 
so that at least one replicate of each treatment and control is present 
on a plate. Block effect is usually considered as random and its interaction 
with any main effect is not considered.}

\item{calibratorLevel}{NULL or one of the levels of the first selected factor in specs argument. If NULL the first level of that factor is used as calibrator.
Optional character vector specifying the order of levels for the main factor.
If \code{NULL}, the first observed level is used as the calibrator.
If provided, the first element of the vector is used as the calibrator level.}

\item{p.adj}{Method for p-value adjustment. See \code{\link[stats]{p.adjust}}.}

\item{analyseAllTarget}{Logical or character.
If \code{TRUE} (default), all target genes are analysed.
Alternatively, a character vector specifying the names (names of their Efficiency columns) of target genes
to be analysed.}

\item{model}{Optional model formula. If provided, this overrides the automatic formula (factorial CRD or RCBD 
based on \code{block} and \code{numOfFactors}). The formula uses 
\code{wDCt} as the response variable. 
For mixed models, random effects can be defined using \code{lmer} syntax 
(e.g., \code{"wDCt ~ Treatment + (1 | id)"}). When using \code{model}, 
the \code{block} and \code{numOfFactors} arguments are ignored for model 
specification, but still used for data structure identification.
  
for fixed effects only, the \code{"lm"} (ordinary least squares) is used. 
\code{"lmer"} is used for mixed effects models 
(requires the \code{lmerTest} package). If a custom formula is provided with 
random effects, the function will use \code{lmerTest::lmer()}; otherwise 
it will use \code{stats::lm()}. Note that \code{emmeans} supports both 
model types and will use appropriate degrees of freedom methods (Satterthwaite by default).}

\item{set_missing_target_Ct_to_40}{If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA.}

\item{se.type}{Character string specifying how standard error is calculated. 
One of \code{"paired.group"}, \code{"two.group"}, or \code{"single.group"}. 
\code{"paired.group"} computes SE from paired differences (used when a random 
\code{id} effect is present), \code{"two.group"} uses the unpaired two-group 
t-test standard error against the reference level, and \code{"single.group"} 
computes SE within each level using a one-group t-test.}

\item{modelBased_se}{Logical. If \code{TRUE} (default), standard errors are
calculated from model-based residuals. If \code{FALSE}, standard errors are calculated directly from the observed 
\code{wDCt} values within each treatment group according to the selected \code{se.type}.  
For single factor data, both methods are the same. It is recommended to use \code{modelBased_se = TRUE} (default).}

\item{...}{Additional arguments. Included for backward compatibility with deprecated \code{mainFactor.column}.}
}
\value{
An object containing expression table, lm model, residuals, raw data and ANOVA table for each gene:
\describe{  
\item{ddCt expression table along with per-gene statistical comparison outputs}{\code{object$relativeExpression}}
\item{ANOVA table}{\code{object$perGene$gene_name$ANOVA_table}}
\item{lm ANOVA}{\code{object$perGene$gene_name$lm}}
\item{lm_formula}{\code{object$perGene$gene_name$lm_formula}}
\item{Residuals}{\code{resid(object$perGene$gene_name$lm)}}
}
}
\description{
Apply Delta Delta Ct (ddCt) analysis to each target gene
and performs per-gene statistical analysis.
}
\details{
ddCt analysis of variance (ANOVA) is performed for 
the main factor (as specified using \code{specs} argument) based on a full model factorial 
experiment by default. Analysis of covariance is also possible depending on the user defined model. 
If the interaction between the main factor and the covariate is significant, ANCOVA is not appropriate.

All the functions for relative expression analysis (including \code{TTEST_DDCt()}, 
\code{WILCOX_DDCt()}, \code{ANOVA_DDCt()}, and \code{ANOVA_DCt()}) return the 
relative expression table which include fold change and corresponding 
statistics. The output of \code{ANOVA_DDCt()}, 
and \code{ANOVA_DCt()} also include lm models, residuals, raw data and ANOVA table 
for each gene. 

The expression table returned by \code{TTEST_DDCt()}, 
\code{WILCOX_DDCt()}, and \code{ANOVA_DDCt()} functions 
include these columns: gene (name of target genes), 
contrast (calibrator level and contrasts for which the relative expression is computed), 
ddCt (mean of weighted delta delta Ct values), RE (relative expression or 
fold change = 2^-ddCt),  log2FC (log(2) of relative expression or fold change), 
pvalue, sig (per-gene significance), LCL (95\% lower confidence level), UCL (95\% upper confidence level),
se (standard error of mean calculated from the weighted delta Ct values of each of the main factor levels),
Lower.se.RE (The lower limit error bar for RE which is 2^(log2(RE) - se)), 
Upper.se.RE (The upper limit error bar for RE which is 2^(log2(RE) + se)),
Lower.se.log2FC (The lower limit error bar for log2 RE), and 
Upper.se.log2FC (The upper limit error bar for log2 RE)
}
\examples{
data1 <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
ANOVA_DDCt(data1,
           numOfFactors = 2,
           numberOfrefGenes = 3,
           block = "block",
           specs = "Concentration",
           p.adj = "none")
           
data2 <- read.csv(system.file("extdata", "data_1factor_one_ref.csv", package = "rtpcr"))          
ANOVA_DDCt(data2,
           numOfFactors = 1,
           numberOfrefGenes = 1,
           block = NULL,
           specs = "Condition",
           p.adj = "none",
           se.type = "single.group")
  
# Repeated measure analysis
a <- ANOVA_DDCt(data_repeated_measure_1,
           numOfFactors = 1,
           numberOfrefGenes = 1,
           block = NULL,
           specs = "time",
           p.adj = "none", model = wDCt ~ time + (1 | id))

a$perGene$Target$ANOVA_table


# Repeated measure analysis: split-plot in time
a <- ANOVA_DDCt(data_repeated_measure_2,
           numOfFactors = 2, numberOfrefGenes = 1,
           specs = "time", block = NULL,
           model = wDCt ~ treatment * time + (1 | id))
   
}
\references{
LivakKJ, Schmittgen TD (2001).
Analysis of Relative Gene Expression Data Using Real-Time Quantitative PCR
and the Double Delta CT Method.
\emph{Methods}, 25(4), 402–408.
doi:10.1006/meth.2001.1262

Ganger MT, Dietz GD, and Ewing SJ (2017).
A common base method for analysis of qPCR data and the application of
simple blocking in qPCR experiments.
\emph{BMC Bioinformatics}, 18, 1–11.

Taylor SC, Nadeau K, Abbasi M, Lachance C, Nguyen M, Fenrich, J. (2019). 
The ultimate qPCR experiment: producing publication quality, reproducible 
data the first time. \emph{Trends in Biotechnology}, 37, 761-774. 

Yuan JS, Reed A, Chen F, Stewart N (2006).
Statistical Analysis of Real-Time PCR Data.
\emph{BMC Bioinformatics}, 7, 85.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiplot.R
\name{multiplot}
\alias{multiplot}
\title{Combine multiple ggplot objects into a single layout}
\usage{
multiplot(..., cols = 1)
}
\arguments{
\item{...}{One or more \code{ggplot2} objects.}

\item{cols}{Integer specifying the number of columns in the layout.}
}
\value{
A grid object displaying multiple plots arranged in the specified layout.
}
\description{
The \code{multiplot} function arranges multiple \code{ggplot2} objects
into a single plotting layout with a specified number of columns.
}
\details{
Multiple \code{ggplot2} objects can be provided either as separate
arguments via \code{...}.
The function uses the \code{grid} package to control the layout.
}
\examples{

# Example using output from TTEST_DDCt
data1 <- read.csv(system.file("extdata", "data_ttest18genes.csv", package = "rtpcr"))
out <- TTEST_DDCt(
  data1,
  paired = FALSE,
  var.equal = TRUE,
  numberOfrefGenes = 1)
  
p1 <- plotFactor(out, 
  x_col = "gene",
  y_col = "log2FC", 
  Lower.se_col = "Lower.se.log2FC",
  Upper.se_col = "Upper.se.log2FC", 
  letters_col = "sig")
  
p2 <- plotFactor(out, 
  x_col = "gene",
  y_col = "RE", 
  Lower.se_col = "Lower.se.RE",
  Upper.se_col = "Upper.se.RE", 
  letters_col = "sig")
  
  

# Example using output from ANOVA_DCt
data2 <- read.csv(system.file("extdata", "data_1factor.csv", package = "rtpcr"))
out2 <- ANOVA_DCt(
  data2,
  numOfFactors = 1,
  numberOfrefGenes = 1,
  block = NULL)
  
df <- out2$relativeExpression

p3 <- plotFactor(
  df,
  x_col = "SA",
  y_col = "RE",
  Lower.se_col = "Lower.se.RE",
  Upper.se_col = "Upper.se.RE",
  letters_col = "sig",
  letters_d = 0.1,
  col_width = 0.7,
  err_width = 0.15,
  fill_colors = "skyblue",
  alpha = 1,
  base_size = 14)

# Combine plots into a single layout
multiplot(p1, p2, cols = 2)

multiplot(p1, p3, cols = 2)
}
\author{
Pedro J. (adapted from \url{https://gist.github.com/pedroj/ffe89c67282f82c1813d})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_repeated_measure_1}
\alias{data_repeated_measure_1}
\title{Repeated measure sample data}
\format{
A data frame with 9 observations and 6 variables:
\describe{
  \item{time}{time course levels}
  \item{id}{experimental factor}
  \item{Target}{Amplification efficiency of target gene}
  \item{Ct_Target}{Ct of target gene}
  \item{Ref}{Amplification efficiency of reference gene}
  \item{Ct_Ref}{Ct of reference gene}
}
}
\source{
NA
}
\usage{
data_repeated_measure_1
}
\description{
A repeated measure sample data in which 3 individuals have been analysed.  In the "id" column, a unique number is assigned to each individual, e.g. all the three number 1 indicate one individual.
samples are taken or measurements are scored over different time points (time column) from each individual.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_Yuan2006PMCBioinf}
\alias{data_Yuan2006PMCBioinf}
\title{Sample data in (one factor with one reference gene)}
\format{
A data frame with 24 observations and 6 variables:
\describe{
  \item{condition}{Experimental factor with two levels (control, treatment)}
  \item{rep}{Biological replicates}
  \item{target}{Mean amplification efficiency of target gene}
  \item{Ct_target}{Ct values of target gene. Each is the mean of technical replicates}
  \item{ref}{Mean amplification efficiency of reference gene}
  \item{Ct_ref}{Ct values of reference gene. Each is the mean of technical replicates}
}
}
\source{
Yuan2006PMCBioinf
}
\usage{
data_Yuan2006PMCBioinf
}
\description{
A sample qPCR data set with one experimental factor (condition)
and one reference gene. Each line belongs to a separate individual
(non-repeated measure experiment).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Means_DDCt.R
\name{Means_DDCt}
\alias{Means_DDCt}
\title{Delta Delta Ct pairwise comparisons using a fitted model}
\usage{
Means_DDCt(model, specs, p.adj = "none")
}
\arguments{
\item{model}{A fitted model object (typically an \code{lmer} or \code{lm} object)
created by \code{ANOVA_DCt()}, \code{ANOVA_DDCt()}.}

\item{specs}{A character string or character vector specifying the predictors or
combinations of predictors over which relative expression values are desired.
This argument follows the specification syntax used by
\code{emmeans::emmeans()} (e.g., \code{"Factor"},
\code{"Factor1 | Factor2"}).}

\item{p.adj}{Character string specifying the method for adjusting p-values.
See \code{\link[stats]{p.adjust}} for available options.}
}
\value{
A data frame containing estimated relative expression values, confidence
intervals, p-values, and significance levels derived from the fitted
model.
}
\description{
Performs ddCt expression analysis using a fitted model object produced by
\code{ANOVA_DCt()} or \code{ANOVA_DDCt()}.
}
\details{
The \code{Means_DDCt} function performs pairwise comparisons of relative expression values fo all combinations using
estimated marginal means derived from a fitted model.
For ANOVA models, relative expression values can be obtained for main effects,
interactions, and sliced (simple) effects.
For ANCOVA models returned by the \pkg{rtpcr} package, only simple
effects are supported.

Internally, this function relies on the \pkg{emmeans} package to
compute marginal means and contrasts, which are then back-transformed
to fold change values using the ddCt framework.
}
\examples{

data <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))

# Obtain a fitted model from ANOVA_DDCt
res <- ANOVA_DDCt(
  data,
  numOfFactors = 3,
  numberOfrefGenes = 1,
  specs = "Type",
  block = NULL)

# Relative expression values for Type main effect
lm <- res$perGene$PO$lm
Means_DDCt(lm, specs = "Type")

# Relative expression values for Concentration main effect
Means_DDCt(lm, specs = "Conc")

# Relative expression values for Concentration sliced by Type
Means_DDCt(lm, specs = "Conc | Type")

# Relative expression values for Concentration sliced by Type and SA
Means_DDCt(lm, specs = "Conc | Type * SA")

}
\author{
Ghader Mirzaghaderi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_Heffer2020PlosOne}
\alias{data_Heffer2020PlosOne}
\title{Sample data in (multi-group design with 6 target genes)}
\format{
A data frame with 18 observations and 16 variables:
\describe{
  \item{Treatment}{Experimental treatment group}
  \item{rep}{Biological replicates}

  \item{Fn1, Ct.Fn1}{Efficiency and Ct values of Fn1 gene}
  \item{Col1a1, Ct.Col1a1}{Efficiency and Ct values of Col1a1 gene}
  \item{Acta2, Ct.Acta2}{Efficiency and Ct values of Acta2 gene}
  \item{TgfB, Ct.TgfB}{Efficiency and Ct values of TgfB gene}
  \item{Tnfa, Ct.Tnfa}{Efficiency and Ct values of Tnfa gene}
  \item{Mcp1, Ct.Mcp1}{Efficiency and Ct values of Mcp1 gene}
  \item{Gapdh, Ct.Gapdh}{Efficiency and Ct values of Gapdh reference gene}
}
}
\source{
Not applicable
}
\usage{
data_Heffer2020PlosOne
}
\description{
A sample qPCR data set with four experimental groups
(Uninjected, SNConly, DMSORPE, SNCRPE) and six target genes
normalized to one reference gene (Gapdh).
Each line belongs to a separate individual (non-repeated measure experiment).
}
\details{
This dataset is suitable for t-test or one-way ANOVA based
expression analysis of multiple genes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_2factor3ref}
\alias{data_2factor3ref}
\title{Sample data in (two factor with blocking factor)}
\format{
A data frame with 18 observations and 8 variables:
\describe{
  \item{factor1}{First experimental factor}
  \item{factor2}{Second experimental factor}
  \item{Rep}{Biological replicates}
  \item{PO}{Mean amplification efficiency of PO gene}
  \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
  \item{GAPDH}{Mean amplification efficiency of GAPDH gene}
  \item{Ct_GAPDH}{Ct values of GAPDH gene. Each is the mean of technical replicates}
  \item{ref2}{Mean amplification efficiency of ref2 gene}
  \item{Ct_ref2}{Ct values of ref2 gene. Each is the mean of technical replicates}
  \item{ref3}{Mean amplification efficiency of ref3 gene}
  \item{Ct_ref3}{Ct values of GAPDH gene. Each is the mean of technical replicates}
}
}
\source{
Not applicable
}
\usage{
data_2factor3ref
}
\description{
A sample qPCR data set with blocking factor. Each line belongs to a separate individual (non-repeated measure experiment).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_1factor_one_ref}
\alias{data_1factor_one_ref}
\title{Sample qPCR data (two different conditions)}
\format{
A data frame with 6 observations and 10 variables:
\describe{
  \item{Condition}{Experimental conditions}
  \item{Rep}{Biological replicates}
  \item{C2H2_26}{amplification efficiency of C2H2_26 gene}
  \item{C2H2_26_Ct}{Ct values of C2H2_26 gene. Each is the mean of technical replicates}
  \item{C2H2_01}{Amplification efficiency of C2H2_01 gene}
  \item{C2H2_01_Ct}{Ct values of C2H2_01 gene. Each is the mean of technical replicates}
  \item{C2H2_12}{Amplification efficiency of C2H2_12 gene}
  \item{C2H2_12_Ct}{Ct values of C2H2_12 gene}
  \item{ref}{Amplification efficiency of ref gene}
  \item{ref_Ct}{Ct values of ref gene}
}
}
\source{
University of Kurdistan
}
\usage{
data_1factor_one_ref
}
\description{
Sample qPCR data (two different conditions)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/globalVariables.R
\name{globalVariables}
\alias{globalVariables}
\title{Internal global variables}
\description{
These objects are declared to avoid R CMD check notes about
non-standard evaluation (e.g., in ggplot2).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_repeated_measure_3bLock}
\alias{data_repeated_measure_3bLock}
\title{Sample data in (repeated-measures design with two reference genes)}
\format{
A data frame with 18 observations and 10 variables:
\describe{
  \item{treatment}{Experimental treatment (untreated, treated)}
  \item{time}{Time point of measurement}
  \item{blk}{Blocking factor}
  \item{id}{Subject/individual identifier for repeated measures}

  \item{Target}{Mean amplification efficiency of target gene}
  \item{Ct_Target}{Ct values of target gene}

  \item{Ref1}{Mean amplification efficiency of reference gene 1}
  \item{Ct_Ref1}{Ct values of reference gene 1}

  \item{Ref2}{Mean amplification efficiency of reference gene 2}
  \item{Ct_Ref2}{Ct values of reference gene 2}
}
}
\source{
Not applicable
}
\usage{
data_repeated_measure_3bLock
}
\description{
A sample qPCR data set with repeated measurements over time,
a blocking factor, and two reference genes. The experiment includes
two treatments (untreated and treated) with repeated measures
on the same individuals (id).
}
\details{
This dataset is suitable for repeated-measures mixed-model analysis
and normalization using multiple reference genes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_wDCt.R
\name{compute_wDCt}
\alias{compute_wDCt}
\title{Cleaning data and weighted delta Ct (wDCt) calculation}
\usage{
compute_wDCt(
  x,
  numOfFactors,
  numberOfrefGenes,
  block,
  set_missing_target_Ct_to_40 = FALSE
)
}
\arguments{
\item{x}{A data frame containing experimental design columns, replicates (integer), target gene
E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
columns must be located at the end of the data frame.}

\item{numOfFactors}{Integer. Number of experimental factor columns
(excluding \code{rep} and optional \code{block}).}

\item{numberOfrefGenes}{Integer. Number of reference genes.}

\item{block}{Character or \code{NULL}. Name of the blocking factor column.
When a qPCR experiment is done in multiple qPCR plates, each plate is considered as a random block 
so that at least one replicate of each treatment and control is present 
on a plate.}

\item{set_missing_target_Ct_to_40}{If \code{TRUE}, missing target gene Ct 
values become 40; if \code{FALSE} (default), they become NA.}
}
\value{
The original data frame along with the weighted delta Ct column.
}
\description{
The \code{compute_wDCt} function cleans the data and computes wDCt. This function is 
automatically applied to the expression analysis functions like \code{ANOVA_DDCt}, 
\code{TTEST_DDCt}, etc. So it should not be applied in advance of expression analysis functions.
}
\details{
The \code{compute_wDCt} function computes weighted delta Ct (wDCt) for the input data. 
Missing data can be denoted by NA in the input data frame. 
Values such as '0' and 'undetermined' (for any E and Ct) are
automatically converted to NA. For target genes, NA for E or Ct measurements cause returning NA for 
the corresponding delta Ct for that replicate (row). 
If there are more than one reference gene, NA in the place of the E or the Ct value cause
skipping that gene and remaining references are geometrically averaged.
The \code{compute_wDCt} function is automatically applied to the expression analysis
functions.
}
\examples{
           
data <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))
data
compute_wDCt(x = data,
             numOfFactors = 2,
             numberOfrefGenes = 3,
             block = "block")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efficiency.R
\name{efficiency}
\alias{efficiency}
\title{Amplification efficiency statistics and standard curves}
\usage{
efficiency(df, base_size = 12, legend_position = c(0.2, 0.2), ...)
}
\arguments{
\item{df}{A data frame containing dilution series and corresponding Ct values.
The first column should represent dilution levels, and the remaining
columns should contain Ct values for different genes.}

\item{base_size}{font size}

\item{legend_position}{legend position}

\item{...}{Additional ggplot2 layer arguments}
}
\value{
A list with the following components:
\describe{
  \item{efficiency}{A data frame containing slope, amplification efficiency (E),
  and R\eqn{^2} statistics for each gene.}
  \item{Slope_compare}{A table comparing slopes between genes.}
  \item{plot}{A \code{ggplot2} object showing standard curves for all genes.}
}
}
\description{
The \code{efficiency} function calculates amplification efficiency (E)
and related statistics, including slope and coefficient of determination
(R\eqn{^2}), and generates standard curves for qPCR assays.
}
\details{
Amplification efficiency is estimated from standard curves generated by
regressing Ct values against the logarithm of template dilution.
For each gene, the function reports the slope of the standard curve,
amplification efficiency (E), and R\eqn{^2} as a measure of goodness of fit.
The function also provides graphical visualization of the standard curves.
}
\examples{

# Load example efficiency data
data <- read.csv(system.file("extdata", "data_efficiency1.csv", package = "rtpcr"))

# Calculate amplification efficiency and generate standard curves
efficiency(data)


ef <- read.csv(system.file("extdata", "data_efficiency_Yuan2006PMCBioinf.csv", package = "rtpcr"))
efficiency(ef)

}
\author{
Ghader Mirzaghaderi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_ttest18genes}
\alias{data_ttest18genes}
\title{Sample data in (two-group design with multiple target genes)}
\format{
A data frame with 8 observations and 38 variables:
\describe{
  \item{Condition}{Experimental group (Control, Treatment)}
  \item{Rep}{Biological replicates}

  \item{ANGPT1, Ct.ANGPT1}{Efficiency and Ct values of ANGPT1 gene}
  \item{ANGPT2, Ct.ANGPT2}{Efficiency and Ct values of ANGPT2 gene}
  \item{CCL2, Ct.CCL2}{Efficiency and Ct values of CCL2 gene}
  \item{CCL5, Ct.CCL5}{Efficiency and Ct values of CCL5 gene}
  \item{CSF2, Ct.CSF2}{Efficiency and Ct values of CSF2 gene}
  \item{FGF2, Ct.FGF2}{Efficiency and Ct values of FGF2 gene}
  \item{IL1A, Ct.IL1A}{Efficiency and Ct values of IL1A gene}
  \item{IL1B, Ct.IL1B}{Efficiency and Ct values of IL1B gene}
  \item{IL6, Ct.IL6}{Efficiency and Ct values of IL6 gene}
  \item{IL8, Ct.IL8}{Efficiency and Ct values of IL8 gene}
  \item{PDGFA, Ct.PDGFA}{Efficiency and Ct values of PDGFA gene}
  \item{PDGFB, Ct.PDGFB}{Efficiency and Ct values of PDGFB gene}
  \item{TGFA, Ct.TGFA}{Efficiency and Ct values of TGFA gene}
  \item{TGFB, Ct.TGFB}{Efficiency and Ct values of TGFB gene}
  \item{TNF, Ct.TNF}{Efficiency and Ct values of TNF gene}
  \item{VEGFA, Ct.VEGFA}{Efficiency and Ct values of VEGFA gene}
  \item{VEGFB, Ct.VEGFB}{Efficiency and Ct values of VEGFB gene}
  \item{VEGFC, Ct.VEGFC}{Efficiency and Ct values of VEGFC gene}
  \item{GAPDH, Ct.GAPDH}{Efficiency and Ct values of GAPDH reference gene}
}
}
\source{
Not applicable
}
\usage{
data_ttest18genes
}
\description{
A sample qPCR data set with two experimental groups (Control and Treatment)
and 18 target genes normalized to one reference gene (GAPDH).
Each line belongs to a separate individual (non-repeated measure experiment).
This dataset is suitable for two-sample t-test based expression analysis.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ANOVA_DCt.R
\name{ANOVA_DCt}
\alias{ANOVA_DCt}
\title{Delta Ct ANOVA analysis with optional model specification}
\usage{
ANOVA_DCt(
  x,
  numOfFactors,
  numberOfrefGenes,
  block = NULL,
  alpha = 0.05,
  p.adj = "none",
  analyseAllTarget = TRUE,
  model = NULL,
  modelBased_se = TRUE,
  set_missing_target_Ct_to_40 = FALSE
)
}
\arguments{
\item{x}{The input data frame containing experimental design columns, target gene
E/Ct column pairs, and reference gene E/Ct column pairs. Reference gene
columns must be located at the end of the data frame. See "Input data 
structure" in vignettes for details about data structure.}

\item{numOfFactors}{Integer. Number of experimental factor columns 
(excluding \code{rep} and optional \code{block}).}

\item{numberOfrefGenes}{Integer. Number of reference genes. Each reference gene
must be represented by two columns (E and Ct).}

\item{block}{Character. Block column name or \code{NULL}. When a qPCR experiment 
is done in multiple qPCR plates, variation resulting from the plates may 
interfere with the actual amount of gene expression. One solution is to 
conduct each plate as a randomized block so that at least one replicate of 
each treatment and control is present on a plate. Block effect is usually 
considered as random and its interaction with any main effect is not considered.
Note: This parameter is ignored if \code{model} is provided.}

\item{alpha}{Statistical level for comparisons (default: 0.05).}

\item{p.adj}{Method for p-value adjustment. See \code{\link[stats]{p.adjust}}.}

\item{analyseAllTarget}{Logical or character. If \code{TRUE} (default), all 
detected target genes are analysed. Alternatively, a character vector 
specifying the names (names of their Efficiency columns) of target genes 
to be analysed.}

\item{model}{Optional model formula. If provided, this overrides the automatic formula (CRD or RCBD 
based on \code{block} and \code{numOfFactors}). The formula uses 
\code{wDCt} as the response variable. 
For mixed models, random effects can be defined using \code{lmer} syntax 
(e.g., \code{"wDCt ~ Treatment + (1|Block)"}). When using \code{model}, 
the \code{block} and \code{numOfFactors} arguments are ignored for model 
specification, but still used for data structure identification.}

\item{modelBased_se}{Logical. If \code{TRUE} (default), standard errors are
calculated from model-based residuals. If \code{FALSE}, standard errors are calculated directly from the observed 
\code{wDCt} values within each treatment group according to the selected \code{se.type}.  
For single factor data, both methods are the same. It is recommended to use \code{modelBased_se = TRUE} (default).}

\item{set_missing_target_Ct_to_40}{If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA.}
}
\value{
An object containing expression tables, lm/lmer models, ANOVA tables, 
residuals, and raw data for each gene:
\describe{
\item{\code{relativeExpression}}{dCt expression table for all treatment 
  combinations along with per-gene statistical grouping}
\item{\code{perGene}}{Nested list containing detailed results for each target gene:
  \itemize{
  \item \code{ANOVA_table}: Full factorial ANOVA table
  \item \code{lm}: lm/lmer model for factorial design
  \item \code{Final_data}: Processed data with wDCt values
  \item \code{resid(object$perGene$gene_name$lm)}: Residuals
  }}
}
}
\description{
Performs Delta Ct (dCt) analysis of the data from a factorial experiment 
with support for both fixed- and mixed-effect models. Per-gene statistical 
analysis and grouping is also performed.
}
\details{
The function performs ANOVA analysis on weighted delta Ct (wDCt) values and returns 
variance components along with an expression table containing:
\itemize{
\item \code{gene}: Name of target genes
\item Factor columns: Experimental design factors
\item \code{dCt}: Mean weighted delta Ct for each treatment combination
\item \code{RE}: Relative expression = fold change = 2^-dCt
\item \code{log2FC}: log2 of RE
\item \code{LCL}: 95\% lower confidence level
\item \code{UCL}: 95\% upper confidence level
\item \code{se}: Standard error
\item \code{Lower.se.RE}: Lower limit error bar for RE
\item \code{Upper.se.RE}: Upper limit error bar for RE
\item \code{Lower.se.log2FC}: Lower limit error bar for log2FC
\item \code{Upper.se.log2FC}: Upper limit error bar for log2FC
\item \code{sig}: Per-gene significance grouping letters
}
}
\examples{
# Default usage with fixed effects
result <- ANOVA_DCt(data_2factorBlock3ref, numOfFactors = 2, numberOfrefGenes = 3, 
                    block = "block")

# Mixed model with random block effect
result <- ANOVA_DCt(data_2factorBlock3ref, numOfFactors = 2, numberOfrefGenes = 3,
                          block = "block")

# Custom mixed model formula with nested random effects
result_custom <- ANOVA_DCt(data_repeated_measure_2, numOfFactors = 2, numberOfrefGenes = 1,
                            block = NULL,
                            model = wDCt ~ treatment * time + (1 | id))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_1factor_Two_ref}
\alias{data_1factor_Two_ref}
\title{Sample qPCR data (one target and two reference genes under two different conditions)}
\format{
A data frame with 6 observations and 8 variables:
\describe{
  \item{Condition}{Experimental conditions}
  \item{Rep}{Biological replicates}
  \item{DER5}{amplification efficiency of DER5 gene}
  \item{Ct_DER5}{Ct values of DER5 gene. Each is the mean of technical replicates}
  \item{Actin}{Amplification efficiency of Actin gene}
  \item{Ct_Actin}{Ct values of Actin gene. Each is the mean of technical replicates}
  \item{HPRT}{Amplification efficiency of HPRT gene}
  \item{Ct_HPRT}{Ct values of HPRT gene}
}
}
\source{
Not applicable
}
\usage{
data_1factor_Two_ref
}
\description{
Sample qPCR data (one target and two reference genes under two different conditions)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanTech.R
\name{meanTech}
\alias{meanTech}
\title{Computing the mean of technical replicates}
\usage{
meanTech(
  x,
  groups,
  numOfFactors,
  numberOfrefGenes,
  block,
  set_missing_target_Ct_to_40 = FALSE
)
}
\arguments{
\item{x}{A raw data frame containing technical replicates.}

\item{groups}{An integer specifying the number of columns before the technical replicate column.}

\item{numOfFactors}{Integer. Number of experimental factor columns}

\item{numberOfrefGenes}{Integer. Number of reference genes.}

\item{block}{Character. Block column name or \code{NULL}.}

\item{set_missing_target_Ct_to_40}{If \code{TRUE}, missing target gene Ct values become 40; if \code{FALSE} (default), they become NA.}
}
\value{
A data frame with the mean of technical replicates for each group.
}
\description{
Computes the arithmetic mean of technical replicates for each sample or group.
This is often performed before ANOVA or other statistical analyses to simplify
comparisons between experimental groups.
}
\details{
The \code{meanTech} function calculates the mean of technical replicates
based on one or more grouping columns. This reduces the dataset to a single
representative value per group, facilitating downstream analysis such as
fold change calculation or ANOVA.
}
\examples{

# Example input data frame with technical replicates
data1 <- read.csv(system.file("extdata", "data_withTechRep.csv", package = "rtpcr"))

# Calculate mean of technical replicates using first four columns as groups
meanTech(data1,
         groups = 2,
         numOfFactors = 1,
         numberOfrefGenes = 1,
         block = NULL)

# Another example using different dataset and grouping columns
data2 <- read.csv(system.file("extdata", "data_Lee_etal2020qPCR.csv", package = "rtpcr"))
meanTech(data2, groups = 3,
         numOfFactors = 2,
         numberOfrefGenes = 1,
         block = NULL)
}
\author{
Ghader Mirzaghaderi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_efficiency_Yuan2006PMCBioinf}
\alias{data_efficiency_Yuan2006PMCBioinf}
\title{Sample qPCR data: amplification efficiency 2}
\format{
A data frame with 12 observations and 5 variables:
\describe{
  \item{dilutions}{dilution factor}
  \item{ref_control}{reference gene in control condition}
  \item{ref_treatment}{reference gene in treatment condition}
  \item{target_control}{target gene in control condition}
  \item{target_treatment}{Reference gene in treatment condition}
}
}
\source{
Yuan2006PMCBioinf
}
\usage{
data_efficiency_Yuan2006PMCBioinf
}
\description{
A sample qPCR dataset for demonstrating efficiency calculation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_Lee_etal2020qPCR}
\alias{data_Lee_etal2020qPCR}
\title{Sample data (with technical replicates)}
\format{
A data frame with 72 observations and 8 variables:
\describe{
  \item{factor1}{experimental factor}
  \item{DS}{DS}
  \item{biolRep}{biological replicate}
  \item{techRep}{technical replicates}
  \item{APOE}{Amplification efficiency of APOE gene}
  \item{Ct_APOE}{Ct of APOE gene}
  \item{GAPDH}{Amplification efficiency of GAPDH gene}
  \item{Ct_GAPDH}{Ct of GAPDH gene}
}
}
\source{
Lee et al, (2020) <doi:10.12688/f1000research.23580.2>
}
\usage{
data_Lee_etal2020qPCR
}
\description{
A sample data for calculating biological replicated. Each line belongs to a separate individual (non-repeated measure experiment).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long_to_wide.R
\name{long_to_wide}
\alias{long_to_wide}
\title{Converts a 4-column qPCR long data format to wide format}
\usage{
long_to_wide(x)
}
\arguments{
\item{x}{a 4-column (Condition, gene, Efficiency, Ct) qPCR long data}
}
\value{
A wide qPCR data frame
}
\description{
Converts a 4-column (Condition, gene, Efficiency, Ct) qPCR long data format to wide format
}
\details{
Converts a 4-column (Condition, gene, Efficiency, Ct) qPCR long data format to wide format
}
\examples{

df <- read.table(header = TRUE, text = "
Condition	Gene	E	Ct
control	C2H2-26	1.8	31.26
control	C2H2-26	1.8	31.01
control	C2H2-26	1.8	30.97
treatment	C2H2-26	1.8	32.65
treatment	C2H2-26	1.8	32.03
treatment	C2H2-26	1.8	32.4
control	C2H2-01	1.75	31.06
control	C2H2-01	1.75	30.41
control	C2H2-01	1.75	30.97
treatment	C2H2-01	1.75	28.85
treatment	C2H2-01	1.75	28.93
treatment	C2H2-01	1.75	28.9
control	C2H2-12	2	28.5
control	C2H2-12	2	28.4
control	C2H2-12	2	28.8
treatment	C2H2-12	2	27.9
treatment	C2H2-12	2	28
treatment	C2H2-12	2	27.9
control	ref	1.9	28.87
control	ref	1.9	28.42
control	ref	1.9	28.53
treatment	ref	1.9	28.31
treatment	ref	1.9	29.14
treatment	ref	1.9	28.63")

long_to_wide(df)
}
\author{
Ghader Mirzaghaderi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_efficiency1}
\alias{data_efficiency1}
\title{Sample qPCR data: amplification efficiency 1}
\format{
A data frame with 21 observations and 4 variables:
\describe{
  \item{Dilutions}{Dilution factor}
  \item{C2H2.26}{Target gene 1}
  \item{C2H2.01}{Target gene 2}
  \item{GAPDH}{Reference gene}
}
}
\source{
Not applicable
}
\usage{
data_efficiency1
}
\description{
A sample qPCR dataset for demonstrating efficiency calculation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_1factor_one_ref_Eff}
\alias{data_1factor_one_ref_Eff}
\title{Sample qPCR data (two different conditions)}
\format{
A data frame with 6 observations and 6 variables:
\describe{
  \item{Con}{Experimental conditions}
  \item{r}{Biological replicates}
  \item{target}{Amplification efficiency of target gene}
  \item{target_Ct}{Ct values of target gene}
  \item{Actin}{Amplification efficiency of reference gene}
  \item{Actin_Ct}{Ct values of reference gene}
}
}
\source{
University of Kurdistan
}
\usage{
data_1factor_one_ref_Eff
}
\description{
Sample qPCR data (two different conditions)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_2factorBlock3ref}
\alias{data_2factorBlock3ref}
\title{Sample data in (two factor with blocking factor and 3 reference genes)}
\format{
A data frame with 18 observations and 8 variables:
\describe{
  \item{Type}{First experimental factor}
  \item{Concentration}{Second experimental factor}
  \item{block}{blocking factor}
  \item{Rep}{Biological replicates}
  \item{PO}{Mean amplification efficiency of PO gene}
  \item{Ct_PO}{Ct values of PO gene. Each is the mean of technical replicates}
  \item{NLM}{Mean amplification efficiency of NLM gene}
  \item{Ct_NLM}{Ct values of NLM gene. Each is the mean of technical replicates}
  \item{ref1}{Mean amplification efficiency of ref1 gene}
  \item{Ct_ref1}{Ct values of ref1 gene. Each is the mean of technical replicates}
  \item{ref2}{Mean amplification efficiency of ref2 gene}
  \item{Ct_ref2}{Ct values of ref2 gene. Each is the mean of technical replicates}
  \item{ref3}{Mean amplification efficiency of ref3 gene}
  \item{Ct_ref3}{Ct values of GAPDH gene. Each is the mean of technical replicates}
}
}
\source{
Not applicable
}
\usage{
data_2factorBlock3ref
}
\description{
A sample qPCR data set with blocking factor and 3 reference genes. Each line belongs to a separate individual (non-repeated measure experiment).
}
\keyword{external}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_data.R
\docType{data}
\name{data_withTechRep}
\alias{data_withTechRep}
\title{Sample data (with technical replicates)}
\format{
A data frame with 18 observations and 9 variables:
\describe{
  \item{Condition}{experimental factor}
  \item{biolrep}{biological replicate}
  \item{techrep}{technical replicates}
  \item{target}{Amplification efficiency of target gene}
  \item{Ct_target}{Ct of target gene}
  \item{ref}{Amplification efficiency of reference gene}
  \item{Ct_ref}{Ct of reference gene}
}
}
\source{
Not applicable
}
\usage{
data_withTechRep
}
\description{
A sample data for calculating biological replicated. Each line belongs to a separate individual (non-repeated measure experiment).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WILCOX_DDCt.R
\name{WILCOX_DDCt}
\alias{WILCOX_DDCt}
\title{Delta Delta Ct method wilcox.test analysis}
\usage{
WILCOX_DDCt(
  x,
  numberOfrefGenes,
  Factor.level.order = NULL,
  paired = FALSE,
  p.adj = "none",
  set_missing_target_Ct_to_40 = FALSE
)
}
\arguments{
\item{x}{A data frame containing experimental conditions, biological replicates, and
amplification efficiency and Ct values for target and reference genes.
The number of biological replicates must be equal across genes. If this 
is not true, or there are \code{NA} values use \code{ANODA_DDCt} function 
for independent samples or \code{REPEATED_DDCt} for paired samples.
See the package vignette for details on the required data structure.}

\item{numberOfrefGenes}{Integer specifying the number of reference genes used for normalization.}

\item{Factor.level.order}{Optional character vector specifying the order of factor levels.
If \code{NULL}, the first level of the factor column is used as the calibrator.}

\item{paired}{Logical; if \code{TRUE}, a paired wilcox.test is performed.}

\item{p.adj}{Method for p-value adjustment. One of
\code{"holm"}, \code{"hochberg"}, \code{"hommel"}, \code{"bonferroni"},
\code{"BH"}, \code{"BY"}, \code{"fdr"}, or \code{"none"}. See \code{\link[stats]{p.adjust}}.}

\item{set_missing_target_Ct_to_40}{If \code{TRUE}, missing target gene Ct 
values become 40; if \code{FALSE} (default), they become NA.}
}
\value{
A table containing RE values, log2FC, p-values, significance, 
confidence intervals, standard errors, and lower/upper SE limits.
}
\description{
The \code{WILCOX_DDCt} function performs fold change expression analysis based on
the \eqn{\Delta \Delta C_T} method using wilcox.test. It supports analysis
of one or more target genes evaluated under two experimental conditions
(e.g. control vs treatment).
}
\details{
Relative expression values are computed using reference gene(s) for normalization.
Both paired and unpaired experimental designs are supported.

Paired samples in quantitative PCR refer to measurements collected from the same
individuals under two different conditions (e.g. before vs after treatment),
whereas unpaired samples originate from different individuals in each condition.
Paired designs allow within-individual comparisons and typically reduce
inter-individual variability.

The function returns expression table. The expression table returned by `TTEST_DDCt()`, 
`WILCOX_DDCt()`, `ANOVA_DDCt()`, `ANCOVA_DDCt()`, and `REPEATED_DDCt()` functions 
include these columns: gene (name of target genes), 
contrast (calibrator level and contrasts for which the relative expression is computed), 
RE (relative expression or fold change),  log2FC (log(2) of relative expression or fold change), 
pvalue, sig (per-gene significance), LCL (95\% lower confidence level), UCL (95\% upper confidence level),
se (standard error of mean calculated from the weighted delta Ct values of each of the main factor levels),
Lower.se.RE (The lower limit error bar for RE which is 2^(log2(RE) - se)), 
Upper.se.RE (The upper limit error bar for RE which is 2^(log2(RE) + se)),
Lower.se.log2FC (The lower limit error bar for log2 RE), and 
Upper.se.log2FC (The upper limit error bar for log2 RE)
}
\examples{
# Example data structure
data <- read.csv(system.file("extdata", "data_Yuan2006PMCBioinf.csv", package = "rtpcr"))

# Unpaired t-test
WILCOX_DDCt(
  data,
  paired = FALSE,
  numberOfrefGenes = 1)


# Two reference genes
data2 <- read.csv(system.file("extdata", "data_1factor_Two_ref.csv", package = "rtpcr"))
WILCOX_DDCt(
  data2,
  numberOfrefGenes = 2,
  p.adj = "none")
}
\references{
Yuan, J. S., Reed, A., Chen, F., and Stewart, N. (2006).
Statistical Analysis of Real-Time PCR Data.
\emph{BMC Bioinformatics}, 7, 85.
}
\author{
Ghader Mirzaghaderi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFactor.R
\name{plotFactor}
\alias{plotFactor}
\title{Bar plot of gene(s) expression for 1-, 2-, or 3-factor experiments}
\usage{
plotFactor(
  data,
  x_col,
  y_col,
  Lower.se_col,
  Upper.se_col,
  group_col = NULL,
  facet_col = NULL,
  letters_col = NULL,
  letters_d = 0.2,
  col_width = 0.8,
  err_width = 0.15,
  dodge_width = 0.8,
  fill_colors = NULL,
  color = NA,
  alpha = 1,
  base_size = 12,
  legend_position = "right",
  removeCalibratorCols = FALSE,
  removeCalibratorText = FALSE,
  ...
)
}
\arguments{
\item{data}{Data frame containing expression results}

\item{x_col}{Character. Column name for x-axis}

\item{y_col}{Character. Column name for bar height}

\item{Lower.se_col}{Character. Column name for lower SE}

\item{Upper.se_col}{Character. Column name for upper SE}

\item{group_col}{Character. Column name for grouping bars (optional)}

\item{facet_col}{Character. Column name for faceting (optional)}

\item{letters_col}{Character. Column name for significance letters (optional)}

\item{letters_d}{Numeric. Vertical offset for letters (default \code{0.2})}

\item{col_width}{Numeric. Width of bars (default \code{0.8})}

\item{err_width}{Numeric. Width of error bars (default \code{0.15})}

\item{dodge_width}{Numeric. Width of dodge for grouped bars (default \code{0.8})}

\item{fill_colors}{Optional vector of fill colors to change the default colors}

\item{color}{Optional color for the bar outline}

\item{alpha}{Numeric. Transparency of bars (default \code{1})}

\item{base_size}{Numeric. Base font size for theme (default \code{12})}

\item{legend_position}{Character or numeric vector. Legend position (default \code{right})}

\item{removeCalibratorCols}{= NULL or remove Calibrator Cols}

\item{removeCalibratorText}{= NULL or remove Calibrator text}

\item{...}{Additional ggplot2 layer arguments}
}
\value{
ggplot2 plot object
}
\description{
Creates a bar plot of relative gene expression (fold change) values
from 1-, 2-, or 3-factor experiments, including error bars and statistical
significance annotations.
}
\examples{
data <- read.csv(system.file("extdata", "data_2factorBlock3ref.csv", package = "rtpcr"))

res <- ANOVA_DDCt(x = data,
  numOfFactors = 2,
  numberOfrefGenes = 3,
  block = "block",
  specs = "Concentration",
  p.adj = "none")

df <- res$relativeExpression

p1 <- plotFactor(
  data = df,
  x_col = "contrast",
  y_col = "RE",
  group_col = "gene",
  facet_col = "gene",
  Lower.se_col = "Lower.se.RE",
  Upper.se_col = "Upper.se.RE",
  letters_col = "sig",
  letters_d = 0.2,
  alpha = 1,
  col_width = 0.7,
  dodge_width = 0.7,
  base_size = 14, 
  legend_position = "none")

p1


data2 <- read.csv(system.file("extdata", "data_3factor.csv", package = "rtpcr"))
#Perform analysis first
res <- ANOVA_DCt(
  data2,
  numOfFactors = 3,
  numberOfrefGenes = 1,
  block = NULL)
  
df <- res$relativeExpression
# Generate three-factor bar plot
 p <- plotFactor(
  df,
  x_col = "SA",       
  y_col = "log2FC",       
  group_col = "Type",   
  facet_col = "Conc",    
  Lower.se_col = "Lower.se.log2FC",
  Upper.se_col = "Upper.se.log2FC",
  letters_col = "sig",
  letters_d = 0.3,
  col_width = 0.7, 
  dodge_width = 0.7,
  #fill_colors = c("blue", "brown"),
  color = "black",
  base_size = 14, 
  alpha = 1,
  legend_position = c(0.1, 0.2))
p
  
}
\author{
Ghader Mirzaghaderi
}
